# SPDX-FileCopyrightText: 2025 Julius Deynet <julius.deynet@stud-mail.uni-wuerzburg.de>
#
# SPDX-License-Identifier: GPL-3.0-or-later

library(GenomicRanges)
library(rtracklayer)
library(Biostrings)
library(ggpubr)
library(ggplot2)
library(dplyr)
library(ggpubr)
library(RColorBrewer)

annotated_peaks = as.data.frame(read.delim("./resources/annotated_peaks_hg38_right.tsv"))
lfc_data = readRDS("./resources/processed_featureCounts_introns_TC_data.rds")

merged = merge(lfc_data, annotated_peaks, by="gene_id")
merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()


comparisons <- list(c("FALSE", "TRUE"))

correlation1 = cor(merged$CpG_island, merged$coverage_myc, method = "pearson", use = "complete.obs")
correlation2 = cor(merged$CpG_island, merged$coverage_myc, method = "spearman", use = "complete.obs")

counts <- merged %>%
  group_by(CpG_island) %>%
  summarise(n = paste("n=", n(), sep = ""))

# box plot comparing coverage_myc between CpG islands and non CpG islands
ggplot(merged, aes(x = CpG_island, y = coverage_myc)) +
  geom_boxplot() +
  labs(title = "Coverage of MYC peaks in CpG islands vs non CpG islands",
       x = "CpG island",
       y = "Coverage of MYC peaks") +
  geom_text(
    data = counts,
    aes(x = CpG_island, y = -25, label = n), # Fixed y position
    size = 4
  )+
  theme_minimal()+ 
  stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparisons, hide.ns = FALSE)

plot = ggplot(merged, aes(x = CpG_island, y = coverage_myc)) +
  geom_boxplot(aes(color = CpG_island), outliers = FALSE) +
  labs(#title = "Coverage of Strongest MYC peaks in CpG islands vs non CpG islands",
       x = "CpG island",
       y = "MYC Coverage") +
  scale_color_manual(values = brewer.pal(n = 2, name = "Dark2")) +
  geom_text(
    data = counts,
    aes(x = CpG_island, y = -20, label = n), # Fixed y position
    size = 4
  )+
  annotate("text", x = 1, y = 140, 
           label = paste0("r = ", round(correlation1, 2), ", p = ", round(correlation2, 2)), 
           hjust = 0, vjust = 1.1, size = 5, color = "black") +
  theme_minimal()+ 
  theme(legend.position = "none") +
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", linewidth = 1),) +
  stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparisons, hide.ns = FALSE, label.y = 100, tip.length = 0.01)
plot
ggsave("./box_coverage_myc_cpg_islands.pdf", plot, width = 9, height = 6)

lfc_data = readRDS("./resources/processed_featureCounts_introns_TC_data.rds")
merged = merge(lfc_data, annotated_peaks, by="gene_id")
merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()

correlation = cor(merged$CpG_island, merged$coverage_myc, method = "spearman", use = "complete.obs")

counts <- merged %>%
  group_by(CpG_island) %>%
  summarise(n = paste("n=", n(), sep = ""))

plot = ggplot(merged, aes(x = CpG_island, y = coverage_myc)) +
  geom_boxplot(aes(color = CpG_island), outliers = FALSE) +
  labs(title = "All Peaks",
       x = "CpG island",
       y = "MYC Coverage") +
  scale_color_manual(values = brewer.pal(n = 2, name = "Dark2")) +
  geom_text(
    data = counts,
    aes(x = CpG_island, y = -20, label = n), # Fixed y position
    size = 4
  )+
  annotate("text", x = 1, y = 150, 
           label = paste0("Correlation: ", round(correlation, 2)), 
           hjust = 0, vjust = 1.1, size = 5, color = "blue") +
  theme_minimal()+ 
  theme(legend.position = "none") +
  theme( plot.title = element_text(hjust = 0.5)) + 
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparisons, hide.ns = FALSE, label.y = 110)
plot
ggsave("./box_coverage_myc_cpg_islands_strongest.svg", plot, width = 9, height = 6)



annotated_peaks_top10 = as.data.frame(read.delim("./resources/annotated_peaks_top10.tsv"))

counts <- annotated_peaks_top10 %>%
  group_by(CpG_island) %>%
  summarise(n = paste("n=", n(), sep = ""))

# box plot comparing coverage_myc between CpG islands and non CpG islands
ggplot(annotated_peaks_top10, aes(x = CpG_island, y = coverage_myc)) +
  geom_boxplot() +
  labs(title = "Coverage of Top 10% MYC peaks in CpG islands vs non CpG islands",
       x = "CpG island",
       y = "Coverage of MYC peaks") +
  geom_text(
    data = counts,
    aes(x = CpG_island, y = -20, label = n), # Fixed y position
    size = 4
  )+
  theme_minimal()+ 
  stat_compare_means(method="wilcox.test", label = "p.format", comparisons = comparisons, hide.ns = FALSE)




annotated_peaks$has_TATAAA = annotated_peaks$TATA > 0

counts <- annotated_peaks %>%
  group_by(has_TATAAA) %>%
  summarise(n = paste("n=", n(), sep = ""))




# box plot comparing coverage_myc between CpG islands and non CpG islands
ggplot(annotated_peaks, aes(x = has_TATAAA, y = coverage_myc)) +
  geom_boxplot(outliers = F) +
  labs(title = "Coverage of MYC peaks with vs without TATA",
       x = "TATAAA",
       y = "Coverage of MYC peaks") +
  geom_text(
    data = counts,
    aes(x = has_TATAAA, y = -20, label = n), # Fixed y position
    size = 4
  )+
  theme_minimal()+ 
  stat_compare_means(method="wilcox.test", label = "p.format", comparisons = comparisons, hide.ns = FALSE, label.y = 70)





ebox_motifs <- c("CACGTG", "CACGCG", "CATGCG", "CATGTG", "CACGAG", "CACATG")
group_by_motif = annotated_peaks

group_by_motif$motif = ebox_motifs[apply(group_by_motif[, ebox_motifs], 1, which.max)]
group_by_motif$motif = ifelse(group_by_motif$sum_eboxes > 1, "2+ E-boxes", ifelse(group_by_motif$sum_eboxes == 0, "No E-box", group_by_motif$motif))

group_by_motif$motif = factor(group_by_motif$motif, levels = c("No E-box", ebox_motifs, "2+ E-boxes"))

counts <- group_by_motif  %>%
  group_by(motif, CpG_island) %>%
  summarise(n = n(), .groups = "drop") %>%
  group_by(motif) %>%
  summarise(n = paste("n=", paste(n, collapse = ", "), sep = ""))

correlation <- cor(group_by_motif$CpG_island, group_by_motif$coverage_myc, method = "spearman")


ggplot() +
  geom_boxplot(data = group_by_motif, aes(x = motif, y = coverage_myc, fill = CpG_island)) +
  labs(title = "coverage_MYC per Motif (max 1) for CpG Islands",
       x = "Motif",
       y = "coverage_MYC",
       fill = "CpG Island") +
  
  stat_compare_means(data = group_by_motif,
                     aes(x = motif, y = coverage_myc, group = CpG_island),
                     label = "p.format", 
                     method = "wilcox.test",
                     label.y = c(650, 650, 650, 650, 650, 650)
  )+
  geom_text(
    data = counts,
    aes(x = motif, y = -6, label = n, group = CpG_island),
    size = 4,
    position = position_dodge(width = 0.9)
  ) +
  annotate("text", x = 1, y = 700, 
           label = paste0("Correlation: ", round(correlation, 2)), 
           hjust = 0, vjust = 1.1, size = 5, color = "blue") +
  theme_minimal() +
  theme( plot.title = element_text(hjust = 0.5))

library(colorspace)

# Define original ggplot2 colors
ggplot_colors <- scales::hue_pal()(length(ebox_motifs))

# Generate lighter and darker versions
lighter_colors <- lighten(ggplot_colors, amount = 0.3)  # Increase brightness
darker_colors  <- darken(ggplot_colors, amount = 0.3)   # Decrease brightness

sub_colors <- as.vector(rbind(lighter_colors, darker_colors))

group_by_motif$group_CpG <- interaction(group_by_motif$motif, group_by_motif$CpG_island)

median_baseline = c(median(group_by_motif[group_by_motif$motif == "No E-box" & !group_by_motif$CpG_island, ]$coverage_myc),
                    median(group_by_motif[group_by_motif$motif == "No E-box" & group_by_motif$CpG_island, ]$coverage_myc))

plot = ggplot(group_by_motif, aes(x = motif, y = coverage_myc)) +
  geom_point(data = group_by_motif[group_by_motif$motif == "No E-box", ], color = "white") +
  geom_point(data = group_by_motif[group_by_motif$motif != "No E-box", ], aes(color = group_CpG, size = "size"), alpha = 0.1, position = position_jitterdodge(jitter.width = 0.2)) +
  scale_color_manual(values = c(lighter_colors, "chocolate2", darker_colors, "chocolate4", "black", "black"), guide = "none") +
  scale_size_manual(values = c(0.5, 0.5), guide = "none") +
  new_scale_color() +
  geom_boxplot(data = group_by_motif[group_by_motif$motif == "No E-box", ], aes(color = group_CpG), outliers = FALSE) +
  geom_hline(yintercept = median_baseline, linetype = "dashed", color = c("azure4", "black")) +
  scale_color_manual(values = c("azure4", "black"), labels = c("FALSE", "TRUE"), name="CpG Island", guide = "legend") +
  new_scale_color() +
  geom_boxplot(data = group_by_motif[group_by_motif$motif != "No E-box", ], aes(color = group_CpG), outliers = FALSE) +
  scale_color_manual(values = c(lighter_colors, "chocolate2", darker_colors, "chocolate4"), guide = "none") +
  labs(title = "coverage_MYC per Motif (max 1) for CpG Islands",
       x = "Motif",
       y = "coverage_MYC",
       fill = "CpG Island") +
  
  
  stat_compare_means(data = group_by_motif,
                     aes(x = motif, y = coverage_myc, group = CpG_island),
                     label = "p.signif", 
                     method = "wilcox.test",
                     label.y = rep(125, 8)
  )+
  geom_text(
    data = counts,
    aes(x = motif, y = -15, label = n),
    size = 4,
    position = position_dodge(width = 0.9),
    #angle = 20
  ) +
  annotate("text", x = 1, y = 140, 
           label = paste0("Correlation: ", round(correlation, 2)), 
           hjust = 0, vjust = 1.1, size = 5, color = "blue") +
  theme_minimal() +
  theme( plot.title = element_text(hjust = 0.5)) +
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  scale_y_continuous(breaks = seq(0, 300, 20), limits = c(-20, 140))
plot
ggsave("./box_coverage_myc_cpg_islands_per_motif.svg", plot, width = 9, height = 6)



lfc_data = readRDS("./resources/processed_featureCounts_introns_TC_data.rds")

merged = merge(lfc_data, group_by_motif, by="gene_id", all.x = TRUE)

# only keep the peak with the highest coverage per gene
merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()

merged_without_NA = merged[!is.na(merged$coverage_myc),]

counts <- merged_without_NA  %>%
  group_by(motif, CpG_island) %>%
  summarise(n = n(), .groups = "drop") %>%
  group_by(motif) %>%
  summarise(n = paste("n=", paste(n, collapse = ", "), sep = ""))

correlation <- cor(merged_without_NA$CpG_island, merged_without_NA$coverage_myc, method = "spearman", use = "complete.obs")

median_baseline = c(median(merged_without_NA[merged_without_NA$motif == "No E-box" & !merged_without_NA$CpG_island, ]$coverage_myc),
                    median(merged_without_NA[merged_without_NA$motif == "No E-box" & merged_without_NA$CpG_island, ]$coverage_myc))

plot = ggplot(merged_without_NA, aes(x = motif, y = coverage_myc)) +
  geom_boxplot(aes(group = group_CpG), color = "white", outliers = FALSE) +
  geom_point(data = merged_without_NA[merged_without_NA$motif != "No E-box", ], aes(color = group_CpG, size = CpG_island), alpha = 0.1, position = position_jitterdodge(jitter.width = 0.2)) +
  scale_color_manual(values = c(lighter_colors, "chocolate2", darker_colors, "chocolate4", "black", "black"), guide = "none") +
  scale_size_manual(values = c(0.5, 0.5), guide = "none") +
  new_scale_color() +
  geom_boxplot(data = merged_without_NA[merged_without_NA$motif == "No E-box", ], aes(color = group_CpG), outliers = FALSE) +
  geom_hline(yintercept = median_baseline, linetype = "dashed", color = c("azure4", "black")) +
  scale_color_manual(values = c("azure4", "black"), labels = c("FALSE", "TRUE"), name="CpG Island", guide = "none") +
  new_scale_color() +
  geom_boxplot(data = merged_without_NA[merged_without_NA$motif != "No E-box", ], aes(color = group_CpG), outliers = FALSE) +
  scale_color_manual(values = c(lighter_colors, "chocolate2", darker_colors, "chocolate4"), guide = "none") +
  labs(#title = "coverage_MYC per Motif (max 1) for CpG Islands for strongest peaks",
       x = "Motif",
       y = "MYC Coverage",
       fill = "CpG Island") +
  
  
  stat_compare_means(data = merged_without_NA,
                     aes(x = motif, y = coverage_myc, group = CpG_island),
                     label = "p.signif", 
                     method = "wilcox.test",
                     label.y = rep(160, 8)
  )+
  geom_text(
    data = counts,
    aes(x = motif, y = -6, label = n),
    size = 4,
    position = position_dodge(width = 0.9)
  ) +
  theme_minimal() +
  theme( plot.title = element_text(hjust = 0.5)) +
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  scale_y_continuous(breaks = seq(0, 300, 20))+ 
  coord_cartesian(ylim = c(-10, 160))
plot
ggsave("./box_coverage_myc_cpg_islands_per_motif_strongest.pdf", plot, width = 9, height = 6)

merged$motif = ifelse(is.na(merged$motif), "No Assigned Peak", as.character(merged$motif))
merged$motif = factor(merged$motif, levels = c("No Assigned Peak", "No E-box", ebox_motifs, "2+ E-boxes"))

counts <- merged %>%
  group_by(motif, CpG_island) %>%
  summarise(n = n(), .groups = "drop") %>%
  group_by(motif) %>%
  summarise(n = paste("n=", paste(n, collapse = ", "), sep = ""))


correlation <- cor(merged$CpG_island, merged$log2FoldChange, method = "spearman", use = "complete.obs")


median_baseline = c(median(merged[merged$motif == "No E-box" & !merged$CpG_island, ]$log2FoldChange),
                    median(merged[merged$motif == "No E-box" & merged$CpG_island, ]$log2FoldChange))


plot = ggplot(merged, aes(x = motif, y = log2FoldChange)) +
  geom_boxplot(aes(group = group_CpG), color = "white", outliers = FALSE) +
  geom_point(data = merged[merged$motif %in% ebox_motifs | merged$motif == "2+ E-boxes", ], aes(color = group_CpG, size = CpG_island), alpha = 0.1, position = position_jitterdodge(jitter.width = 0.2)) +
  scale_color_manual(values = c(lighter_colors, "chocolate2", darker_colors, "chocolate4", "black", "black"), guide = "none") +
  scale_size_manual(values = c(0.5, 0.5), guide = "none") +
  new_scale_color() +
  geom_boxplot(data = merged[merged$motif == "No Assigned Peak", ], color = "grey", outliers = FALSE) +
  geom_boxplot(data = merged[merged$motif == "No E-box", ], aes(color = group_CpG), outliers = FALSE) +
  geom_hline(yintercept = median_baseline, linetype = "dashed", color = c("azure4", "black")) +
  geom_boxplot(data = merged[merged$motif == "No Assigned Peak", ], color = "grey", outliers = FALSE) +
  scale_color_manual(values = c("azure4", "black"), labels = c("FALSE", "TRUE"), name="CpG Island", guide = "none") +
  new_scale_color() +
  geom_boxplot(data = merged[merged$motif %in% ebox_motifs | merged$motif == "2+ E-boxes", ], aes(color = group_CpG), outliers = FALSE) +
  scale_color_manual(values = c(lighter_colors, "chocolate2", darker_colors, "chocolate4"), guide = "none") +
  labs(#title = "LFC of intronic TC Talk-seq data per Motif (max 1) of strongest peak for CpG Islands",
       x = "Motif",
       y = "-log2(Fold Change MYC vs. MYC Depletion)",
       fill = "CpG Island") +
  
  stat_compare_means(data = merged,
                     aes(x = motif, y = log2FoldChange, group = CpG_island),
                     label = "p.signif", 
                     method = "wilcox.test",
                     label.y = rep(2.1, 8)
  )+
  geom_text(
    data = counts,
    aes(x = motif, y = -1.5, label = n),
    size = 4,
    position = position_dodge(width = 0.9)
  ) +
  theme_minimal() +
  theme( plot.title = element_text(hjust = 0.5)) +
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  coord_cartesian(ylim = c(-1.5, 2.1))
plot
ggsave("./box_lfc_myc_cpg_islands_per_motif_strongest.pdf", plot, width = 9, height = 6)




data = annotated_peaks

calc_percentiles = function(title) {
  
  percentage_CpGIsland = function(percentile) {
    
    percentile = percentile / 100
    
    top_n = quantile(data$peak_score, percentile)
    
    data = data[data$peak_score >= top_n, ]
    
    sum(data$CpG_island) / nrow(data)
  }
  
  percentage_TATA = function(percentile) {
    
    percentile = percentile / 100
    
    top_n = quantile(data$peak_score, percentile)
    
    data = data[data$peak_score >= top_n, ]
    
    nrow(subset(data, TATA > 0)) / nrow(data)
  }
  
  percentage_CpGIsland_df = data.frame(percentile = 0:99, percentage_CpGIsland = sapply(0:99, percentage_CpGIsland))
  percentage_TATA_CpGIsland_df = data.frame(percentile = 0:99, percentage_TATA = sapply(0:99, percentage_TATA))
  
  # arg max
  #df_coverage_myc[which.max(df_coverage_myc$percentage_coverage_myc), ]
  
  combined = merge(percentage_CpGIsland_df, percentage_TATA_CpGIsland_df, by = "percentile")
  #combined_lfc = merge(combined_lfc, df_lfc_coverage_myc_near, by = "percentile")
  
  ggplot(combined, aes(x = percentile)) +
    geom_line(aes(y = percentage_CpGIsland, color = "CpG Island")) +
    geom_line(aes(y = percentage_TATA, color = "TATA")) +
    labs(x = "Top n-th percentile", 
         y = "Percentage of peaks", 
         color = "") +
    scale_color_manual(values = c( "skyblue", "brown3")) +
    #ggtitle(title) +
    theme_minimal() +
    theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
    scale_x_continuous(breaks = seq(0, 100, 10)) +
    scale_y_continuous(labels = scales::percent)
  
}

plot = calc_percentiles("Percentage of peaks for top n-th percentile")
plot
ggsave("./percentile_cpg_tata.pdf", plot, width = 9, height = 6)



filtered_peaks = annotated_peaks[annotated_peaks$sum_eboxes == 0,]

correlation = cor(filtered_peaks$CpG_island, filtered_peaks$coverage_myc, method = "spearman", use = "complete.obs")

counts <- filtered_peaks %>%
  group_by(CpG_island) %>%
  summarise(n = paste("n=", n(), sep = ""))
comparison = list(c("FALSE", "TRUE"))
ggplot(filtered_peaks, aes(x = CpG_island, y = coverage_myc)) +
  geom_boxplot() +
  labs(title = "Coverage of MYC peaks without E-boxes for with vs without CpG islands",
       x = "CpG island",
       y = "Coverage of MYC peaks") +
  geom_text(
    data = counts,
    aes(x = CpG_island, y = -20, label = n), # Fixed y position
    size = 4
  )+
  annotate("text", x = 1, y = 500, 
           label = paste0("Correlation: ", round(correlation, 2)), 
           hjust = 0, vjust = 0, size = 5, color = "blue") +
  theme_minimal()+ 
  stat_compare_means(method="wilcox.test", label = "p.format", comparisons = comparison, hide.ns = FALSE, label.y = c(450))


lfc_data = readRDS("./resources/processed_featureCounts_introns_TC_data.rds")

merged = merge(lfc_data, annotated_peaks, by="gene_id", all.x = TRUE)
merged = merged[merged$sum_eboxes == 0 | is.na(merged$sum_eboxes),]
merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()

correlation <- cor(merged$CpG_island, merged$log2FoldChange, method = "spearman", use = "complete.obs")

merged$CpG_island = ifelse(is.na(merged$CpG_island), "No Assigned Peak", as.character(merged$CpG_island))
merged$CpG_island = factor(merged$CpG_island, levels = c("No Assigned Peak", "FALSE", "TRUE"))

comparisons = list(c("No Assigned Peak", "FALSE"), c("FALSE", "TRUE"))

counts <- merged %>%
  group_by(CpG_island) %>%
  summarise(n = paste("n=", n(), sep = ""))

ggplot(merged, aes(x = CpG_island, y = log2FoldChange)) +
  geom_boxplot() +
  labs(title = "LFC of intronic TC Talk-seq data for MYC peaks without E-boxes for CpG islands vs non CpG islands",
       x = "CpG island",
       y = "-log2(Fold Change MYC vs. MYC Depletion)") +
  geom_text(
    data = counts,
    aes(x = CpG_island, y = -5, label = n), # Fixed y position
    size = 4
  )+
  annotate("text", x = 1, y = 6.5, 
           label = paste0("Correlation: ", round(correlation, 2)), 
           hjust = 0, vjust = 0, size = 5, color = "blue") +
  theme_minimal()+ 
  stat_compare_means(method="wilcox.test", label = "p.format", comparisons = comparisons, hide.ns = FALSE, label.y = c(5, 5))




lfc_data = readRDS("./resources/processed_featureCounts_introns_TC_data.rds")
lfc_sc_data = readRDS("./resources/processed_grandR_single_cell_new_data.rds")


annotated_peaks_top10 = as.data.frame(read.delim("./resources/annotated_peaks_hg38_right_top10.tsv"))

create_cpg_island_figures = function(lfc_data, annotated_peaks, title, fill_color) {

merged = merge(lfc_data, annotated_peaks, by="gene_id", all.x = TRUE)
merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()

correlation1 <- cor(merged$CpG_island, merged$log2FoldChange, method = "pearson", use = "complete.obs")
correlation2 <- cor(merged$CpG_island, merged$log2FoldChange, method = "spearman", use = "complete.obs")

merged$CpG_island = ifelse(is.na(merged$CpG_island), "No Assigned Peak", as.character(merged$CpG_island))
merged$CpG_island = factor(merged$CpG_island, levels = c("No Assigned Peak", "FALSE", "TRUE"))

comparisons = list(c("No Assigned Peak", "FALSE"), c("FALSE", "TRUE"))

counts <- merged %>%
  group_by(CpG_island) %>%
  summarise(n = paste("n=", n(), sep = ""))

y_min = min(merged$log2FoldChange, na.rm = TRUE)
y_max = max(merged$log2FoldChange, na.rm = TRUE)
p_val_pos = rep(y_max + 0.2, 2)

# box plot comparing coverage_myc between CpG islands and non CpG islands
# ggplot(merged, aes(x = CpG_island, y = log2FoldChange)) +
#   geom_boxplot(data = merged[merged$CpG_island == "No Assigned Peak", ], color = "grey") +
#   geom_boxplot(data = merged[merged$CpG_island != "No Assigned Peak", ], aes(color = CpG_island)) +
#   labs(title = paste("LFC of", lfc_data_name, "data for CpG islands vs non CpG islands from strongest of", annotated_peaks_name, "peaks"),
#        x = "CpG island",
#        y = "-log2(Fold Change MYC vs. MYC Depletion)") +
#   geom_text(
#     data = counts,
#     aes(x = CpG_island, y = y_min - 0.5, label = n), # Fixed y position
#     size = 4
#   )+
#   annotate("text", x = 1, y = y_max + 1, 
#            label = paste0("Correlation: ", round(correlation, 2)), 
#            hjust = 0, vjust = 0, size = 5, color = "blue") +
#   theme_minimal()+ 
#   theme(legend.position = "none")+
#   theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
#   stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparisons, hide.ns = FALSE, label.y = p_val_pos) + 
#   scale_y_continuous(breaks = seq(as.integer(y_min), as.integer(y_max), 1))
y_min = -1
y_max = 2
p_val_pos = rep(y_max + 0.2, 2)
ggplot(merged, aes(x = CpG_island, y = log2FoldChange)) +
  geom_boxplot(data = merged[merged$CpG_island == "No Assigned Peak", ],color = "grey", outliers = FALSE) +
  geom_boxplot(data = merged[merged$CpG_island != "No Assigned Peak", ], aes(color = CpG_island), outliers = FALSE) +
  labs(title = title,
       x = "CpG island",
       y = "-log2(Fold Change MYC vs. MYC Depletion)") +
  scale_color_manual(values = brewer.pal(n = 2, name = "Dark2")) +
  geom_text(
    data = counts,
    aes(x = CpG_island, y = y_min - 0.5, label = n), # Fixed y position
    size = 4
  )+
  annotate("text", x = 1, y = y_max + 1, 
           label = paste0("r = ", round(correlation1, 2), ", p = ", round(correlation2, 2)), 
           hjust = 0, vjust = 0, size = 5, color = "blue") +
  theme_minimal()+ 
  theme( plot.title = element_text(hjust = 0.5)) + 
  theme(legend.position = "none")+
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparisons, hide.ns = FALSE, label.y = p_val_pos) + 
  scale_y_continuous(breaks = seq(as.integer(y_min), as.integer(y_max), 1))
}

plot = create_cpg_island_figures(lfc_data, annotated_peaks, "All Peaks", alpha("lightblue", 0.7))
ggsave("./box_lfc_myc_cpg_islands_intronic_tc_talk.pdf", plot, width = 9, height = 6)
plot = create_cpg_island_figures(lfc_sc_data, annotated_peaks, "All Peaks", alpha("lightblue", 0.7))
ggsave("./box_lfc_myc_cpg_islands_new_slam.svg", plot, width = 9, height = 6)
plot = create_cpg_island_figures(lfc_data, annotated_peaks_top10, "Top 10% of Peaks", alpha("deepskyblue3", 0.5))
ggsave("./box_lfc_myc_cpg_islands_intronic_tc_talk_top10.pdf", plot, width = 9, height = 6)
plot = create_cpg_island_figures(lfc_sc_data, annotated_peaks_top10, "Top 10% of Peaks", alpha("deepskyblue3", 0.5))
ggsave("./box_lfc_myc_cpg_islands_new_slam_top10.svg", plot, width = 9, height = 6)



annotated_peaks$TATA_existence = annotated_peaks$TATA > 0

correlation = cor(annotated_peaks$TATA_existence, annotated_peaks$coverage_myc, method = "spearman", use = "complete.obs")

counts <- annotated_peaks %>%
  group_by(TATA_existence) %>%
  summarise(n = paste("n=", n(), sep = ""))

comparison = list(c("FALSE", "TRUE"))

plot = ggplot(annotated_peaks, aes(x = TATA_existence, y = coverage_myc)) +
  geom_boxplot(outliers = FALSE) +
  labs(title = "Coverage of MYC peaks with vs without TATA",
       x = "TATA",
       y = "Coverage of MYC peaks") +
  geom_text(
    data = counts,
    aes(x = TATA_existence, y = -20, label = n), # Fixed y position
    size = 4
  )+
  annotate("text", x = 1, y = 110, 
           label = paste0("Correlation: ", round(correlation, 2)), 
           hjust = 0, vjust = 0, size = 5, color = "blue") +
  theme_minimal()+ 
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparison, hide.ns = FALSE, label.y = c(80))
plot
ggsave("./box_coverage_existence_tata.svg", plot, width = 9, height = 6)

merged = merge(lfc_data, annotated_peaks, by="gene_id")
merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()

correlation = cor(merged$TATA_existence, merged$coverage_myc, method = "spearman", use = "complete.obs")

counts <- merged %>%
  group_by(TATA_existence) %>%
  summarise(n = paste("n=", n(), sep = ""))

plot = ggplot(merged, aes(x = TATA_existence, y = coverage_myc)) +
  geom_boxplot(outliers = FALSE) +
  labs(title = "Coverage of MYC peaks with vs without TATA",
       x = "TATA",
       y = "Coverage of MYC peaks") +
  geom_text(
    data = counts,
    aes(x = TATA_existence, y = -20, label = n), # Fixed y position
    size = 4
  )+
  annotate("text", x = 1, y = 135, 
           label = paste0("Correlation: ", round(correlation, 2)), 
           hjust = 0, vjust = 0, size = 5, color = "blue") +
  theme_minimal()+ 
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparison, hide.ns = FALSE, label.y = c(105))
plot
ggsave("./box_coverage_existence_tata_strongest.svg", plot, width = 9, height = 6)


filtered_peaks = annotated_peaks[!annotated_peaks$CpG_island,]

correlation = cor(filtered_peaks$TATA_existence, filtered_peaks$coverage_myc, method = "spearman", use = "complete.obs")

counts <- filtered_peaks %>%
  group_by(TATA_existence) %>%
  summarise(n = paste("n=", n(), sep = ""))

comparison = list(c("FALSE", "TRUE"))

ggplot(filtered_peaks, aes(x = TATA_existence, y = coverage_myc)) +
  geom_boxplot(outliers = FALSE) +
  labs(title = "Coverage of MYC peaks without CpG Islands for with vs without TATA",
       x = "TATA",
       y = "Coverage of MYC peaks") +
  geom_text(
    data = counts,
    aes(x = TATA_existence, y = -20, label = n), # Fixed y position
    size = 4
  )+
  annotate("text", x = 1, y = 50, 
           label = paste0("Correlation: ", round(correlation, 2)), 
           hjust = 0, vjust = 0, size = 5, color = "blue") +
  theme_minimal()+ 
  stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparison, hide.ns = FALSE, label.y = 30)



lfc_data = readRDS("./resources/processed_featureCounts_introns_TC_data.rds")
lfc_sc_data = readRDS("./resources/processed_grandR_single_cell_new_data.rds")


create_tata_plots_top_peak = function(lfc_data, annotated_peaks, lfc_data_name, annotated_peaks_name, fill_color) {
  
  merged = merge(lfc_data, annotated_peaks, by="gene_id", all.x = TRUE)
  merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()
  
  
  unique_genes = merged[!duplicated(merged$gene_id), ]
  
  unique_genes$tata_exists = ifelse(!is.na(unique_genes$TATA), ifelse(unique_genes$TATA > 0, "1+ E-boxes", "No E-boxes"), "No Assigned Peak")
  unique_genes$tata_exists = factor(unique_genes$tata_exists, levels = c("No Assigned Peak", "No E-boxes", "1+ E-boxes"))
  
  counts_TATA = sort(unique(unique_genes$TATA))
  unique_genes$TATA = ifelse(is.na(unique_genes$TATA), "No Assigned Peak", unique_genes$TATA)
  unique_genes$TATA = factor(unique_genes$TATA, levels = c("No Assigned Peak", as.character(counts_TATA)))
  
  d = unique_genes
  
  create_plot = function(category, label) {
    
    category_levels <- sort(unique(d[[category]]))
    
    # ensure category has string type
    if (!is.factor(d[[category]])) {
      d[[category]] <- factor(d[[category]], levels = as.character(category_levels))
    }
    
    counts <- d %>%
      group_by(!!sym(category)) %>%  # Use !!sym for dynamic column names
      summarise(n = paste("n=", n(), sep = ""))
    
    sequential_comparisons <- list()
    
    for (i in 1:(length(category_levels) - 1)) {
      sequential_comparisons[[i]] <- c(as.character(category_levels[i]), as.character(category_levels[i + 1]))
    }
    
    correlation <- cor(as.numeric(d[[category]]), d$log2FoldChange, method = "spearman", use = "pairwise.complete.obs")
    
    y_max = max(d$log2FoldChange, na.rm = TRUE)
    p_val_pos = rep(y_max + 0.3 , length(sequential_comparisons))
    
    y_min = min(d$log2FoldChange, na.rm = TRUE)
    
    # Generate the boxplot
    ggplot(d, aes(x = !!sym(category), group = !!sym(category), y = log2FoldChange)) +  # Use !!sym to dynamically reference columns
      geom_boxplot(fill = fill_color) +
      labs(title = paste("LFC of", lfc_data_name, "data for", label, "from strongest of", annotated_peaks_name, "peaks" ),
           x = label,
           y = "-log2(Fold Change MYC vs. MYC Depletion)") +
      
      # Add counts as text labels
      geom_text(
        data = counts,  # Use counts summarised above
        aes(x = !!sym(category), y = y_min - 0.5, label = n),  # Place text above the boxes
        vjust = -1, size = 4
      ) +
      annotate("text", x = 1, y = y_max + 1.5, 
               label = paste0("Correlation: ", round(correlation, 2)), 
               hjust = 0, vjust = 0, size = 5, color = "blue") +
      
      # Add p-values using wilcox.test
      # throws errors in many cases
      stat_compare_means(method = "wilcox.test", label = "p.signif", comparisons = sequential_comparisons, hide.ns = FALSE, label.y = p_val_pos) +
      theme_minimal()+
      theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", linewidth = 1),)
  }
  
  list(create_plot("tata_exists", "Existence of TATA-boxes"), create_plot("TATA", "Number of TATA-boxes"))
}

plots = create_tata_plots_top_peak(lfc_data, annotated_peaks, "intronic TC Talk-seq", "all", alpha("lightblue", 0.7))
ggsave("./box_existence_tata_strongest_peak_intronic_TC_talk.svg", plots[[1]], width = 9, height = 6)
ggsave("./box_number_tata_strongest_peak_intronic_tc_talk.svg", plots[[2]], width = 9, height = 6)
plots = create_tata_plots_top_peak(lfc_data, annotated_peaks_top10, "intronic TC Talk-seq", "top 10% of", alpha("deepskyblue3", 0.5))
ggsave("./box_existence_tata_strongest_peak_intronic_TC_talk_top10.svg", plots[[1]], width = 9, height = 6)
ggsave("./box_number_tata_strongest_peak_intronic_tc_talk_top10.svg", plots[[2]], width = 9, height = 6)
plots = create_tata_plots_top_peak(lfc_sc_data, annotated_peaks, "new scSLAM-seq", "all", alpha("lightblue", 0.7))
ggsave("./box_existence_tata_strongest_peak_new_slam.svg", plots[[1]], width = 9, height = 6)
ggsave("./box_number_tata_strongest_peak_new_slam.svg", plots[[2]], width = 9, height = 6)
plots = create_tata_plots_top_peak(lfc_sc_data, annotated_peaks_top10, "new scSLAM-seq", "top 10% of", alpha("deepskyblue3", 0.5))
ggsave("./box_existence_tata_strongest_peak_new_slam_top10.svg", plots[[1]], width = 9, height = 6)
ggsave("./box_number_tata_strongest_peak_new_slam_top10.svg", plots[[2]], width = 9, height = 6)


merged = merge(lfc_data, annotated_peaks, by="gene_id", all.x = TRUE)
merged = merged[!merged$CpG_island | is.na(merged$CpG_island),]
merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()

correlation <- cor(merged$TATA_existence, merged$log2FoldChange, method = "spearman", use = "complete.obs")

merged$TATA_existence = ifelse(is.na(merged$TATA_existence), "No Assigned Peak", as.character(merged$TATA_existence))
merged$TATA_existence = factor(merged$TATA_existence, levels = c("No Assigned Peak", "FALSE", "TRUE"))

comparisons = list(c("No Assigned Peak", "FALSE"), c("FALSE", "TRUE"), c("No Assigned Peak", "TRUE"))

counts <- merged %>%
  group_by(TATA_existence) %>%
  summarise(n = paste("n=", n(), sep = ""))

ggplot(merged, aes(x = TATA_existence, y = log2FoldChange)) +
  geom_boxplot() +
  labs(title = "LFC of intronic TC Talk-seq data for MYC peaks without CpG Islands for with vs without TATA",
       x = "TATA",
       y = "-log2(Fold Change MYC vs. MYC Depletion)") +
  geom_text(
    data = counts,
    aes(x = TATA_existence, y = -5, label = n), # Fixed y position
    size = 4
  )+
  annotate("text", x = 1, y = 6.5, 
           label = paste0("Correlation: ", round(correlation, 2)), 
           hjust = 0, vjust = 0, size = 5, color = "blue") +
  theme_minimal()+ 
  stat_compare_means(method="wilcox.test", label = "p.format", comparisons = comparisons, hide.ns = FALSE, label.y = c(5, 5))

ggplot(merged, aes(x = TATA_existence, y = log2FoldChange)) +
  geom_boxplot(outliers = FALSE) +
  labs(title = "LFC of intronic TC Talk-seq data for MYC peaks without CpG Islands for with vs without TATA",
       x = "TATA",
       y = "-log2(Fold Change MYC vs. MYC Depletion)") +
  geom_text(
    data = counts,
    aes(x = TATA_existence, y = -2, label = n), # Fixed y position
    size = 4
  )+
  annotate("text", x = 1, y = 2.5, 
           label = paste0("Correlation: ", round(correlation, 2)), 
           hjust = 0, vjust = 0, size = 5, color = "blue") +
  theme_minimal()+ 
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparisons, hide.ns = FALSE, label.y = c(1.8, 1.8, 2))

merged = merge(lfc_data, annotated_peaks, by="gene_id")
merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()



filtered_peaks = merged

filtered_peaks$category = ifelse(filtered_peaks$CpG_island & filtered_peaks$TATA > 0, "CpG Island & TATA", ifelse(filtered_peaks$CpG_island, "CpG Island", ifelse(filtered_peaks$TATA > 0, "TATA", "No CpG Island & No TATA")))
filtered_peaks$category = factor(filtered_peaks$category, levels = c("No CpG Island & No TATA", "TATA", "CpG Island", "CpG Island & TATA"))
comparison = list(c("No CpG Island & No TATA", "TATA"),  c("TATA", "CpG Island"), c("No CpG Island & No TATA", "CpG Island"), c("CpG Island", "CpG Island & TATA"))

counts <- filtered_peaks %>%
  group_by(category) %>%
  summarise(n = paste("n=", n(), sep = ""))

plot = ggplot(filtered_peaks, aes(x = category, y = coverage_myc)) +
  geom_boxplot(aes(color = "white"),outliers = FALSE) +
  geom_jitter(data=filtered_peaks[filtered_peaks$category != "No Assigned Peak",], aes(color = category), width = 0.2, alpha = 0.1) +
  geom_boxplot(aes(color = category),outliers = FALSE) +
  scale_color_manual(values = c("black", "brown3", "skyblue", "violet", "white")) +
  labs(#title = "Coverage_MYC of Strongest Peaks for Combinations of TATA-boxes and CpG Islands",
       x = NULL,
       y = "MYC Coverage") +
  geom_text(
    data = counts,
    aes(x = category, y = -10, label = n), # Fixed y position
    size = 4
  )+
  guides(color = "none") +
  coord_cartesian(ylim = c(-10, 160))+
  theme_minimal()+ 
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparison, hide.ns = FALSE, label.y = c(110, 110, 130, 110))
plot
ggsave("./box_coverage_combinations_CpG_TATA_strongest.pdf", plot, width = 9, height = 6)

filtered_peaks = annotated_peaks

filtered_peaks$category = ifelse(filtered_peaks$CpG_island & filtered_peaks$TATA > 0, "CpG Island & TATA", ifelse(filtered_peaks$CpG_island, "CpG Island", ifelse(filtered_peaks$TATA > 0, "TATA", "No CpG Island & No TATA")))
filtered_peaks$category = factor(filtered_peaks$category, levels = c("No CpG Island & No TATA", "TATA", "CpG Island", "CpG Island & TATA"))

lfc_data = readRDS("./resources/processed_featureCounts_introns_TC_data.rds")
merged = merge(lfc_data, filtered_peaks, by="gene_id", all.x = TRUE)
merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()

merged$category = ifelse(is.na(merged$category), "No Assigned Peak", as.character(merged$category))
merged$category = factor(merged$category, levels = c("No Assigned Peak", "No CpG Island & No TATA", "TATA", "CpG Island", "CpG Island & TATA"))

comparison = append(list(c("No Assigned Peak", "No CpG Island & No TATA")), comparison)

counts <- merged %>%
  group_by(category) %>%
  summarise(n = paste("n=", n(), sep = ""))

plot = ggplot(merged, aes(x = category, y = log2FoldChange)) +
  geom_boxplot(aes(color = "white"),outliers = FALSE) +
  geom_jitter(data=merged[merged$category != "No Assigned Peak",], aes(color = category), width = 0.2, alpha = 0.1) +
  geom_boxplot(aes(color = category),outliers = FALSE) +
  scale_color_manual(values = c("grey", "black", "brown3", "skyblue", "violet", "white")) +
  labs(#title = "LFC of intronic TC Talk-seq data for Strongest MYC peaks for Combinations of TATA-boxes and CpG Islands",
       x = NULL,
       y = "-log2(Fold Change MYC vs. MYC Depletion") +
  geom_text(
    data = counts,
    aes(x = category, y = -1.5, label = n), # Fixed y position
    size = 4
  )+
  guides(color = "none") +
  coord_cartesian(ylim = c(-1.5, 2.2))+
  theme_minimal()+ 
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparison, hide.ns = FALSE, label.y = c(1.6, 1.6, 1.6, 1.9, 1.6))

plot
ggsave("./box_lfc_combinations_CpG_TATA.pdf", plot, width = 9, height = 6)


ebox_motifs <- c("CACGTG", "CACGCG", "CATGCG", "CATGTG", "CACGAG", "CACATG")
group_by_motif = annotated_peaks
group_by_motif$TATA_existence = group_by_motif$TATA > 0

group_by_motif$motif = ebox_motifs[apply(group_by_motif[, ebox_motifs], 1, which.max)]
group_by_motif$motif = ifelse(group_by_motif$sum_eboxes > 1, "2+ E-boxes", ifelse(group_by_motif$sum_eboxes == 0, "No E-box", group_by_motif$motif))

group_by_motif$motif = factor(group_by_motif$motif, levels = c("No E-box", ebox_motifs, "2+ E-boxes"))


counts <- group_by_motif %>%
  group_by(motif, TATA_existence) %>%
  summarise(n = n(), .groups = "drop") %>%
  group_by(motif) %>%
  summarise(n = paste("n=", paste(n, collapse = ", "), sep = ""))


correlation <- cor(group_by_motif$TATA_existence, group_by_motif$coverage_myc, method = "spearman")


ggplot() +
  geom_boxplot(data = group_by_motif, aes(x = motif, y = coverage_myc, fill = TATA_existence)) +
  labs(title = "coverage_MYC per Motif (max 1) for TATA-boxes",
       x = "Motif",
       y = "coverage_MYC",
       fill = "TATA-box Island") +
  
  stat_compare_means(data = group_by_motif,
                     aes(x = motif, y = coverage_myc, group = TATA_existence),
                     label = "p.format", 
                     method = "wilcox.test",
                     label.y = c(650, 650, 650, 650, 650, 650)
  )+
  geom_text(
    data = counts,
    aes(x = motif, y = -6, label = n, group = TATA_existence),
    size = 4,
    position = position_dodge(width = 0.9)
  ) +
  annotate("text", x = 1, y = 700, 
           label = paste0("Correlation: ", round(correlation, 2)), 
           hjust = 0, vjust = 1.1, size = 5, color = "blue") +
  theme_minimal() +
  theme( plot.title = element_text(hjust = 0.5))


library(colorspace)

# Define original ggplot2 colors
ggplot_colors <- scales::hue_pal()(length(ebox_motifs))

# Generate lighter and darker versions
lighter_colors <- lighten(ggplot_colors, amount = 0.3)  # Increase brightness
darker_colors  <- darken(ggplot_colors, amount = 0.3)   # Decrease brightness

sub_colors <- as.vector(rbind(lighter_colors, darker_colors))

group_by_motif$group_TATA <- interaction(group_by_motif$motif, group_by_motif$TATA_existence)

plot = ggplot(group_by_motif, aes(x = motif, y = coverage_myc)) +
  geom_point(data = group_by_motif[group_by_motif$motif == "No E-box", ], color = "white") +
  geom_point(data = group_by_motif[group_by_motif$motif != "No E-box", ], aes(color = group_TATA, fill = TATA_existence), size=0.5, alpha = 0.1, position = position_jitterdodge(jitter.width = 0.2)) +
  scale_color_manual(values = c(lighter_colors, "chocolate2", darker_colors, "chocolate4", "black", "black"), guide = "none") +
  scale_fill_manual(values = c("white", "white"), guide = "none") +
  new_scale_color() +
  geom_boxplot(data = group_by_motif[group_by_motif$motif == "No E-box", ], aes(color = group_TATA), outliers = FALSE) +
  scale_color_manual(values = c("azure4", "black"), labels = c("FALSE", "TRUE"), name="TATA-box", guide = "legend") +
  new_scale_color() +
  geom_boxplot(data = group_by_motif[group_by_motif$motif != "No E-box", ], aes(color = group_TATA), outliers = FALSE) +
  scale_color_manual(values = c(lighter_colors, "chocolate2", darker_colors, "chocolate4"), guide = "none") +
  labs(title = "coverage_MYC per Motif (max 1) for TATA-boxes",
       x = "Motif",
       y = "coverage_MYC",
       fill = "TATA-box") +
  
  stat_compare_means(data = group_by_motif,
                     aes(x = motif, y = coverage_myc, group = TATA_existence),
                     label = "p.signif", 
                     method = "wilcox.test",
                     label.y = rep(120, 8)
  )+
  geom_text(
    data = counts,
    aes(x = motif, y = -15, label = n),
    size = 4,
    position = position_dodge(width = 0.9)
  ) +
  annotate("text", x = 1, y = 130, 
           label = paste0("Correlation: ", round(correlation, 2)), 
           hjust = 0, vjust = 1.1, size = 5, color = "blue") +
  theme_minimal() +
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  theme( plot.title = element_text(hjust = 0.5)) + 
  scale_y_continuous(limits = c(-15, 130))
plot
ggsave("./box_coverage_TATA_per_motif.svg", plot, width = 9, height = 6)



ebox_motifs <- c("CACGTG", "CACGCG", "CATGCG", "CATGTG", "CACGAG", "CACATG")
group_by_motif_without_cpg = annotated_peaks[!annotated_peaks$CpG_island,]
group_by_motif_without_cpg$TATA_existence = group_by_motif_without_cpg$TATA > 0

group_by_motif_without_cpg$motif = ebox_motifs[apply(group_by_motif_without_cpg[, ebox_motifs], 1, which.max)]
group_by_motif_without_cpg$motif = ifelse(group_by_motif_without_cpg$sum_eboxes > 1, "2+ E-boxes", ifelse(group_by_motif_without_cpg$sum_eboxes == 0, "No E-box", group_by_motif_without_cpg$motif))

group_by_motif_without_cpg$motif = factor(group_by_motif_without_cpg$motif, levels = c("No E-box", ebox_motifs, "2+ E-boxes"))


group_by_motif_without_cpg$group_TATA <- interaction(group_by_motif_without_cpg$motif, group_by_motif_without_cpg$TATA_existence)

counts <- group_by_motif_without_cpg %>%
  group_by(motif, TATA_existence) %>%
  summarise(n = n(), .groups = "drop") %>%
  group_by(motif) %>%
  summarise(n = paste("n=", paste(n, collapse = ", "), sep = ""))

correlation <- cor(group_by_motif_without_cpg$TATA_existence, group_by_motif_without_cpg$coverage_myc, method = "spearman")

plot = ggplot(group_by_motif_without_cpg, aes(x = motif, y = coverage_myc)) +
  geom_point(data = group_by_motif_without_cpg[group_by_motif_without_cpg$motif == "No E-box", ], color = "white") +
  geom_point(data = group_by_motif_without_cpg[group_by_motif_without_cpg$motif != "No E-box", ], aes(color = group_TATA, fill = TATA_existence), size=0.5, alpha = 0.1, position = position_jitterdodge(jitter.width = 0.2)) +
  scale_color_manual(values = c(lighter_colors, "chocolate2", darker_colors, "chocolate4", "black", "black"), guide = "none") +
  scale_fill_manual(values = c("white", "white"), guide = "none") +
  new_scale_color() +
  geom_boxplot(data = group_by_motif_without_cpg[group_by_motif_without_cpg$motif == "No E-box", ], aes(color = group_TATA), outliers = FALSE) +
  scale_color_manual(values = c("azure4", "black"), labels = c("FALSE", "TRUE"), name="TATA-box", guide = "legend") +
  new_scale_color() +
  geom_boxplot(data = group_by_motif_without_cpg[group_by_motif_without_cpg$motif != "No E-box", ], aes(color = group_TATA), outliers = FALSE) +
  scale_color_manual(values = c(lighter_colors, "chocolate2", darker_colors, "chocolate4"), guide = "none") +
  labs(title = "coverage_MYC for Peaks without CpG Islands per Motif (max 1) for TATA-boxes",
       x = "Motif",
       y = "coverage_MYC",
       fill = "TATA-box") +
  
  stat_compare_means(data = group_by_motif_without_cpg,
                     aes(x = motif, y = coverage_myc, group = TATA_existence),
                     label = "p.signif", 
                     method = "wilcox.test",
                     label.y = rep(55, 8)
  )+
  geom_text(
    data = counts,
    aes(x = motif, y = -10, label = n),
    size = 4,
    position = position_dodge(width = 0.9)
  ) +
  annotate("text", x = 1, y = 60, 
           label = paste0("Correlation: ", round(correlation, 2)), 
           hjust = 0, vjust = 1.1, size = 5, color = "blue") +
  theme_minimal() +
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  theme( plot.title = element_text(hjust = 0.5)) + 
  scale_y_continuous(limits = c(-10, 60))
plot
ggsave("./box_coverage_TATA_per_motif_no_cpg.svg", plot, width = 9, height = 6)
  





lfc_data = readRDS("./resources/processed_featureCounts_introns_TC_data.rds")

merged = merge(lfc_data, group_by_motif, by="gene_id", all.x = TRUE)

# only keep the peak with the highest coverage per gene
merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()

merged_no_na = merged[!is.na(merged$coverage_myc),]

merged_no_na$group_TATA <- interaction(merged_no_na$motif, merged_no_na$TATA_existence)

counts <- merged_no_na %>%
  group_by(motif, TATA_existence) %>%
  summarise(n = n(), .groups = "drop") %>%
  group_by(motif) %>%
  summarise(n = paste("n=", paste(n, collapse = ", "), sep = ""))

correlation <- cor(merged_no_na$TATA_existence, merged_no_na$coverage_myc, method = "spearman")


plot = ggplot(merged_no_na, aes(x = motif, y = coverage_myc)) +
  geom_point(data = merged_no_na[merged_no_na$motif == "No E-box", ], color = "white") +
  geom_point(data = merged_no_na[merged_no_na$motif != "No E-box", ], aes(color = group_TATA, fill = TATA_existence), size=0.5, alpha = 0.1, position = position_jitterdodge(jitter.width = 0.2)) +
  scale_color_manual(values = c(lighter_colors, "chocolate2", darker_colors, "chocolate4", "black", "black"), guide = "none") +
  scale_fill_manual(values = c("white", "white"), guide = "none") +
  new_scale_color() +
  geom_boxplot(data = merged_no_na[merged_no_na$motif == "No E-box", ], aes(color = group_TATA), outliers = FALSE) +
  scale_color_manual(values = c("azure4", "black"), labels = c("FALSE", "TRUE"), name="TATA-box", guide = "legend") +
  new_scale_color() +
  geom_boxplot(data = merged_no_na[merged_no_na$motif != "No E-box", ], aes(color = group_TATA), outliers = FALSE) +
  scale_color_manual(values = c(lighter_colors, "chocolate2", darker_colors, "chocolate4"), guide = "none") +
  labs(title = "coverage_MYC per Motif (max 1) for TATA-boxes for strongest peak",
       x = "Motif",
       y = "coverage_MYC",
       fill = "TATA-box") +
  
  stat_compare_means(data = merged_no_na,
                     aes(x = motif, y = coverage_myc, group = TATA_existence),
                     label = "p.signif", 
                     method = "wilcox.test",
                     label.y = rep(140, 8)
  )+
  geom_text(
    data = counts,
    aes(x = motif, y = -15, label = n),
    size = 4,
    position = position_dodge(width = 0.9)
  ) +
  annotate("text", x = 1, y = 160, 
           label = paste0("Correlation: ", round(correlation, 2)), 
           hjust = 0, vjust = 1.1, size = 5, color = "blue") +
  theme_minimal() +
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  theme( plot.title = element_text(hjust = 0.5)) + 
  scale_y_continuous(limits = c(-15, 160))
plot
ggsave("./box_coverage_TATA_per_motif_strongest.svg", plot, width = 9, height = 6)


merged$motif = ifelse(is.na(merged$motif), "No Assigned Peak", as.character(merged$motif))
merged$motif = factor(merged$motif, levels = c("No Assigned Peak", "No E-box", ebox_motifs, "2+ E-boxes"))

merged$group_TATA <- interaction(merged$motif, merged$TATA_existence)

counts <- merged %>%
  group_by(motif, TATA_existence) %>%
  summarise(n = n(), .groups = "drop") %>%
  group_by(motif) %>%
  summarise(n = paste("n=", paste(n, collapse = ", "), sep = ""))

correlation <- cor(merged$TATA_existence, merged$log2FoldChange, method = "spearman", use = "complete.obs")


plot = ggplot(merged, aes(x = motif, y = log2FoldChange)) +
  geom_point(data = merged[merged$motif == "No Assigned Peak", ], color = "white") +
  geom_point(data = merged[merged$motif == "No E-box", ], color = "white") +
  geom_point(data = merged[merged$motif %in% ebox_motifs | merged$motif == "2+ E-boxes", ], aes(color = group_TATA, fill = TATA_existence), size=0.5, alpha = 0.1, position = position_jitterdodge(jitter.width = 0.2)) +
  scale_color_manual(values = c(lighter_colors, "chocolate2", darker_colors, "chocolate4", "black", "black"), guide = "none") +
  scale_fill_manual(values = c("white", "white"), guide = "none") +
  new_scale_color() +
  geom_boxplot(data = merged[merged$motif == "No Assigned Peak", ], color = "grey", outliers = FALSE) +
  geom_boxplot(data = merged[merged$motif == "No E-box", ], aes(color = group_TATA), outliers = FALSE) +
  scale_color_manual(values = c("azure4", "black"), labels = c("FALSE", "TRUE"), name="TATA-box", guide = "legend") +
  new_scale_color() +
  geom_boxplot(data = merged[merged$motif %in% ebox_motifs | merged$motif == "2+ E-boxes", ], aes(color = group_TATA), outliers = FALSE) +
  scale_color_manual(values = c(lighter_colors, "chocolate2", darker_colors, "chocolate4"), guide = "none") +
  labs(title = "LFC of intronic TC Talk-seq data per Motif (max 1) of strongest peak for TATA-boxes",
       x = "Motif",
       y = "-log2(Fold Change MYC vs. MYC Depletion)",
       fill = "TATA-box") +
  
  stat_compare_means(data = merged,
                     aes(x = motif, y = log2FoldChange, group = TATA_existence),
                     label = "p.signif", 
                     method = "wilcox.test",
                     label.y = rep(2.4, 8)
  )+
  geom_text(
    data = counts,
    aes(x = motif, y = -1.5, label = n),
    size = 4,
    position = position_dodge(width = 0.9)
  ) +
  annotate("text", x = 1, y = 2.7, 
           label = paste0("Correlation: ", round(correlation, 2)), 
           hjust = 0, vjust = 1.1, size = 5, color = "blue") +
  theme_minimal() +
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  theme( plot.title = element_text(hjust = 0.5)) + 
  scale_y_continuous(limits = c(-1.5, 2.7))
plot
ggsave("./box_lfc_TATA_per_motif.svg", plot, width = 9, height = 6)



lfc_data = readRDS("./resources/processed_featureCounts_introns_TC_data.rds")

merged = merge(lfc_data, group_by_motif_without_cpg, by="gene_id", all.x = TRUE)

# only keep the peak with the highest coverage per gene
merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()

merged_no_na = merged[!is.na(merged$coverage_myc),]

merged_no_na$group_TATA <- interaction(merged_no_na$motif, merged_no_na$TATA_existence)

counts <- merged_no_na %>%
  group_by(motif, TATA_existence) %>%
  summarise(n = n(), .groups = "drop") %>%
  group_by(motif) %>%
  summarise(n = paste("n=", paste(n, collapse = ", "), sep = ""))

correlation <- cor(merged_no_na$TATA_existence, merged_no_na$coverage_myc, method = "spearman")

plot = ggplot(merged_no_na, aes(x = motif, y = coverage_myc)) +
  geom_point(data = merged_no_na[merged_no_na$motif == "No E-box", ], color = "white") +
  geom_point(data = merged_no_na[merged_no_na$motif != "No E-box", ], aes(color = group_TATA, fill = TATA_existence), size=0.5, alpha = 0.1, position = position_jitterdodge(jitter.width = 0.2)) +
  scale_color_manual(values = c(lighter_colors, "chocolate2", darker_colors, "chocolate4", "black", "black"), guide = "none") +
  scale_fill_manual(values = c("white", "white"), guide = "none") +
  new_scale_color() +
  geom_boxplot(data = merged_no_na[merged_no_na$motif == "No E-box", ], aes(color = group_TATA), outliers = FALSE) +
  scale_color_manual(values = c("azure4", "black"), labels = c("FALSE", "TRUE"), name="TATA-box", guide = "legend") +
  new_scale_color() +
  geom_boxplot(data = merged_no_na[merged_no_na$motif != "No E-box", ], aes(color = group_TATA), outliers = FALSE) +
  scale_color_manual(values = c(lighter_colors, "chocolate2", darker_colors, "chocolate4"), guide = "none") +
  labs(title = "coverage_MYC for Peaks without CpG Islands per Motif (max 1) for TATA-boxes for strongest peaks",
       x = "Motif",
       y = "coverage_MYC",
       fill = "TATA-box") +
  
  stat_compare_means(data = merged_no_na,
                     aes(x = motif, y = coverage_myc, group = TATA_existence),
                     label = "p.signif", 
                     method = "wilcox.test",
                     label.y = rep(75, 8)
  )+
  geom_text(
    data = counts,
    aes(x = motif, y = -10, label = n),
    size = 4,
    position = position_dodge(width = 0.9)
  ) +
  annotate("text", x = 1, y = 80, 
           label = paste0("Correlation: ", round(correlation, 2)), 
           hjust = 0, vjust = 1.1, size = 5, color = "blue") +
  theme_minimal() +
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  theme( plot.title = element_text(hjust = 0.5)) + 
  scale_y_continuous(limits = c(-10, 80))
plot
ggsave("./box_coverage_TATA_per_motif_no_cpg_strongest.svg", plot, width = 9, height = 6)



merged$motif = ifelse(is.na(merged$motif), "No Assigned Peak", as.character(merged$motif))
merged$motif = factor(merged$motif, levels = c("No Assigned Peak", "No E-box", ebox_motifs, "2+ E-boxes"))

merged$group_TATA <- interaction(merged$motif, merged$TATA_existence)

counts <- merged %>%
  group_by(motif, TATA_existence) %>%
  summarise(n = n(), .groups = "drop") %>%
  group_by(motif) %>%
  summarise(n = paste("n=", paste(n, collapse = ", "), sep = ""))

correlation <- cor(merged$TATA_existence, merged$log2FoldChange, method = "spearman", use = "complete.obs")


plot = ggplot(merged, aes(x = motif, y = log2FoldChange)) +
  geom_point(data = merged[merged$motif == "No Assigned Peak", ], color = "white") +
  geom_point(data = merged[merged$motif == "No E-box", ], color = "white") +
  geom_point(data = merged[merged$motif %in% ebox_motifs | merged$motif == "2+ E-boxes", ], aes(color = group_TATA, fill = TATA_existence), size=0.5, alpha = 0.1, position = position_jitterdodge(jitter.width = 0.2)) +
  scale_color_manual(values = c(lighter_colors, "chocolate2", darker_colors, "chocolate4", "black", "black"), guide = "none") +
  scale_fill_manual(values = c("white", "white"), guide = "none") +
  new_scale_color() +
  geom_boxplot(data = merged[merged$motif == "No Assigned Peak", ], color = "grey", outliers = FALSE) +
  geom_boxplot(data = merged[merged$motif == "No E-box", ], aes(color = group_TATA), outliers = FALSE) +
  scale_color_manual(values = c("azure4", "black"), labels = c("FALSE", "TRUE"), name="TATA-box", guide = "legend") +
  new_scale_color() +
  geom_boxplot(data = merged[merged$motif %in% ebox_motifs | merged$motif == "2+ E-boxes", ], aes(color = group_TATA), outliers = FALSE) +
  scale_color_manual(values = c(lighter_colors, "chocolate2", darker_colors, "chocolate4"), guide = "none") +
  labs(title = "LFC of intronic TC Talk-seq data per Motif (max 1) of strongest peak without CpG Island for TATA-boxes",
       x = "Motif",
       y = "-log2(Fold Change MYC vs. MYC Depletion)",
       fill = "TATA-box") +
  
  stat_compare_means(data = merged,
                     aes(x = motif, y = log2FoldChange, group = TATA_existence),
                     label = "p.signif", 
                     method = "wilcox.test",
                     label.y = rep(2.4, 8)
  )+
  geom_text(
    data = counts,
    aes(x = motif, y = -1.5, label = n),
    size = 4,
    position = position_dodge(width = 0.9)
  ) +
  annotate("text", x = 1, y = 2.7, 
           label = paste0("Correlation: ", round(correlation, 2)), 
           hjust = 0, vjust = 1.1, size = 5, color = "blue") +
  theme_minimal() +
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  theme( plot.title = element_text(hjust = 0.5)) + 
  scale_y_continuous(limits = c(-1.5, 2.7))
plot
ggsave("./box_lfc_TATA_per_motif_no_cpg.svg", plot, width = 9, height = 6)

