# SPDX-FileCopyrightText: 2025 Julius Deynet <julius.deynet@stud-mail.uni-wuerzburg.de>
#
# SPDX-License-Identifier: GPL-3.0-or-later

suppressPackageStartupMessages({
  library(ggplot2)
  library(dplyr)
})


# Depletion vs MYC
bursting_data = as.data.frame(read.csv("./resources/bursting_all_genes.csv"))

#bursting_data = subset(bursting_data, bestmodel != "no change")

names(bursting_data)[names(bursting_data) == "km"] = "burst_frequency"
names(bursting_data)[names(bursting_data) == "B"] = "burst_size"


ebox_motifs <- c("CACGTG", "CACGCG", "CATGCG", "CATGTG", "CACGAG", "CACATG")

lfc_data = readRDS("./resources/processed_featureCounts_introns_TC_data.rds")
lfc_sc_data = readRDS("./resources/processed_grandR_single_cell_new_data.rds")

lfc_data = merge(lfc_data, bursting_data, by.x="gene_name", by.y="gene")
lfc_sc_data = merge(lfc_sc_data, bursting_data, by.x="gene_name", by.y="gene")

cor(lfc_data$burst_frequency + lfc_data$burst_size, lfc_data$log2FoldChange, method = "spearman", use = "pairwise.complete.obs")

merged = merge(lfc_sc_data, annotated_peaks, by="gene_id", all.x = TRUE)
merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()

merged$bestmodel = factor(merged$bestmodel, levels = c("no change", "size", "frequency", "both"))

comparison = list(c("no change", "size"), c("no change", "frequency"),  c("no change", "both"))

baseline = median(merged[merged$bestmodel == "no change", ]$log2FoldChange)

counts <- merged %>%
  group_by(bestmodel) %>%
  summarise(n = paste("n=", n(), sep = ""))

ggplot(merged[merged$log2FoldChange > 0, ], aes(x = bestmodel, y = abs(log2FoldChange))) +
  geom_hline(yintercept = baseline, linetype = "dashed", color = "black") + 
  geom_boxplot() +
  theme_minimal() +
  stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparison) +
  geom_text(
    data = counts,
    aes(x = bestmodel, y = -0.2, label = n), # Fixed y position
    size = 4
  )

#merged = merged[merged$log2FoldChange > 0, ]

subset = (subset(merged, !is.na(coverage_myc)))

counts <- subset %>%
  group_by(bestmodel) %>%
  summarise(n = paste("n=", n(), sep = ""))

comparisons = list(c("no change", "size"), c("no change", "frequency"), c("no change", "both"),  c("size", "frequency"), c("size", "both"), c("frequency", "both"))

baseline = median(subset[subset$bestmodel == "no change", ]$coverage_myc)

plot = ggplot(subset, aes(x = bestmodel, y = coverage_myc)) + 
  geom_jitter(aes(color = bestmodel), width = 0.2, alpha = 0.1) +
  geom_hline(yintercept = baseline, linetype = "dashed", color = "black") + 
  geom_boxplot( outliers = FALSE, aes(color = bestmodel)) +
  scale_color_manual(values = c("black", "brown3", "skyblue", "violet")) +
  labs(x = "Bursting Category",
       y = "Coverage_MYC") +
  #scale_color_manual(values = brewer.pal(n = 4, name = "Dark2")) +
  geom_text(
    data = counts,
    aes(x = bestmodel, y = -20, label = n), # Fixed y position
    size = 4
  )+
  theme_minimal()+ 
  theme(legend.position = "none") +
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparisons, hide.ns = FALSE, label.y = c(190, 210, 230, 190, 250, 190)) +
  scale_y_continuous(limits = c(-20, 280))
plot
ggsave("./box_coverage_MYC_bursting_categories_all.pdf", plot, width = 9, height = 6)


counts <- merged %>%
  group_by(bestmodel) %>%
  summarise(n = paste("n=", n(), sep = ""))

comparisons = list(c("no change", "size"), c("no change", "frequency"), c("no change", "both"),  c("size", "frequency"), c("size", "both"), c("frequency", "both"))

baseline = median(merged[merged$bestmodel == "no change", ]$log2FoldChange)


plot = ggplot(merged, aes(x = bestmodel, y = log2FoldChange)) + 
  geom_jitter(aes(color = bestmodel), width = 0.2, alpha = 0.1) +
  geom_hline(yintercept = baseline, linetype = "dashed", color = "black") + 
  geom_boxplot( outliers = FALSE, aes(color = bestmodel)) +
  scale_color_manual(values = c("black", "brown3", "skyblue", "violet")) +
  labs(x = "Bursting Category",
       y = "all -log2(Fold Change MYC vs. MYC Depletion)") +
  #scale_color_manual(values = brewer.pal(n = 4, name = "Dark2")) +
  geom_text(
    data = counts,
    aes(x = bestmodel, y = -1.5, label = n), # Fixed y position
    size = 4
  )+
  theme_minimal()+ 
  theme(legend.position = "none") +
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparisons, hide.ns = FALSE, label.y = c(1.5, 1.7, 1.9, 1.5, 2.1, 1.5))
plot
ggsave("./box_LFC_bursting_categories_all.pdf", plot, width = 9, height = 6)



group_by_motif = annotated_peaks

group_by_motif$motif = ebox_motifs[apply(group_by_motif[, ebox_motifs], 1, which.max)]
group_by_motif$motif = ifelse(group_by_motif$sum_eboxes > 1, "2+ E-boxes", ifelse(group_by_motif$sum_eboxes == 0, "No E-box", group_by_motif$motif))

group_by_motif$motif = factor(group_by_motif$motif, levels = c("No E-box", ebox_motifs, "2+ E-boxes"))



merged = merge(lfc_data, annotated_peaks, by="gene_id", all.x = TRUE)
merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()
cor(merged$coverage_myc, merged$log2FoldChange, method = "spearman", use = "pairwise.complete.obs")

merged$sum_eboxes = ifelse(is.na(merged$sum_eboxes), -1, merged$sum_eboxes)
merged[merged$sum_eboxes == -1, ebox_motifs] = -1

merged <- merged %>%
  mutate(
    motif = case_when(
      sum_eboxes == -1 ~ "No Assigned Peak",
      sum_eboxes == 0 ~ "No E-box",
      sum_eboxes == 1 ~ ebox_motifs[apply(.[, ebox_motifs], 1, which.max)],
      TRUE ~ "2+ E-boxes",
    )
  )




merged$motif = factor(merged$motif, levels = c("No Assigned Peak", "No E-box", ebox_motifs, "2+ E-boxes"))


comparisons <- list(c("No E-box", "CACGTG"), c("No E-box", "CACGCG"), c("No E-box", "CATGCG"), c("No E-box", "CATGTG"), c("No E-box", "CACGAG"), c("No E-box", "CACATG"))



counts <- merged %>%
  group_by(motif) %>%
  summarise(n = paste("n=", n(), sep = ""))

new_comparisons = append(comparisons, list(c("No Assigned Peak", "No single E-box"), c("No Assigned Peak", "No E-box")), after = 0)

correlation = cor(merged$sum_eboxes, merged$burst_size, method = "spearman", use = "pairwise.complete.obs")

median_baseline = median(merged[merged$motif == "No E-box", ]$burst_size)

plot = ggplot(merged, aes(x = motif, y = burst_size)) + 
  geom_boxplot(data = merged[merged$motif == "No Assigned Peak", ],outliers = FALSE, color = "grey") +
  geom_boxplot(data = merged[merged$motif == "No E-box", ],outliers = FALSE) +
  geom_jitter(data = merged[as.character(merged$motif) %in% ebox_motifs, ], aes(color = motif), width = 0.2, alpha = 0.1) +
  geom_jitter(data = merged[merged$motif == "2+ E-boxes", ], width = 0.2, alpha = 0.1, color="chocolate3") +
  geom_hline(yintercept = median_baseline, linetype = "dashed", color = "black") +
  geom_boxplot(data = merged[merged$motif == "No Assigned Peak", ],outliers = FALSE, color = "grey") +
  geom_boxplot(data = merged[as.character(merged$motif) %in% ebox_motifs, ],outliers = FALSE, aes(color = motif)) +
  geom_boxplot(data = merged[merged$motif == "2+ E-boxes", ],outliers = FALSE, color = "chocolate3") +
  labs(title = "Burst Size Across Genes for strongest Peaks highlighting 1 E-box Motif",
       x = "Motif",
       y = "-log2(Fold Change Burst Size MYC vs. MYC Depletion)") +
  geom_text(
    data = counts,
    aes(x = motif, y = -1.5, label = n), # Fixed y position
    size = 4
  )+
  theme(panel.grid = element_blank())+
  guides(color = "none") +
  theme_minimal()+
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparisons, hide.ns = FALSE, label.y = seq(0.3, 5.3, 0.3), tip.length = 0.01)  +
  scale_y_continuous(breaks = seq(-1, 4, 1), limits = c(-1.5, 2.6))

plot

ggsave("./box_burst_size_ebox_per_motif.svg", plot, width = 9, height = 6)



correlation = cor(merged$sum_eboxes, merged$burst_frequency, method = "spearman", use = "pairwise.complete.obs")

median_baseline = median(merged[merged$motif == "No E-box", ]$burst_frequency)

plot = ggplot(merged, aes(x = motif, y = burst_frequency)) + 
  geom_boxplot(data = merged[merged$motif == "No Assigned Peak", ],outliers = FALSE, color = "grey") +
  geom_boxplot(data = merged[merged$motif == "No E-box", ],outliers = FALSE) +
  geom_jitter(data = merged[as.character(merged$motif) %in% ebox_motifs, ], aes(color = motif), width = 0.2, alpha = 0.1) +
  geom_jitter(data = merged[merged$motif == "2+ E-boxes", ], width = 0.2, alpha = 0.1, color="chocolate3") +
  geom_hline(yintercept = median_baseline, linetype = "dashed", color = "black") +
  geom_boxplot(data = merged[merged$motif == "No Assigned Peak", ],outliers = FALSE, color = "grey") +
  geom_boxplot(data = merged[as.character(merged$motif) %in% ebox_motifs, ],outliers = FALSE, aes(color = motif)) +
  geom_boxplot(data = merged[merged$motif == "2+ E-boxes", ],outliers = FALSE, color = "chocolate3") +
  labs(title = "Burst Frequency Across Genes for strongest Peaks highlighting 1 E-box Motif",
       x = "Motif",
       y = "-log2(Fold Change Burst Frequency MYC vs. MYC Depletion)") +
  geom_text(
    data = counts,
    aes(x = motif, y = -1.5, label = n), # Fixed y position
    size = 4
  )+
  theme(panel.grid = element_blank())+
  guides(color = "none") +
  theme_minimal()+
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparisons, hide.ns = FALSE, label.y = seq(0.5, 5.3, 0.3), tip.length = 0.01)  +
  scale_y_continuous(breaks = seq(-1, 4, 1), limits = c(-1.5, 2.7))

plot

ggsave("./box_burst_frequency_ebox_per_motif.svg", plot, width = 9, height = 6)

library(colorspace)

# Define original ggplot2 colors
ggplot_colors <- scales::hue_pal()(length(ebox_motifs))

# Generate lighter and darker versions
lighter_colors <- lighten(ggplot_colors, amount = 0.3)  # Increase brightness
darker_colors  <- darken(ggplot_colors, amount = 0.3)   # Decrease brightness

sub_colors <- as.vector(rbind(lighter_colors, darker_colors))

merged$group_CpG <- interaction(merged$motif, merged$CpG_island)

counts <- merged %>%
  group_by(motif, CpG_island) %>%
  summarise(n = n(), .groups = "drop") %>%
  group_by(motif) %>%
  summarise(n = paste("n=", paste(n, collapse = ", "), sep = ""))


correlation <- cor(merged$CpG_island, merged$burst_size, method = "spearman", use = "complete.obs")

median_baseline = median(merged[merged$motif == "No E-box", ]$burst_size)

plot = ggplot(merged, aes(x = motif, y = burst_size)) +
  geom_point(data = merged[merged$motif == "No Assigned Peak", ], color = "white") +
  geom_point(data = merged[merged$motif == "No E-box", ], color = "white") +
  geom_point(data = merged[merged$motif %in% ebox_motifs | merged$motif == "2+ E-boxes", ], aes(color = group_CpG, fill = CpG_island), size=0.5, alpha = 0.1, position = position_jitterdodge(jitter.width = 0.2)) +
  scale_color_manual(values = c(lighter_colors, "chocolate2", darker_colors, "chocolate4", "black", "black"), guide = "none") +
  scale_fill_manual(values = c("white", "white"), guide = "none") +
  new_scale_color() +
  geom_boxplot(data = merged[merged$motif == "No Assigned Peak", ], color = "grey", outliers = FALSE) +
  geom_boxplot(data = merged[merged$motif == "No E-box", ], aes(color = group_CpG), outliers = FALSE) +
  geom_hline(yintercept = median_baseline, linetype = "dashed", color = "black") +
  geom_boxplot(data = merged[merged$motif == "No Assigned Peak", ], color = "grey", outliers = FALSE) +
  scale_color_manual(values = c("azure4", "black"), labels = c("FALSE", "TRUE"), name="CpG Island", guide = "legend") +
  new_scale_color() +
  geom_boxplot(data = merged[merged$motif %in% ebox_motifs | merged$motif == "2+ E-boxes", ], aes(color = group_CpG), outliers = FALSE) +
  scale_color_manual(values = c(lighter_colors, "chocolate2", darker_colors, "chocolate4"), guide = "none") +
  labs(title = "Burst Size of intronic TC Talk-seq data per Motif (max 1) of strongest peak for CpG Islands",
       x = "Motif",
       y = "-log2(Fold Change Burst Size MYC vs. MYC Depletion)",
       fill = "CpG Island") +
  
  stat_compare_means(data = merged,
                     aes(x = motif, y = log2FoldChange, group = CpG_island),
                     label = "p.signif", 
                     method = "wilcox.test",
                     label.y = 1
  )+
  geom_text(
    data = counts,
    aes(x = motif, y = -1.3, label = n),
    size = 4,
    position = position_dodge(width = 0.9)
  ) +
  annotate("text", x = 1, y = 1.2, 
           label = paste0("Correlation: ", round(correlation, 2)), 
           hjust = 0, vjust = 1.1, size = 5, color = "blue") +
  theme_minimal() +
  theme( plot.title = element_text(hjust = 0.5)) +
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  scale_y_continuous(limits = c(-1.3, 1.2))
plot
ggsave("./box_burst_size_cpg_islands_per_motif_strongest.svg", plot, width = 9, height = 6)


correlation <- cor(merged$CpG_island, merged$burst_frequency, method = "spearman", use = "complete.obs")

median_baseline = median(merged[merged$motif == "No E-box", ]$burst_frequency)

plot = ggplot(merged, aes(x = motif, y = burst_frequency)) +
  geom_point(data = merged[merged$motif == "No Assigned Peak", ], color = "white") +
  geom_point(data = merged[merged$motif == "No E-box", ], color = "white") +
  geom_point(data = merged[merged$motif %in% ebox_motifs | merged$motif == "2+ E-boxes", ], aes(color = group_CpG, fill = CpG_island), size=0.5, alpha = 0.1, position = position_jitterdodge(jitter.width = 0.2)) +
  scale_color_manual(values = c(lighter_colors, "chocolate2", darker_colors, "chocolate4", "black", "black"), guide = "none") +
  scale_fill_manual(values = c("white", "white"), guide = "none") +
  new_scale_color() +
  geom_boxplot(data = merged[merged$motif == "No Assigned Peak", ], color = "grey", outliers = FALSE) +
  geom_boxplot(data = merged[merged$motif == "No E-box", ], aes(color = group_CpG), outliers = FALSE) +
  geom_hline(yintercept = median_baseline, linetype = "dashed", color = "black") +
  geom_boxplot(data = merged[merged$motif == "No Assigned Peak", ], color = "grey", outliers = FALSE) +
  scale_color_manual(values = c("azure4", "black"), labels = c("FALSE", "TRUE"), name="CpG Island", guide = "legend") +
  new_scale_color() +
  geom_boxplot(data = merged[merged$motif %in% ebox_motifs | merged$motif == "2+ E-boxes", ], aes(color = group_CpG), outliers = FALSE) +
  scale_color_manual(values = c(lighter_colors, "chocolate2", darker_colors, "chocolate4"), guide = "none") +
  labs(title = "Burst Frequency of intronic TC Talk-seq data per Motif (max 1) of strongest peak for CpG Islands",
       x = "Motif",
       y = "-log2(Fold Change Burst Frequency MYC vs. MYC Depletion)",
       fill = "CpG Island") +
  
  stat_compare_means(data = merged,
                     aes(x = motif, y = log2FoldChange, group = CpG_island),
                     label = "p.signif", 
                     method = "wilcox.test",
                     label.y = 1
  )+
  geom_text(
    data = counts,
    aes(x = motif, y = -1.3, label = n),
    size = 4,
    position = position_dodge(width = 0.9)
  ) +
  annotate("text", x = 1, y = 1.2, 
           label = paste0("Correlation: ", round(correlation, 2)), 
           hjust = 0, vjust = 1.1, size = 5, color = "blue") +
  theme_minimal() +
  theme( plot.title = element_text(hjust = 0.5)) +
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  scale_y_continuous(limits = c(-1.3, 1.2))
plot
ggsave("./box_burst_frequency_cpg_islands_per_motif_strongest.svg", plot, width = 9, height = 6)




annotated_peaks = as.data.frame(read.delim("./resources/annotated_peaks_hg38_right.tsv"))
annotated_peaks_top10 = as.data.frame(read.delim("./resources/annotated_peaks_hg38_right_top10.tsv"))


create_ebox_plots_top_peak = function(lfc_data, annotated_peaks, lfc_data_name, annotated_peaks_name, fill_color, to_compare) {
  merged = merge(lfc_data, annotated_peaks, by="gene_id", all.x = TRUE)
  merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()
  
  
  unique_genes = merged[!duplicated(merged$gene_id), ]
  
  unique_genes$ebox_exists = ifelse(!is.na(unique_genes$sum_eboxes), ifelse(unique_genes$sum_eboxes > 0, "1+ E-boxes", "No E-boxes"), "No Assigned Peak")
  unique_genes$ebox_exists = factor(unique_genes$ebox_exists, levels = c("No Assigned Peak", "No E-boxes", "1+ E-boxes"))
  
  counts_eboxes = sort(unique(unique_genes$sum_eboxes))
  unique_genes$sum_eboxes = ifelse(is.na(unique_genes$sum_eboxes), "No Assigned Peak", unique_genes$sum_eboxes)
  unique_genes$sum_eboxes = factor(unique_genes$sum_eboxes, levels = c("No Assigned Peak", as.character(counts_eboxes)))
  
  d = unique_genes
  
  d = d %>% filter(abs(!!sym(to_compare)) <= 5)
  
  create_plot = function(category, label) {
    
    category_levels <- sort(unique(d[[category]]))
    
    # ensure category has string type
    if (!is.factor(d[[category]])) {
      d[[category]] <- factor(d[[category]], levels = as.character(category_levels))
    }
    
    counts <- d %>%
      group_by(!!sym(category)) %>%  # Use !!sym for dynamic column names
      summarise(n = paste("n=", n(), sep = ""))
    
    sequential_comparisons <- list()
    
    for (i in 1:(length(category_levels) - 1)) {
      sequential_comparisons[[i]] <- c(as.character(category_levels[i]), as.character(category_levels[i + 1]))
    }
    
    correlation <- cor(as.numeric(d[[category]]), as.numeric(d[[to_compare]]), method = "spearman", use = "pairwise.complete.obs")
    
    y_max = max(d[[to_compare]], na.rm = TRUE)
    p_val_pos = rep(y_max + 0.3 , length(sequential_comparisons))
    
    y_min = min(d[[to_compare]], na.rm = TRUE)
    
    # Generate the boxplot
    ggplot(d, aes(x = !!sym(category), group = !!sym(category), y = !!sym(to_compare))) +  # Use !!sym to dynamically reference columns
      geom_boxplot(fill = fill_color) +
      labs(title = paste("LFC of", lfc_data_name, "data for", label, "from strongest of", annotated_peaks_name, "peaks" ),
           x = label,
           y = paste("-log2(Fold Change", to_compare,  "MYC vs. MYC Depletion)")) +
      
      # Add counts as text labels
      geom_text(
        data = counts,  # Use counts summarised above
        aes(x = !!sym(category), y = y_min - 0.5, label = n),  # Place text above the boxes
        vjust = -1, size = 4
      ) +
      annotate("text", x = 1, y = y_max + 1.5, 
               label = paste0("Correlation: ", round(correlation, 2)), 
               hjust = 0, vjust = 0, size = 5, color = "blue") +
      
      # Add p-values using wilcox.test
      # throws errors in many cases
      stat_compare_means(method = "wilcox.test", label = "p.signif", comparisons = sequential_comparisons, hide.ns = FALSE, label.y = p_val_pos) +
      theme_minimal()+
      theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", linewidth = 1),)
  }
  
  list(create_plot("ebox_exists", "Existence of E-boxes"), create_plot("sum_eboxes", "Number of E-boxes"))
}

plots = create_ebox_plots_top_peak(lfc_data, annotated_peaks, "intronic TC Talk-seq", "all", alpha("lightblue", 0.7), "burst_size")
ggsave("./box_existence_eboxes_burst_size_strongest_intronic_TC_talk.svg", plots[[1]], width = 9, height = 6)
ggsave("./box_number_eboxes_burst_size_strongest_intronic_tc_talk.svg", plots[[2]], width = 9, height = 6)
plots = create_ebox_plots_top_peak(lfc_data, annotated_peaks_top10, "intronic TC Talk-seq", "top 10% of", alpha("deepskyblue3", 0.5), "burst_size")
ggsave("./box_existence_eboxes_burst_size_strongest_intronic_TC_talk_top10.svg", plots[[1]], width = 9, height = 6)
ggsave("./box_number_eboxes_burst_size_strongest_intronic_tc_talk_top10.svg", plots[[2]], width = 9, height = 6)

plots = create_ebox_plots_top_peak(lfc_data, annotated_peaks, "intronic TC Talk-seq", "all", alpha("lightblue", 0.7), "burst_frequency")
ggsave("./box_existence_eboxes_burst_frequency_strongest_intronic_TC_talk.svg", plots[[1]], width = 9, height = 6)
ggsave("./box_number_eboxes_burst_frequency_strongest_intronic_tc_talk.svg", plots[[2]], width = 9, height = 6)
plots = create_ebox_plots_top_peak(lfc_data, annotated_peaks_top10, "intronic TC Talk-seq", "top 10% of", alpha("deepskyblue3", 0.5), "burst_frequency")
ggsave("./box_existence_eboxes_burst_frequency_strongest_intronic_TC_talk_top10.svg", plots[[1]], width = 9, height = 6)
ggsave("./box_number_eboxes_burst_frequency_strongest_intronic_tc_talk_top10.svg", plots[[2]], width = 9, height = 6)



annotated_peaks_top10 = as.data.frame(read.delim("./resources/annotated_peaks_hg38_right_top10.tsv"))

create_cpg_island_figures = function(lfc_data, annotated_peaks, lfc_data_name, annotated_peaks_name, fill_color, to_compare) {
  
  merged = merge(lfc_data, annotated_peaks, by="gene_id", all.x = TRUE)
  merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()
  
  correlation <- cor(merged$CpG_island, merged[[to_compare]], method = "spearman", use = "complete.obs")
  
  merged$CpG_island = ifelse(is.na(merged$CpG_island), "No Assigned Peak", as.character(merged$CpG_island))
  merged$CpG_island = factor(merged$CpG_island, levels = c("No Assigned Peak", "FALSE", "TRUE"))
  
  comparisons = list(c("No Assigned Peak", "FALSE"), c("FALSE", "TRUE"))
  
  counts <- merged %>%
    group_by(CpG_island) %>%
    summarise(n = paste("n=", n(), sep = ""))
  
  y_min = min(merged[[to_compare]], na.rm = TRUE)
  y_max = max(merged[[to_compare]], na.rm = TRUE)
  p_val_pos = rep(y_max + 0.2, 2)
  
  y_min = -1
  y_max = 0.3
  p_val_pos = rep(y_max + 0.2, 2)
  ggplot(merged, aes(x = CpG_island, y = !!sym(to_compare))) +
    geom_boxplot(data = merged[merged$CpG_island == "No Assigned Peak", ], outliers = FALSE) +
    geom_boxplot(data = merged[merged$CpG_island != "No Assigned Peak", ], outliers = FALSE) +
    labs(title = paste("LFC of", lfc_data_name, "data for CpG islands vs non CpG islands from strongest of", annotated_peaks_name, "peaks"),
         x = "CpG island",
         y = paste("-log2(Fold Change", to_compare, "MYC vs. MYC Depletion)")) +
    geom_text(
      data = counts,
      aes(x = CpG_island, y = y_min - 0.5, label = n), # Fixed y position
      size = 4
    )+
    annotate("text", x = 1, y = y_max + 1, 
             label = paste0("Correlation: ", round(correlation, 2)), 
             hjust = 0, vjust = 0, size = 5, color = "blue") +
    theme_minimal()+ 
    theme(legend.position = "none")+
    theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
    stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparisons, hide.ns = FALSE, label.y = p_val_pos, tip.length = 0.01)
}

plot = create_cpg_island_figures(lfc_data, annotated_peaks, "intronic TC Talk-seq", "all", alpha("lightblue", 0.7), "burst_size")
ggsave("./box_burst_size_cpg_islands_intronic_tc_talk.svg", plot, width = 9, height = 6)
plot = create_cpg_island_figures(lfc_data, annotated_peaks_top10, "intronic TC Talk-seq", "top 10% of", alpha("deepskyblue3", 0.5), "burst_size")
ggsave("./box_burst_size_cpg_islands_intronic_tc_talk_top10.svg", plot, width = 9, height = 6)

plot = create_cpg_island_figures(lfc_data, annotated_peaks, "intronic TC Talk-seq", "all", alpha("lightblue", 0.7), "burst_frequency")
ggsave("./box_burst_frequency_cpg_islands_intronic_tc_talk.svg", plot, width = 9, height = 6)
plot = create_cpg_island_figures(lfc_data, annotated_peaks_top10, "intronic TC Talk-seq", "top 10% of", alpha("deepskyblue3", 0.5), "burst_frequency")
ggsave("./box_burst_frequency_cpg_islands_intronic_tc_talk_top10.svg", plot, width = 9, height = 6)



filtered_peaks = annotated_peaks

filtered_peaks$category = ifelse(filtered_peaks$CpG_island & filtered_peaks$TATA > 0, "CpG Island & TATA", ifelse(filtered_peaks$CpG_island, "CpG Island", ifelse(filtered_peaks$TATA > 0, "TATA", "No CpG Island & No TATA")))
filtered_peaks$category = factor(filtered_peaks$category, levels = c("No CpG Island & No TATA", "TATA", "CpG Island", "CpG Island & TATA"))

merged = merge(lfc_data, filtered_peaks, by="gene_id", all.x = TRUE)
merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()

merged$category = ifelse(is.na(merged$category), "No Assigned Peak", as.character(merged$category))
merged$category = factor(merged$category, levels = c("No Assigned Peak", "No CpG Island & No TATA", "TATA", "CpG Island", "CpG Island & TATA"))

comparison = list(c("No CpG Island & No TATA", "TATA"),  c("TATA", "CpG Island"), c("No CpG Island & No TATA", "CpG Island"), c("CpG Island", "CpG Island & TATA"))
comparison = append(list(c("No Assigned Peak", "No CpG Island & No TATA")), comparison)

counts <- merged %>%
  group_by(category) %>%
  summarise(n = paste("n=", n(), sep = ""))

plot = ggplot(merged, aes(x = category, y = burst_size)) +
  geom_boxplot(aes(color = "white"),outliers = FALSE) +
  geom_jitter(data=merged[merged$category != "No Assigned Peak",], aes(color = category), width = 0.2, alpha = 0.1) +
  geom_boxplot(aes(color = category),outliers = FALSE) +
  scale_color_manual(values = c("grey", "black", "brown3", "skyblue", "violet", "white")) +
  labs(title = "Burst size of intronic TC Talk-seq data for MYC peaks for Combinations of TATA-boxes and CpG Islands",
       x = "Category",
       y = "-log2(Fold Change Burst Size MYC vs. MYC Depletion)") +
  geom_text(
    data = counts,
    aes(x = category, y = -1.5, label = n), # Fixed y position
    size = 4
  )+
  guides(color = "none") +
  scale_y_continuous(limits = c(-1.5, 2)) +
  theme_minimal()+ 
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparison, hide.ns = FALSE, label.y = c(0.2, 0.2, 0.2, 0.5, 0.2), tip.length = 0.02)

plot
ggsave("./box_burst_size_combinations_CpG_TATA.svg", plot, width = 9, height = 6)


plot = ggplot(merged, aes(x = category, y = burst_frequency)) +
  geom_boxplot(aes(color = "white"),outliers = FALSE) +
  geom_jitter(data=merged[merged$category != "No Assigned Peak",], aes(color = category), width = 0.2, alpha = 0.1) +
  geom_boxplot(aes(color = category),outliers = FALSE) +
  scale_color_manual(values = c("grey", "black", "brown3", "skyblue", "violet", "white")) +
  labs(title = "Burst frequency of intronic TC Talk-seq data for MYC peaks for Combinations of TATA-boxes and CpG Islands",
       x = "Category",
       y = "-log2(Fold Change Burst Frequency MYC vs. MYC Depletion)") +
  geom_text(
    data = counts,
    aes(x = category, y = -1.5, label = n), # Fixed y position
    size = 4
  )+
  guides(color = "none") +
  scale_y_continuous(limits = c(-1.5, 2)) +
  theme_minimal()+ 
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparison, hide.ns = FALSE, label.y = c(0.3, 0.3, 0.3, 0.6, 0.3), tip.length = 0.02)

plot
ggsave("./box_burst_frequency_combinations_CpG_TATA.svg", plot, width = 9, height = 6)




data = as.data.frame(read.delim("./resources/annotated_peaks_hg38_right.tsv"))
filtered_data = as.data.frame(read.delim("./resources/annotated_peaks_hg38_right_top10.tsv"))


create_figures = function(lfc_data, peak_data, lfc_data_name, peak_data_name, fill_color, to_compare) {
  library(dplyr)
  library(tidyr)
  library(ggplot2)
  
  lfc_data$logPadj_capped = pmin(lfc_data$logPadj, 100)
  
  gene_counts = table(peak_data$gene_id)
  
  lfc_data$peak_count_with_na = as.numeric(gene_counts[match(lfc_data$gene_id, names(gene_counts))])
  
  lfc_data$peak_count = as.numeric(gene_counts[match(lfc_data$gene_id, names(gene_counts))])
  lfc_data$peak_count[is.na(lfc_data$peak_count)] = 0
  
  lfc_data$peak_category <- ifelse(lfc_data$peak_count == 0, "No Annotated Peaks", "1+ peaks")
  
  toUse = lfc_data
  
  toUse[[to_compare]] = pmin(toUse[[to_compare]], 4)
  toUse[[to_compare]] = pmax(toUse[[to_compare]], -4)
  
  toUse = toUse %>% distinct(gene_id, .keep_all = TRUE)
  
  library(dplyr)
  library(tidyr)
  
  # Define the thresholds
  logPadj_threshold <- 1.3
  log2FoldChange_threshold <- 1
  
  x_range <- range(toUse$log2FoldChange, na.rm = TRUE)
  y_range <- range(toUse$logPadj_capped, na.rm = TRUE)
  
  
  library(dplyr)
  # Rename categories
  toUse = toUse %>%
    mutate(peak_category = ifelse(peak_category == "No Annotated Peaks", "No Peaks", "1+ Peaks"))
  
  toUse$peak_category <- factor(toUse$peak_category, levels = c("No Peaks", "1+ Peaks"))
  
  counts <- toUse %>%
    group_by(peak_category) %>%
    summarise(n = paste("n=", n(), sep = ""))
  
  library(ggpubr)
  
  y_min = min(toUse[[to_compare]], na.rm = TRUE)
  
  p2 = ggplot(toUse, aes(x = peak_category, y = !!sym(to_compare))) +
    geom_boxplot(fill = fill_color) +
    labs(x = "Peak Category", y = paste("-log2(Fold Change", to_compare, "MYC vs. MYC Depletion) (capped at ±4)")) +
    ggtitle(paste(peak_data_name, "MYC Binding Sites for", lfc_data_name, "data")) +
    theme_minimal() +
    theme(plot.title = element_text(hjust = 0.5)) +
    
    # Add counts as text labels
    geom_text(
      data = counts,  # Use counts summarised above
      aes(x = peak_category, y = y_min - 0.5, label = n),  # Place text above the boxes
      size = 4
    ) +
    
    theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", linewidth = 1),) +
    stat_compare_means(method = "wilcox.test", label = "p.signif", comparisons = list(c("No Peaks", "1+ Peaks")), hide.ns = FALSE) +
    scale_y_continuous(breaks = seq(-4, 4, 1))
  
  
  
  category_levels <- sort(unique(toUse$peak_count))
  
  # ensure category has string type
  toUse$peak_count <- factor(toUse$peak_count, levels = as.character(category_levels))
  
  counts <- toUse %>%
    group_by(peak_count) %>%
    summarise(n = paste("n=", n(), sep = ""))
  
  sequential_comparisons <- list()
  
  for (i in 1:(length(category_levels) - 1)) {
    sequential_comparisons[[i]] <- c(as.character(category_levels[i]), as.character(category_levels[i + 1]))
  }
  
  library(ggpubr)
  
  correlation <- cor(as.numeric(toUse$peak_count), toUse[[to_compare]], method = "spearman", use = "pairwise.complete.obs")
  
  #y_min = min(toUse[[to_compare]], na.rm = TRUE)
  #y_max = max(toUse[[to_compare]], na.rm = TRUE)
  y_min = -1
  y_max = 1
  
  # create the right number of positions
  p_val_pos = rep(y_max + 0.5, length(sequential_comparisons)) 
  
  p3 = ggplot(toUse, aes(x = peak_count, group = peak_count, y = !!sym(to_compare))) +
    geom_boxplot(fill = fill_color, outliers = F) +
    labs(x = "Number of Peaks", y = paste("-log2(Fold Change", to_compare, "MYC vs. MYC Depletion)")) +
    ggtitle(paste(peak_data_name, "MYC Binding Sites for", to_compare, "in", lfc_data_name, "data")) +
    theme_minimal() +
    theme(plot.title = element_text(hjust = 0.1)) +
    
    # Add counts as text labels
    geom_text(
      data = counts,  # Use counts summarised above
      aes(x = peak_count, y = y_min - 0.5, label = n),  # Place text above the boxes
      size = 4
    ) +
    annotate("text", x = 1, y = y_max + 1.5, 
             label = paste0("Correlation: ", round(correlation, 2)), 
             hjust = 0, vjust = 0, size = 5, color = "blue") +
    
    theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", linewidth = 1),) +
    stat_compare_means(method = "wilcox.test", label = "p.signif", comparisons = sequential_comparisons, hide.ns = FALSE, label.y = p_val_pos) +
    scale_y_continuous(breaks = seq(-4, 4, 1))
  
  
  list(p2, p3)
}

plots = create_figures(lfc_data, data, "intronic TC Talk-seq", "All", alpha("lightblue", 0.7), "burst_size")
ggsave("./box_burst_size_number_peaks_talk.svg", plots[[2]], width = 9, height = 6)

plots = create_figures(lfc_data, filtered_data, "intronic TC Talk-seq", "Top 10%", alpha("deepskyblue3", 0.5), "burst_size")
ggsave("./box_burst_size_number_peaks_talk_top10.svg", plots[[2]], width = 9, height = 6)


plots = create_figures(lfc_data, data, "intronic TC Talk-seq", "All", alpha("lightblue", 0.7), "burst_frequency")
ggsave("./box_burst_frequency_number_peaks_talk.svg", plots[[2]], width = 9, height = 6)

plots = create_figures(lfc_data, filtered_data, "intronic TC Talk-seq", "Top 10%", alpha("deepskyblue3", 0.5), "burst_frequency")
ggsave("./box_burst_frequency_number_peaks_talk_top10.svg", plots[[2]], width = 9, height = 6)

merged$bursting_diff = merged$burst_frequency - merged$burst_size

correlation = cor(merged$coverage_myc, merged$bursting_diff, method = "spearman", use = "complete.obs")

merged$bursting_diff = pmin(merged$bursting_diff, 5)
merged$bursting_diff = pmax(merged$bursting_diff, -5)

x_min = min(merged$bursting_diff, na.rm = TRUE)

y_min = min(merged$coverage_myc, na.rm = TRUE)
y_max = max(merged$coverage_myc, na.rm = TRUE)
y_range = y_max - y_min

merged$density = density2d(merged$bursting_diff, merged$coverage_myc)

plot = ggplot(merged, aes(x = bursting_diff, y = coverage_myc)) +
  
  # Plot the non-MYC signature genes first (background)
  geom_point(data = merged[merged$MYC_signature == "Other", ],
             aes(color = density), size = 1.5) +
  
  # Use a color gradient for density of "Other" genes
  scale_color_gradient(low = "lightblue", high = "darkblue", name = "Density (Other genes)") +
  
  new_scale_color() +
  
  # Plot the MYC signature genes on top with fixed red color and create a label for the legend
  geom_point(data = merged[merged$MYC_signature == "MYC_signature", ],
             aes(color = "MYC_signature"), size = 1.5) +
  scale_color_manual(values = c("MYC_signature" = "red"),
                     name = "") +
  theme_minimal() +
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", linewidth = 1),) +
  geom_smooth(method="lm", color = "blue") + 
  geom_smooth(data = merged[merged$MYC_signature == "MYC_signature", ], method="lm", color = "red") + 
  labs(x = "-log2(Fold Change Burst Frequency / Size MYC vs. MYC Depletion) (capped at ±5)", y = "Coverage MYC (strongest peak)") +
  annotate("text", x = x_min, y = y_max + 0.1 * y_range, 
           label = paste0("Correlation: ", round(correlation, 2)), 
           hjust = 0, vjust = 0, size = 5, color = "blue")
plot
ggsave("./scatter_bursting_diff_vs_strongest_coverage.pdf", plot, width = 9, height = 6)

promoter_coverages = as.data.frame(read.delim("./resources/promoter_coverages_hg_38.tsv"))

merged = merge(lfc_sc_data, promoter_coverages, by="gene_id")

merged$bursting_diff = merged$burst_frequency - merged$burst_size

correlation = cor(merged$coverage_myc, merged$bursting_diff, method = "spearman", use = "complete.obs")

merged$bursting_diff = pmin(merged$bursting_diff, 5)
merged$bursting_diff = pmax(merged$bursting_diff, -5)

merged$coverage_myc = pmin(merged$coverage_myc, 400)
merged$coverage_myc = pmax(merged$coverage_myc, -100)

x_min = min(merged$bursting_diff, na.rm = TRUE)

y_min = min(merged$coverage_myc, na.rm = TRUE)
y_max = max(merged$coverage_myc, na.rm = TRUE)
y_range = y_max - y_min

merged$density = density2d(merged$bursting_diff, merged$coverage_myc)

plot = ggplot(merged, aes(x = bursting_diff, y = coverage_myc)) +
  
  # Plot the non-MYC signature genes first (background)
  geom_point(data = merged[merged$MYC_signature == "Other", ],
             aes(color = density), size = 1.5) +
  
  # Use a color gradient for density of "Other" genes
  scale_color_gradient(low = "lightblue", high = "darkblue", name = "Density (Other genes)") +
  
  new_scale_color() +
  
  # Plot the MYC signature genes on top with fixed red color and create a label for the legend
  geom_point(data = merged[merged$MYC_signature == "MYC_signature", ],
             aes(color = "MYC_signature"), size = 1.5) +
  scale_color_manual(values = c("MYC_signature" = "red"),
                     name = "") +
  theme_minimal() +
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", linewidth = 1),) +
  geom_smooth(method="lm", color = "blue") + 
  geom_smooth(data = merged[merged$MYC_signature == "MYC_signature", ], method="lm", color = "red") + 
  labs(x = "-log2(Fold Change Burst Frequency / Size MYC vs. MYC Depletion) (capped at ±5)", y = "Promoter Coverage MYC (capped at -50 & 400)") +
  annotate("text", x = x_min, y = y_max + 0.1 * y_range, 
           label = paste0("Correlation: ", round(correlation, 2)), 
           hjust = 0, vjust = 0, size = 5, color = "blue")
plot
ggsave("./scatter_bursting_diff_vs_promoter_coverage.pdf", plot, width = 9, height = 6)



bursting_data = as.data.frame(read.csv("./resources/bursting_all_genes.csv"))
annotated_peaks = as.data.frame(read.delim("./resources/annotated_peaks_hg38_right.tsv"))

#bursting_data = subset(bursting_data, bestmodel != "no change")

names(bursting_data)[names(bursting_data) == "km"] = "burst_frequency"
names(bursting_data)[names(bursting_data) == "B"] = "burst_size"


ebox_motifs <- c("CACGTG", "CACGCG", "CATGCG", "CATGTG", "CACGAG", "CACATG")

lfc_data = readRDS("./resources/processed_featureCounts_introns_TC_data.rds")
lfc_sc_data = readRDS("./resources/processed_grandR_single_cell_new_data.rds")

lfc_data = merge(lfc_data, bursting_data, by.x="gene_name", by.y="gene")
lfc_sc_data = merge(lfc_sc_data, bursting_data, by.x="gene_name", by.y="gene")

cor(lfc_data$burst_frequency + lfc_data$burst_size, lfc_data$log2FoldChange, method = "spearman", use = "pairwise.complete.obs")

merged = merge(lfc_sc_data, annotated_peaks, by="gene_id", all.x = TRUE)
merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()


merged$bursting_diff = merged$burst_frequency - merged$burst_size

correlation1 = cor(merged$CACGTG, merged$bursting_diff, method = "pearson", use = "complete.obs")
correlation2 = cor(merged$CACGTG, merged$bursting_diff, method = "spearman", use = "complete.obs")


merged$bursting_diff = pmin(merged$bursting_diff, 4)
merged$bursting_diff = pmax(merged$bursting_diff, -4)

x_min = min(merged$bursting_diff, na.rm = TRUE)

y_min = min(merged$CACGTG, na.rm = TRUE)
y_max = max(merged$CACGTG, na.rm = TRUE)
y_range = y_max - y_min

merged$CACGTG = pmin(merged$CACGTG, 2)

levels = c("0", "1", "2+")

merged$CACGTG = ifelse(is.na(merged$CACGTG), "No Assigned Peaks", as.character(merged$CACGTG))

merged$CACGTG = ifelse(merged$CACGTG == "2", "2+", merged$CACGTG)

merged$CACGTG = factor(merged$CACGTG, levels = c("No Assigned Peaks", levels))

comparisons = list(c("No Assigned Peaks", "0"), c("0", "1"), c("1", "2+"), c("0", "2+"))

color <- scales::hue_pal()(6)[1]


counts <- merged %>%
  group_by(CACGTG) %>%
  summarise(n = paste("n=", n(), sep = ""))

plot = ggplot(merged, aes(x = CACGTG, y = bursting_diff)) +
  geom_boxplot(data = merged[merged$CACGTG == "No Assigned Peaks",], outliers = FALSE, color = "grey") +
  geom_boxplot(data = merged[merged$CACGTG == "0",], outliers = FALSE, color = "black") +
  geom_jitter(data = merged[merged$CACGTG == "1" | merged$CACGTG == "2+",],  color = color, width = 0.2, alpha = 0.1) +
  geom_boxplot(data = merged[merged$CACGTG == "1" | merged$CACGTG == "2+",], outliers = FALSE, color = color) +
  theme_minimal() +
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", linewidth = 1),) +
  labs(y = "-log2(Fold Change Burst Frequency / Size MYC vs. MYC Depletion) (capped at ±4)", x = "CACGTG") +
  annotate("text", x = 1, y = 3, 
           label = paste0("r = ", round(correlation1, 2), ", p = ", round(correlation2, 2)), 
           hjust = 0, vjust = 0, size = 5, color = "blue") +
  geom_text(
    data = counts,
    aes(x = CACGTG, y = -2, label = n), # Fixed y position
    size = 4
  ) + 
  coord_cartesian(ylim = c(-2, 3)) +
  stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparisons, hide.ns = FALSE, label.y = c(2, 2, 2, 2.3)) 
plot
ggsave("./box_bursting_diff_CACGTG.pdf", plot, width = 9, height = 6)
ggsave("./box_bursting_diff_CACGTG_8_6.pdf", plot, width = 8, height = 6)



group_by_motif = annotated_peaks

group_by_motif$motif = ebox_motifs[apply(group_by_motif[, ebox_motifs], 1, which.max)]
group_by_motif$motif = ifelse(group_by_motif$sum_eboxes > 1, "2+ E-boxes", ifelse(group_by_motif$sum_eboxes == 0, "No E-box", group_by_motif$motif))

group_by_motif$motif = factor(group_by_motif$motif, levels = c("No E-box", ebox_motifs, "2+ E-boxes"))

merged = merge(lfc_sc_data, group_by_motif, by="gene_id", all.x = TRUE)
merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()

merged$bursting_diff = merged$burst_frequency - merged$burst_size


merged$sum_eboxes = ifelse(is.na(merged$sum_eboxes), -1, merged$sum_eboxes)
merged[merged$sum_eboxes == -1, ebox_motifs] = -1

merged <- merged %>%
  mutate(
    motif = case_when(
      sum_eboxes == -1 ~ "No Annotated Peaks",
      sum_eboxes == 0 ~ "No E-box",
      sum_eboxes == 1 ~ ebox_motifs[apply(.[, ebox_motifs], 1, which.max)],
      TRUE ~ "2+ E-boxes",
    )
  )


ebox_motifs = c("CACGTG", "CACGCG", "CATGCG", "CATGTG", "CACGAG", "CACATG")

merged$motif = factor(merged$motif, levels = c("No Annotated Peaks", "No E-box", ebox_motifs, "2+ E-boxes"))


counts <- merged %>%
  group_by(motif) %>%
  summarise(n = paste("n=", n(), sep = ""))

comparison <- list(c("No E-box", "CACGTG"), c("No E-box", "CACGCG"), c("No E-box", "CATGCG"), c("No E-box", "CATGTG"), c("No E-box", "CACGAG"), c("No E-box", "CACATG"), c("No E-box", "2+ E-boxes"))

correlation = cor(merged$sum_eboxes, merged$bursting_diff, method = "spearman", use = "pairwise.complete.obs")

median_baseline = median(merged[merged$motif == "No E-box", ]$bursting_diff)

plot = ggplot(merged, aes(x = motif, y = bursting_diff)) + 
  geom_boxplot(data = merged[merged$motif == "No Annotated Peaks", ],outliers = FALSE, color = "grey") +
  geom_boxplot(data = merged[merged$motif == "No E-box", ],outliers = FALSE) +
  geom_jitter(data = merged[as.character(merged$motif) %in% ebox_motifs, ], aes(color = motif), width = 0.2, alpha = 0.1) +
  geom_jitter(data = merged[merged$motif == "2+ E-boxes", ], width = 0.2, alpha = 0.1, color="chocolate3") +
  geom_hline(yintercept = median_baseline, linetype = "dashed", color = "black") +
  geom_boxplot(data = merged[merged$motif == "No Annotated Peaks", ],outliers = FALSE, color = "grey") +
  geom_boxplot(data = merged[as.character(merged$motif) %in% ebox_motifs, ],outliers = FALSE, aes(color = motif)) +
  geom_boxplot(data = merged[merged$motif == "2+ E-boxes", ],outliers = FALSE, color = "chocolate3") +
  labs(#title = "LFC Across Genes for strongest Peaks highlighting 1 E-box Motif",
    x = NULL,
    y = "-log2(Fold Change Burst Frequency / Size MYC vs. MYC Depletion) (capped at ±4)") +
  geom_text(
    data = counts,
    aes(x = motif, y = -2, label = n), # Fixed y position
    size = 4
  )+
  stat_compare_means(data = merged, aes(x = motif, y = bursting_diff), method="wilcox.test", label = "p.signif", comparisons = comparison, hide.ns = FALSE, label.y = seq(0.2, 5.3, 0.3), tip.length = 0.002)  +
  theme(panel.grid = element_blank())+
  guides(color = "none") +
  theme_minimal()+
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  scale_y_continuous(breaks = seq(-1, 4, 1), limits = c(-2, 4.3))

plot
ggsave("./box_bursting_diff_ebox_motifs.pdf", plot, width = 9, height = 6)
ggsave("./box_bursting_diff_ebox_motifs_10_6.pdf", plot, width = 10, height = 6)


levels = sort(unique(merged$sum_eboxes))

merged$sum_eboxes = factor(merged$sum_eboxes, levels = as.character(levels))

comparison = list(c("0", "1"), c("0", "2"), c("0", "3"), c("0", "4"))

ggplot(merged, aes(x = sum_eboxes, y = bursting_diff, group = sum_eboxes)) + 
  geom_boxplot(outliers = F) +
  labs(title = "Number of E-boxes in MYC Peaks",
       x = "E-box Motif",
       y = "Number of E-boxes") +
  theme_minimal() +
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),)+
  stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparison, hide.ns = FALSE, label.y = seq(0.2, 5.3, 0.3), tip.length = 0.002) 
  
