# SPDX-FileCopyrightText: 2025 Julius Deynet <julius.deynet@stud-mail.uni-wuerzburg.de>
#
# SPDX-License-Identifier: GPL-3.0-or-later

library(GenomicRanges)
library(Rsamtools)
library(rtracklayer)
library(ggplot2)

# Load gene annotation
genes <- readRDS("./resources/processed_featureCounts_introns_TC_data.rds")

annotated_peaks = as.data.frame(read.delim("./resources/annotated_peaks_hg38_right.tsv"))

gene_counts = table(annotated_peaks$gene_id)

genes$peak_count = as.numeric(gene_counts[match(genes$gene_id, names(gene_counts))])
genes$peak_count[is.na(genes$peak_count)] = 0

# Determine TSS positions
genes$TSS <- ifelse(genes$strand == "+", genes$start_position, genes$end_position)

filtered_genes = genes

regions <- GRanges(seqnames = filtered_genes$chromosome, 
                   ranges = IRanges(start = filtered_genes$TSS - 3000, end = filtered_genes$TSS + 3000))

# Load BAM file
bamfile <- BamFile("bams/MYC.DMSO.bam")
coverage <- coverage(bamfile)

bamfile <- BamFile("bams/Input.merged.bam")
coverage_input <- coverage(bamfile)

# Extract coverage data
coverage_list <- vector("list", length(regions))

for (i in seq_along(regions)) {
  min_pos <- min(start(coverage[[as.character(seqnames(regions[i]))]]))
  max_pos <- max(end(coverage[[as.character(seqnames(regions[i]))]]))
  
  if (start(regions[i]) >= min_pos &
      end(regions[i]) <= max_pos) {
  
  temp_cov <- coverage[[as.character(seqnames(regions[i]))]][start(regions[i]):end(regions[i])]
  temp_cov <- as.numeric(temp_cov)
  
  temp_cov_ctr <- coverage_input[[as.character(seqnames(regions[i]))]][start(regions[i]):end(regions[i])]
  temp_cov_ctr <- as.numeric(temp_cov_ctr)
  
  temp_cov <- temp_cov - (temp_cov_ctr * 940008/1512647)
  
  # Reverse the coverage if the gene is on the negative strand
  if (filtered_genes$strand[i] == "-") {
    temp_cov <- rev(temp_cov)
  }
  
  coverage_list[[i]] <- temp_cov
  
  }
}

# Convert to matrix
coverage_matrix <- do.call(rbind, coverage_list)

# Compute average coverage per position
average_coverage <- colMeans(coverage_matrix, na.rm = TRUE)

# compute upstream average coverage
sum(colMeans(coverage_matrix[,1:3000], na.rm = TRUE))

# compute downstream average coverage
sum(colMeans(coverage_matrix[,3001:6000], na.rm = TRUE))

# Create a vector for positions relative to TSS
positions <- -3000:3000

plot = ggplot(data = data.frame(positions, average_coverage), aes(x = positions, y = average_coverage)) +
  geom_line() +
  xlab("Distance to TSS") +
  ylab("Average Coverage MYC") +
  #ggtitle("Average ChIP-seq Coverage around TSS") +
  theme_minimal() +
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  scale_x_continuous(breaks = seq(-3000, 3000, 1000)) +
  geom_area(fill = "skyblue", alpha = 0.7)+
  annotate(
    "segment",
    x = -3000, xend = 3000,  # Adjust start and end positions as needed
    y = -0.5, yend = -0.5,  # Adjust y position
    arrow = arrow(length = unit(0.2, "inches")),
    color = "black",
    size = 0.5
  )
plot
ggsave("./distanceToTSS_coverage_all_peaks.pdf", plot, width = 9, height = 4)

