# SPDX-FileCopyrightText: 2025 Julius Deynet <julius.deynet@stud-mail.uni-wuerzburg.de>
#
# SPDX-License-Identifier: GPL-3.0-or-later

library(GenomicRanges)
library(rtracklayer)
library(Biostrings)
library(reshape2)
library(ggpubr)
library(dplyr)
library(ggplot2)
library(scales)

annotated_peaks = as.data.frame(read.delim("./resources/annotated_peaks_hg38_right.tsv"))

unique_peaks = annotated_peaks[!duplicated(annotated_peaks$peak_id), ]
ebox_motifs <- c("CACGTG", "CACGCG", "CATGCG", "CATGTG", "CACGAG", "CACATG")

# # Replace 'your_file.fastq' with the actual path to your FASTQ file
# fastq_file <- "/ChIPseq_A375_Input_merged.fastq.gz"
# 
# # Read FASTQ sequences
# fastq_data <- readDNAStringSet(fastq_file, format="fastq")
# 
# # Count nucleotide occurrences
# nuc_counts <- alphabetFrequency(fastq_data, baseOnly=TRUE)[, c("A", "C", "G", "T")]
# 
# # Compute nucleotide frequencies
# total_bases <- sum(nuc_counts)
# nuc_counts = colSums(nuc_counts)
# freqs <- nuc_counts / total_bases

freqs = c(0.2910, 0.2144, 0.2201, 0.2745 )
names(freqs) <- c("A", "C", "G", "T")




# Function to compute probability of a motif based on nucleotide frequencies
compute_motif_prob_wild <- function(motif, freqs) {
  prod(sapply(strsplit(motif, "")[[1]], function(base) {
    if(base == "N") 1 else freqs[base]
  }))
}

# Compute probabilities for each motif
motif_probs <- sapply(c(ebox_motifs, "CANNTG"), compute_motif_prob_wild, freqs = freqs)

# Parameters
num_windows <- nrow(unique_peaks)
window_size <- 500
motif_length <- 6
num_positions <- window_size - motif_length + 1  # Possible start positions per window

# Compute expected occurrences for each motif
expected_counts <- num_positions * motif_probs * num_windows

# Print results
print(expected_counts)

temp = colSums(unique_peaks[, c("sum_eboxes_and_CANNTG_total", "CACGCG", "CATGCG", "CACGAG")])
counted_canntg = as.numeric(temp[1] - temp[2] - temp[3] - temp[4])


library(UpSetR)
library(RColorBrewer)

lfc_data = readRDS("./resources/processed_featureCounts_introns_TC_data.rds")

merged = merge(lfc_data, annotated_peaks, by="gene_id")
merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()

unique_peaks = merged[!duplicated(merged$gene_id), ]


upset_data <- unique_peaks[, ebox_motifs]

binary_matrix <- as.data.frame(lapply(upset_data, function(x) ifelse(x > 0, 1, 0)))

ggplot_colors <- c(scales::hue_pal()(length(ebox_motifs)))  # Get ggplot default palette

# sort motif_colors by column sums
ggplot_colors <- ggplot_colors[order(colSums(binary_matrix), decreasing = TRUE)]

# Generate UpSet plot with ggplot default-like colors
upset(as.data.frame(binary_matrix), 
      sets = colnames(binary_matrix), 
      nsets = 6, 
      sets.bar.color = ggplot_colors)
plot = upset(as.data.frame(binary_matrix), 
      sets = colnames(binary_matrix), 
      nsets = 6, 
      sets.bar.color = ggplot_colors,
      nintersects = 21)


plot
#save upset plot
svg("./upset_ebox_motifs_top15.svg", width = 9, height = 4)
plot
dev.off()




group_by_motif = annotated_peaks

group_by_motif$motif = ebox_motifs[apply(group_by_motif[, ebox_motifs], 1, which.max)]
group_by_motif$motif = ifelse(group_by_motif$sum_eboxes > 1, "2+ E-boxes", ifelse(group_by_motif$sum_eboxes == 0, "No E-box", group_by_motif$motif))

group_by_motif$motif = factor(group_by_motif$motif, levels = c("No E-box", ebox_motifs, "2+ E-boxes"))

counts <- group_by_motif %>%
  group_by(motif) %>%
  summarise(n = paste("n=", n(), sep = ""))

cor(annotated_peaks$sum_eboxes, annotated_peaks$coverage_myc, method = "spearman", use = "pairwise.complete.obs")

comparisons <- list(c("No E-box", "CACGTG"), c("No E-box", "CACGCG"), c("No E-box", "CATGCG"), c("No E-box", "CATGTG"), c("No E-box", "CACGAG"), c("No E-box", "CACATG"))

median_baseline = median(group_by_motif[group_by_motif$motif == "No E-box", ]$coverage_myc)

plot = ggplot(group_by_motif, aes(x = motif, y = coverage_myc)) +
  geom_boxplot(data = group_by_motif[group_by_motif$motif == "No E-box", ],outliers = FALSE) +
  geom_jitter(data = group_by_motif[as.character(group_by_motif$motif) %in% ebox_motifs, ], aes(color = motif), width = 0.2, alpha = 0.1) +
  geom_jitter(data = group_by_motif[group_by_motif$motif == "2+ E-boxes", ], width = 0.2, alpha = 0.1, color="chocolate3") +
  geom_hline(yintercept = median_baseline, linetype = "dashed", color = "black") +
  geom_boxplot(data = group_by_motif[as.character(group_by_motif$motif) %in% ebox_motifs, ],outliers = FALSE, aes(color = motif)) +
  geom_boxplot(data = group_by_motif[group_by_motif$motif == "2+ E-boxes", ],outliers = FALSE, color = "chocolate3") +
  labs(title = "Coverage_MYC Across Peaks highlighting 1 E-box Motif",
       x = "Motif",
       y = "Coverage_MYC") +
  geom_text(
    data = counts,
    aes(x = motif, y = -10, label = n), # Fixed y position
    size = 4
  )+ 
  guides(color = "none") +
  theme_minimal()+ 
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparisons, hide.ns = FALSE, label.y = seq(90, 280, 20)) +
  scale_y_continuous(breaks = seq(0, 300, 20), limits = c(-10, 220))

plot

ggsave("./box_coverage_ebox_per_motif.pdf", plot, width = 9, height = 6, device = svglite::svglite)


lfc_data = readRDS("./resources/processed_featureCounts_introns_TC_data.rds")

merged = merge(lfc_data, annotated_peaks, by="gene_id", all.x = TRUE)
merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()
cor(merged$coverage_myc, merged$log2FoldChange, method = "spearman", use = "pairwise.complete.obs")

merged$sum_eboxes = ifelse(is.na(merged$sum_eboxes), -1, merged$sum_eboxes)
merged[merged$sum_eboxes == -1, ebox_motifs] = -1

merged <- merged %>%
  mutate(
    motif = case_when(
      sum_eboxes == -1 ~ "No Peak",
      sum_eboxes == 0 ~ "No E-box",
      sum_eboxes == 1 ~ ebox_motifs[apply(.[, ebox_motifs], 1, which.max)],
      TRUE ~ "2+ E-boxes",
    )
  )




merged$motif = factor(merged$motif, levels = c("No Peak", "No E-box", ebox_motifs, "2+ E-boxes"))


subset = merged[merged$motif != "No Peak", ]

subset$motif = factor(subset$motif, levels = c("No E-box", ebox_motifs, "2+ E-boxes"))

counts <- subset %>%
  group_by(motif) %>%
  summarise(n = paste("n=", n(), sep = ""))

comparisons <- list(c("No E-box", "CACGTG"), c("No E-box", "CACGCG"), c("No E-box", "CATGCG"), c("No E-box", "CATGTG"), c("No E-box", "CACGAG"), c("No E-box", "CACATG"))

correlation = cor(subset$sum_eboxes, subset$coverage_myc, method = "spearman", use = "pairwise.complete.obs")

median_baseline = median(subset[subset$motif == "No E-box", ]$coverage_myc)

plot = ggplot(subset, aes(x = motif, y = coverage_myc)) +
  geom_boxplot(data = subset[subset$motif == "No E-box", ],outliers = FALSE) +
  geom_jitter(data = subset[as.character(subset$motif) %in% ebox_motifs, ], aes(color = motif), width = 0.2, alpha = 0.1) +
  geom_jitter(data = subset[subset$motif == "2+ E-boxes", ], width = 0.2, alpha = 0.1, color="chocolate3") +
  geom_hline(yintercept = median_baseline, linetype = "dashed", color = "black") +
  geom_boxplot(data = subset[as.character(subset$motif) %in% ebox_motifs, ],outliers = FALSE, aes(color = motif)) +
  #scale_color_brewer(palette = "Set1") +
  geom_boxplot(data = subset[subset$motif == "2+ E-boxes", ],outliers = FALSE, color = "chocolate3") +
  labs(#title = "Coverage_MYC Across Strongest Peaks highlighting 1 E-box Motif",
       x = "Motif",
       y = "MYC Coverage") +
  geom_text(
    data = counts,
    aes(x = motif, y = -10, label = n), # Fixed y position
    size = 4
  )+ 
  guides(color = "none") +
  theme_minimal()+ 
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparisons, hide.ns = FALSE, label.y = seq(130, 290, 20)) +
  scale_y_continuous(breaks = seq(0, 300, 50)) +
  coord_cartesian(ylim = c(-10, 270))

plot

ggsave("./box_coverage_ebox_per_motif_strongest.pdf", plot, width = 9, height = 6)



counts <- merged %>%
  group_by(motif) %>%
  summarise(n = paste("n=", n(), sep = ""))

new_comparisons = append(comparisons, list(c("No Peak", "No single E-box"), c("No Peak", "No E-box")), after = 0)

correlation = cor(merged$sum_eboxes, merged$log2FoldChange, method = "spearman", use = "pairwise.complete.obs")

median_baseline = median(merged[merged$motif == "No E-box", ]$log2FoldChange)

plot = ggplot(merged, aes(x = motif, y = log2FoldChange)) + 
  geom_boxplot(data = merged[merged$motif == "No Peak", ],outliers = FALSE, color = "grey") +
  geom_boxplot(data = merged[merged$motif == "No E-box", ],outliers = FALSE) +
  geom_jitter(data = merged[as.character(merged$motif) %in% ebox_motifs, ], aes(color = motif), width = 0.2, alpha = 0.1) +
  geom_jitter(data = merged[merged$motif == "2+ E-boxes", ], width = 0.2, alpha = 0.1, color="chocolate3") +
  geom_hline(yintercept = median_baseline, linetype = "dashed", color = "black") +
  geom_boxplot(data = merged[merged$motif == "No Peak", ],outliers = FALSE, color = "grey") +
  geom_boxplot(data = merged[as.character(merged$motif) %in% ebox_motifs, ],outliers = FALSE, aes(color = motif)) +
  geom_boxplot(data = merged[merged$motif == "2+ E-boxes", ],outliers = FALSE, color = "chocolate3") +
  labs(#title = "LFC Across Genes for strongest Peaks highlighting 1 E-box Motif",
       x = "Motif",
       y = "-log2(Fold Change MYC vs. MYC Depletion)") +
  geom_text(
    data = counts,
    aes(x = motif, y = -1.5, label = n), # Fixed y position
    size = 4
  )+
  theme(panel.grid = element_blank())+
  guides(color = "none") +
  theme_minimal()+
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  stat_compare_means(method="wilcox.test", label = "p.signif", comparisons = comparisons, hide.ns = FALSE, label.y = seq(2.1, 5.3, 0.3))  +
  scale_y_continuous(breaks = seq(-1, 4, 1)) +
  coord_cartesian(ylim = c(-1.5, 4.3))

plot

ggsave("./box_lfc_ebox_per_motif.pdf", plot, width = 9, height = 6)


merged = merge(lfc_data, annotated_peaks, by="gene_id")


merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()


#d = annotated_peaks
d = merged
category = "sum_eboxes"

correlation1 <- cor(as.numeric(d$sum_eboxes), d$coverage_myc, method = "pearson", use = "pairwise.complete.obs")
correlation2 <- cor(as.numeric(d$sum_eboxes), d$coverage_myc, method = "spearman", use = "pairwise.complete.obs")

category_levels <- c("0", "1", "2", "3", "4+")

# ensure category has string type
d[[category]] <- factor(ifelse(d[[category]] >= 4, "4+", d[[category]]), levels = as.character(category_levels))

counts <- d %>%
  group_by(!!sym(category)) %>%  # Use !!sym for dynamic column names
  summarise(n = paste("n=", n(), sep = ""))

sequential_comparisons <- list()

for (i in 1:(length(category_levels) - 1)) {
  sequential_comparisons[[i]] <- c(as.character(category_levels[i]), as.character(category_levels[i + 1]))
}


#y_max = max(d$coverage_myc, na.rm = TRUE)
y_max = 170
p_val_pos = rep(y_max , length(sequential_comparisons))

# Generate the boxplot
plot = ggplot(d, aes(x = !!sym(category), group = !!sym(category), y = coverage_myc)) +  # Use !!sym to dynamically reference columns
  geom_boxplot(outliers = FALSE) +
  labs(#title = paste("Coverage_MYC for Number of E-boxes in strongest Peaks" ),
       x = "Number of E-boxes",
       y = "MYC Coverage") +
  
  # Add counts as text labels
  geom_text(
    data = counts,  # Use counts summarised above
    aes(x = !!sym(category), y = -25, label = n),  # Place text above the boxes
    vjust = -1, size = 4
  ) +
  annotate("text", x = 1, y = y_max + 45, 
           label = paste0("r = ", round(correlation1, 2), ", p = ", round(correlation2, 2)), 
           hjust = 0, vjust = 0, size = 5, color = "blue") +
  
  # Add p-values using wilcox.test
  # throws errors in many cases
  stat_compare_means(method = "wilcox.test", label = "p.signif", comparisons = sequential_comparisons, hide.ns = FALSE, label.y = p_val_pos) +
  theme_minimal() +
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", linewidth = 1),) +
  scale_y_continuous(breaks = seq(0, 250, 50))

plot

ggsave("./box_coverage_number_eboxes_strongest.pdf", plot, width = 9, height = 6)






annotated_peaks = as.data.frame(read.delim("./resources/annotated_peaks_hg38_right.tsv"))
annotated_peaks_top10 = as.data.frame(read.delim("./resources/annotated_peaks_hg38_right_top10.tsv"))

lfc_sc_data = readRDS("./resources/processed_grandR_single_cell_new_data.rds")
lfc_data = readRDS("./resources/processed_featureCounts_introns_TC_data.rds")

create_ebox_plots_top_peak = function(lfc_data, annotated_peaks, lfc_data_name, annotated_peaks_name, fill_color) {
  
  merged = merge(lfc_data, annotated_peaks, by="gene_id", all.x = TRUE)
  merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()
  
  
  unique_genes = merged[!duplicated(merged$gene_id), ]
  
  unique_genes$ebox_exists = ifelse(!is.na(unique_genes$sum_eboxes), ifelse(unique_genes$sum_eboxes > 0, "1+ E-boxes", "No E-boxes"), "No Assigned Peak")
  unique_genes$ebox_exists = factor(unique_genes$ebox_exists, levels = c("No Assigned Peak", "No E-boxes", "1+ E-boxes"))
  
  correlation1 <- cor(as.numeric(merged$sum_eboxes), merged$log2FoldChange, method = "pearson", use = "pairwise.complete.obs")
  correlation2 <- cor(as.numeric(merged$sum_eboxes), merged$log2FoldChange, method = "spearman", use = "pairwise.complete.obs")
  
  counts_eboxes = c("0", "1", "2", "3", "4+")
  unique_genes$sum_eboxes = ifelse(is.na(unique_genes$sum_eboxes), "No Assigned Peak", ifelse(unique_genes$sum_eboxes >= 4, "4+", as.character(unique_genes$sum_eboxes)))
  unique_genes$sum_eboxes = factor(unique_genes$sum_eboxes, levels = c("No Assigned Peak", as.character(counts_eboxes)))
  
  print(paste("Mean LFC with peaks:", mean(subset(unique_genes, ebox_exists != "No Assigned Peak")$log2FoldChange)))
  print(paste("Mean LFC strongest no E-box:", mean(subset(unique_genes, ebox_exists == "No E-boxes")$log2FoldChange)))
  print(paste("Mean LFC strongest 1+ E-boxes:", mean(subset(unique_genes, ebox_exists == "1+ E-boxes")$log2FoldChange)))
  print(paste("Difference:", mean(subset(unique_genes, ebox_exists == "1+ E-boxes")$log2FoldChange) - mean(subset(unique_genes, ebox_exists == "No E-boxes")$log2FoldChange)))
  
  d = unique_genes
  
  create_plot = function(category, label) {
    
    
    counts <- d %>%
      group_by(!!sym(category)) %>%  # Use !!sym for dynamic column names
      summarise(n = paste("n=", n(), sep = ""))
    
    category_levels <- c("No Assigned Peak", as.character(counts_eboxes))
    
    sequential_comparisons <- list()
    
    for (i in 1:(length(category_levels) - 1)) {
      sequential_comparisons[[i]] <- c(as.character(category_levels[i]), as.character(category_levels[i + 1]))
    }
    
    y_max = max(d$log2FoldChange, na.rm = TRUE)
    p_val_pos = rep(y_max + 0.3 , length(sequential_comparisons))
    
    y_min = min(d$log2FoldChange, na.rm = TRUE)
    
    # Generate the boxplot
    ggplot(d, aes(x = !!sym(category), group = !!sym(category), y = log2FoldChange)) +  # Use !!sym to dynamically reference columns
      geom_boxplot(fill = fill_color) +
      labs(title = lfc_data_name,
           x = label,
           y = "-log2(Fold Change MYC vs. MYC Depletion)") +
      
      # Add counts as text labels
      geom_text(
        data = counts,  # Use counts summarised above
        aes(x = !!sym(category), y = y_min - 1, label = n),  # Place text above the boxes
        vjust = -1, size = 4
      ) +
      annotate("text", x = 1, y = y_max + 1.5, 
               label = paste0("r = ", round(correlation1, 2), ", p = ", round(correlation2, 2)), 
               hjust = 0, vjust = 0, size = 5, color = "blue") +
      
      # Add p-values using wilcox.test
      # throws errors in many cases
      stat_compare_means(method = "wilcox.test", label = "p.signif", comparisons = sequential_comparisons, hide.ns = FALSE, label.y = p_val_pos) +
      theme_minimal()+
      scale_y_continuous(breaks = seq(-2, 4, 1)) +
      theme( plot.title = element_text(hjust = 0.5)) +
    theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", linewidth = 1),)
  }
  
  list(create_plot("ebox_exists", "Existence of E-boxes"),
       create_plot("sum_eboxes", "Number of E-boxes"))
}

plots = create_ebox_plots_top_peak(lfc_data, annotated_peaks, "intronic TC Talk-seq", "all", alpha("lightblue", 0.7))
ggsave("./box_existence_eboxes_strongest_peak_intronic_TC_talk.svg", plots[[1]], width = 9, height = 6)
ggsave("./box_number_eboxes_strongest_peak_intronic_tc_talk.pdf", plots[[2]], width = 9, height = 6)
plots = create_ebox_plots_top_peak(lfc_data, annotated_peaks_top10, "intronic TC Talk-seq", "top 10% of", alpha("deepskyblue3", 0.5))
ggsave("./box_existence_eboxes_strongest_peak_intronic_TC_talk_top10.svg", plots[[1]], width = 9, height = 6)
ggsave("./box_number_eboxes_strongest_peak_intronic_tc_talk_top10.svg", plots[[2]], width = 9, height = 6)
plots = create_ebox_plots_top_peak(lfc_sc_data, annotated_peaks, "new scSLAM-seq", "all", alpha("lightblue", 0.7))
ggsave("./box_existence_eboxes_strongest_peak_new_slam.svg", plots[[1]], width = 9, height = 6)
ggsave("./box_number_eboxes_strongest_peak_new_slam.pdf", plots[[2]], width = 9, height = 6)
plots = create_ebox_plots_top_peak(lfc_sc_data, annotated_peaks_top10, "new scSLAM-seq", "top 10% of", alpha("deepskyblue3", 0.5))
ggsave("./box_existence_eboxes_strongest_peak_new_slam_top10.svg", plots[[1]], width = 9, height = 6)
ggsave("./box_number_eboxes_strongest_peak_new_slam_top10.svg", plots[[2]], width = 9, height = 6)

