# SPDX-FileCopyrightText: 2025 Julius Deynet <julius.deynet@stud-mail.uni-wuerzburg.de>
#
# SPDX-License-Identifier: GPL-3.0-or-later

library(dplyr)

annotated_peaks = as.data.frame(read.delim("./resources/annotated_peaks_hg38_right.tsv"))
annotated_peaks_top10 = as.data.frame(read.delim("./resources/annotated_peaks_hg38_right_top10.tsv"))

lfc_data = readRDS("./resources/processed_featureCounts_introns_TC_data.rds")
merged = merge(lfc_data, annotated_peaks, by="gene_id")
merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()

merged = subset(merged, log2FoldChange >= 1)

# Count occurrences of each peak_id
peak_counts <- merged %>%
  dplyr::count(peak_id, name = "count")

# Print result
print(peak_counts)

unique(peak_counts$count)

nrow(subset(peak_counts, count == 2))

# for 100 random gene pairs, calculate the average difference in log2FoldChange
set.seed(123)
n = 10000
avg_diffs = numeric(n)
for (i in 1:n) {
  # Sample 2 rows
  genes = sample(unique(merged$gene_id), 2)
  # Calculate average difference in log2FoldChange
  avg_diffs[i] = abs(diff(subset(merged, gene_id %in% genes)$log2FoldChange))
}
mean(avg_diffs)


merged %>% group_by(peak_id) %>% filter(n() == 2) %>% summarise(avg_sd = sd(log2FoldChange, na.rm = TRUE)) %>% ungroup() %>% pull(avg_sd) %>% mean()

merged %>% 
  group_by(peak_id) %>% 
  filter(n() == 2) %>% 
  summarise(abs_diff = abs(diff(log2FoldChange))) %>% 
  ungroup() %>% 
  pull(abs_diff) %>% 
  mean()

merged %>% 
  group_by(peak_id) %>% 
  filter(n() == 2) %>% 
  filter(n_distinct(sign(distanceToTSS)) == 2) %>%
  summarise(abs_diff = abs(diff(log2FoldChange))) %>% 
  ungroup() %>% 
  pull(abs_diff) %>% 
  mean()

s = subset(merged, log2FoldChange >= 1)
s %>% 
  group_by(peak_id) %>% 
  filter(n() == 2) %>% 
  filter(n_distinct(sign(distanceToTSS)) == 2) %>%
  summarise(abs_diff = abs(diff(log2FoldChange))) %>% 
  ungroup() %>% 
  pull(abs_diff) %>% 
  mean()

temp = s %>% 
  group_by(peak_id) %>% 
  filter(n() == 2)


merged %>% group_by(peak_id) %>% filter(n() == 2) %>% summarise(same_strand = n_distinct(strand.y) == 1) %>% ungroup() %>% pull(same_strand) %>% table()
merged %>% group_by(peak_id) %>% filter(n() == 2) %>% summarise(same_strand = n_distinct(sign(distanceToTSS)) == 1) %>% ungroup() %>% pull(same_strand) %>% table()

temp = merged %>% 
  group_by(peak_id) %>% 
  filter(n() == 2) %>% 
  filter(n_distinct(sign(distanceToTSS)) == 1) 

subset = merged %>% group_by(peak_id) %>% filter(n() == 2)

mean(subset(subset, strand.y == "+")$log2FoldChange)
mean(subset(subset, strand.y == "-")$log2FoldChange)

mean(subset(merged, strand.y == "+")$log2FoldChange)
mean(subset(merged, strand.y == "-")$log2FoldChange)

subset %>% filter(distanceToTSS > 0) %>% pull(log2FoldChange) %>% mean()
subset %>% filter(distanceToTSS < 0) %>% pull(log2FoldChange) %>% mean()




a = merged %>% filter(distanceToTSS >= 0)
b = merged %>% filter(distanceToTSS < 0)

mean(a$log2FoldChange)
mean(b$log2FoldChange)
sd(a$log2FoldChange)
sd(b$log2FoldChange)

cor(a$log2FoldChange, a$distanceToTSS)
cor(b$log2FoldChange, b$distanceToTSS)

cor(a$log2FoldChange, a$coverage_myc)
cor(b$log2FoldChange, b$coverage_myc)

mean(a$coverage_myc)
mean(b$coverage_myc)



binned_data <- merged %>%
  mutate(bin = cut_width(distanceToTSS, width = 100, center = 0)) %>%
  group_by(bin) %>%
  summarise(mean_log2FC = mean(log2FoldChange), .groups = "drop")

# Convert bin levels to numeric midpoints for proper plotting
binned_data <- binned_data %>%
  mutate(bin_mid = as.numeric(sub("\\((.*),.*", "\\1", bin)) + 50)

merged$smoothed = predict(loess(log2FoldChange ~ distanceToTSS, data = merged, span = 0.3))

# average lfc per distance to TSS
matrix = merged %>% group_by(distanceToTSS) %>% summarise(mean_lfc = mean(log2FoldChange), .groups = "drop") %>% as.data.frame()


# Create smooth line plot
plot = ggplot(merged, aes(x = distanceToTSS, y = smoothed)) +
  geom_ribbon(aes(ymin = min(merged$smoothed), ymax = smoothed), fill = "skyblue", alpha = 0.7) +  # Ensure area follows the curve
  geom_line(color = "black") +
  scale_color_gradient(low = "blue", high = "darkblue") +
  theme_minimal() +
  labs(x = "Distance to TSS",
       y = "-log2(Fold Change MYC vs. MYC Depletion)") +
  scale_x_continuous(breaks = seq(-3000, 3000, by = 1000)) +
  theme(panel.grid = element_blank(), 
        axis.line = element_line(color = "black"), 
        axis.ticks = element_line(color = "black", size = 1),
        plot.title = element_text(hjust = 0.5)) +
  annotate(
    "segment",
    x = -3000, xend = 3000,  # Adjust start and end positions as needed
    y = -0.2, yend = -0.2,  # Adjust y position
    arrow = arrow(length = unit(0.2, "inches")),
    color = "black",
    size = 0.5
  )

plot

ggsave("./distance_vs_lfc_smoothed.pdf", plot, width = 9, height = 4)


a = subset(merged, distanceToTSS >= 0)
b = subset(merged, distanceToTSS < 0)
cor(a$coverage_myc, a$log2FoldChange, method = "spearman", use = "pairwise.complete.obs")
cor(b$coverage_myc, b$log2FoldChange, method = "spearman", use = "pairwise.complete.obs")


# Extract smoothed data
smoothed_data <- ggplot_build(plot)$data[[1]]

# Find peak (max y value)
smoothed_data %>%
  filter(y == max(y))
