# SPDX-FileCopyrightText: 2025 Julius Deynet <julius.deynet@stud-mail.uni-wuerzburg.de>
#
# SPDX-License-Identifier: GPL-3.0-or-later

input <- vroom::vroom(".hg38_chip_seq_bams/coverage_JH477_A375_MYC_promoter_peaks_Input.bed",
                      col_names = c(
                        "peak_chr", "peak_start", "peak_end", "peak_id", "peak_score", "chr_gene", "gene_tss_start",
                        "gene_tss_end", "ensembl_gene_id", "hgnc_symbol", "strand", "distanceToTSS",
                        "coverage_input"
                      )
)

ctr <- vroom::vroom(".hg38_chip_seq_bams/coverage_JH477_A375_MYC_promoter_peaks_MYC.bed",
                      col_names = c(
                        "peak_chr", "peak_start", "peak_end", "peak_id", "peak_score", "chr_gene", "gene_tss_start",
                        "gene_tss_end", "ensembl_gene_id", "hgnc_symbol", "strand", "distanceToTSS",
                        "coverage_ctr"
                      )
)


# Normalize input as done in Analyze.R by Markus
input$coverage_input = input$coverage_input * 940008/1512647



annotated_peaks = ctr
annotated_peaks$coverage_myc = ctr$coverage_ctr - input$coverage_input
annotated_peaks$coverage_myc_ratio = ctr$coverage_ctr / input$coverage_input
annotated_peaks$coverage_input = input$coverage_input

s = subset(annotated_peaks, coverage_input == 0)

cor(annotated_peaks$coverage_myc, annotated_peaks$peak_score, method = "spearman")
cor(annotated_peaks$coverage_myc_ratio, annotated_peaks$peak_score, method = "spearman")

annotated_peaks$hgnc_symbol = NULL
annotated_peaks <- annotated_peaks %>% filter(grepl("^[0-9XY]+$", peak_chr))
annotated_peaks$peak_chr = paste0("chr", annotated_peaks$peak_chr)
names(annotated_peaks)[names(annotated_peaks) == "ensembl_gene_id"] = "gene_id"
names(annotated_peaks)[names(annotated_peaks) == "distance_peak_to_tss"] = "distanceToTSS"
annotated_peaks$peak_start  = annotated_peaks$peak_start + 500
annotated_peaks$peak_end  = annotated_peaks$peak_end - 500


# ## Perform liftover from hg19 to hg38
# library(rtracklayer)
# library(GenomicRanges)
# library(dplyr)
# library(liftOver)
# 
# chain_file_url <- "https://hgdownload.cse.ucsc.edu/goldenpath/hg19/liftOver/hg19ToHg38.over.chain.gz"
# chain_file <- ".hg19ToHg38.over.chain.gz"
# if (!file.exists(chain_file)) {
#   download.file(chain_file_url, destfile = chain_file)
# }
# 
# # Step 2: Import the chain file
# chain <- import.chain(".hg19ToHg38.over.chain")
# 
# gr_hg19 <- GRanges(seqnames = annotated_peaks$peak_chr,
#               ranges = IRanges(start = annotated_peaks$peak_start, end = annotated_peaks$peak_end),
#               peak_id = annotated_peaks$peak_id)
# # Step 4: Perform liftover
# lifted_gr <- liftOver(gr_hg19, chain)
# 
# # Handle unlifted ranges (e.g., regions that didn't map to HG38)
# # This example keeps only successfully lifted regions
# lifted_gr <- unlist(lifted_gr)  # Flatten the GRangesList into GRanges
# lifted_gr <- lifted_gr[!duplicated(lifted_gr$peak_id)]
# 
# # Match lifted coordinates back to original DataFrame (assuming 1:1 mapping)
# # Create a DataFrame from the lifted GRanges object
# lifted_df <- data.frame(
#   peak_chr_hg38 = as.character(seqnames(lifted_gr)),
#   peak_start_hg38 = start(lifted_gr),
#   peak_end_hg38 = end(lifted_gr),
#   peak_id = lifted_gr$peak_id
# )
# 
# annotated_peaks = merge(annotated_peaks, lifted_df, by = "peak_id", all.x = TRUE)



## Count E-boxes in the sequences
library(GenomicRanges)
library(rtracklayer)
library(Biostrings)
library(BSgenome.Hsapiens.UCSC.hg38)
library(reshape2)
library(ggpubr)
library(dplyr)
library(ggplot2)
library(scales)

ref_genome <- BSgenome.Hsapiens.UCSC.hg38

gr <- GRanges(seqnames = annotated_peaks$peak_chr,
              ranges = IRanges(start = annotated_peaks$peak_start - 249, end = annotated_peaks$peak_end + 249),
              peak_id = annotated_peaks$peak_id)

sequences <- getSeq(ref_genome, gr)

bases <- c("A", "T", "C", "G")
all_motifs <- c()

# Iterate over all combinations of two 'N' positions
for (b1 in bases) {
  for (b2 in bases) {
    motif <- paste0("CA", b1, b2, "TG")
    all_motifs <- c(all_motifs, motif)
  }
}

all_motifs = c(all_motifs, "CACGCG", "CATGCG", "CACGAG")

count_eboxes_per_seq <- function(seq, motifs) {
  sapply(motifs, function(motif) countPattern(motif, seq))
}


# Apply the function to count E-boxes for each sequence
counts_list <- lapply(sequences, count_eboxes_per_seq, motifs = all_motifs)

# Convert to dataframe
counts_df <- do.call(rbind, counts_list)
colnames(counts_df) <- c(all_motifs)

original_ebox_motifs <- c("CACGTG", "CACGCG", "CATGCG", "CATGTG", "CACGAG", "CACATG")

# Combine with original dataframe
annotated_peaks <- cbind(annotated_peaks, counts_df[, original_ebox_motifs])

# Update the sum_eboxes column
annotated_peaks$sum_eboxes <- rowSums(annotated_peaks[, original_ebox_motifs])
annotated_peaks$sum_eboxes_and_CANNTG_total <- rowSums(counts_df[, all_motifs])



## CpG Island Overlap
library(GenomicRanges)
library(rtracklayer)
library(Biostrings)
library(ggpubr)
library(ggplot2)
library(BSgenome.Hsapiens.UCSC.hg38)
library(data.table)

session <- browserSession()
genome(session) <- "hg38"

query <- ucscTableQuery(session, table="cpgIslandExt")
cpg_islands_hg38 <- track(query)  # GRanges object


gr <- GRanges(seqnames = annotated_peaks$peak_chr,
              ranges = IRanges(start = annotated_peaks$peak_start - 249, end = annotated_peaks$peak_end + 249))


overlaps <- findOverlaps(gr, cpg_islands_hg38)
overlaps_df <- as.data.frame(overlaps)
contained <- overlaps_df$queryHits


annotated_peaks$CpG_island <- FALSE
annotated_peaks$CpG_island[contained] <- TRUE



gr <- GRanges(seqnames = annotated_peaks$peak_chr,
              ranges = IRanges(start = annotated_peaks$peak_start - 249, end = annotated_peaks$peak_end + 249),
              peak_id = annotated_peaks$peak_id,
              gene_id = annotated_peaks$gene_id)
hits <- findOverlaps(gr, cpg_islands_hg38)

# 2. Compute the intersection for each overlap pair
intersectionRanges <- pintersect(gr[queryHits(hits)], cpg_islands_hg38[subjectHits(hits)])
# assign the corresponding peak_id from the peaks GRanges
intersectionRanges$peak_id <- gr$peak_id[queryHits(hits)]
intersectionRanges$gene_id <- gr$gene_id[queryHits(hits)]

ref_genome <- BSgenome.Hsapiens.UCSC.hg38
ebox_motifs <- c("CACGTG", "CACGCG", "CATGCG", "CATGTG", "CACGAG", "CACATG")

sequences <- getSeq(ref_genome, intersectionRanges)

count_eboxes_per_seq <- function(seq, motifs) {
  sapply(motifs, function(motif) countPattern(motif, seq))
}

# Apply the function to count E-boxes for each sequence
counts_list <- lapply(sequences, count_eboxes_per_seq, motifs = ebox_motifs)

ebox_inside_cpg_names = paste0(ebox_motifs, "_inside_cpg")

# Convert the list of counts to a data frame
counts_df <- do.call(rbind, counts_list)
colnames(counts_df) <- ebox_inside_cpg_names
counts_df = as.data.frame(counts_df)
counts_df$sum_eboxes_inside_cpg = rowSums(counts_df)
counts_df$peak_id = intersectionRanges$peak_id
counts_df$gene_id = intersectionRanges$gene_id

counts_df = counts_df %>% group_by(peak_id, gene_id) %>% summarise_all(sum)



annotated_peaks = merge(annotated_peaks, counts_df, by = c("peak_id", "gene_id"), all.x = TRUE)

annotated_peaks[is.na(annotated_peaks$sum_eboxes_inside_cpg), ebox_inside_cpg_names] = c(0, 0, 0, 0, 0, 0)
annotated_peaks$sum_eboxes_inside_cpg[is.na(annotated_peaks$sum_eboxes_inside_cpg)] = 0



tata_motifs = c("TATAAAA", "TATATAA", "TATAAAT", "TATATAT")
ref_genome <- BSgenome.Hsapiens.UCSC.hg38

gr <- GRanges(seqnames = annotated_peaks$peak_chr,
              ranges = IRanges(start = annotated_peaks$peak_start - 249, end = annotated_peaks$peak_end + 249))

sequences <- getSeq(ref_genome, gr)

count_TATA_per_seq <- function(seq, motifs) {
  sum(sapply(motifs, function(motif) countPattern(motif, seq)))
}

counts <- lapply(sequences, count_TATA_per_seq, motifs = tata_motifs)

# Convert the list of counts to a data frame
counts_df <- do.call(rbind, counts)

annotated_peaks <- cbind(annotated_peaks, counts_df)

names(annotated_peaks)[names(annotated_peaks) == "counts_df"] = "TATA"

write.table(annotated_peaks, "./resources/annotated_peaks_hg38_right.tsv", sep = "\t", row.names = FALSE, quote = FALSE)

 top10th_percentile = quantile(annotated_peaks$coverage_myc, 0.9)
 
 annotated_peaks_top10 = annotated_peaks[annotated_peaks$coverage_myc >= top10th_percentile, ]
 write.table(annotated_peaks_top10, "./resources/annotated_peaks_hg38_right_top10.tsv", sep = "\t", row.names = FALSE, quote = FALSE)
