# SPDX-FileCopyrightText: 2025 Julius Deynet <julius.deynet@stud-mail.uni-wuerzburg.de>
#
# SPDX-License-Identifier: GPL-3.0-or-later

suppressPackageStartupMessages({
  library(rtracklayer)
  library(GenomicRanges)
  library(GenomicAlignments)
  library(IRanges)
  library(DESeq2)
  library(DT)
  library(shiny)
  library(ggplot2)
  library(ggrepel)
  library(dplyr)
  library(tidyr)
  library(grandR)
  library(BiocParallel)
  library(ggpubr)
  library(ggnewscale)
  library(RColorBrewer)
  library(colorspace)
})

# Basic structure:
#   1. Load Data or calculate data as fallback
#   2. Define Module for getting coverage/regulation/bursting data 
#        (used in Global Volcano/Box and Motif Box Comparison)
#   3. Define UI
#   4. Define Server with logic for different tabs. There, data getter functions
#        are defined first, followed by the actual plotting



annotated_peaks = as.data.frame(read.delim("./resources/annotated_peaks_hg38_right.tsv"))
annotated_peaks$peak_start = annotated_peaks$peak_start - 249
annotated_peaks$peak_end = annotated_peaks$peak_end + 249
distance_categories = c(
  "3-2kb upstream",
  "2-1kb upstream",
  "1-0kb upstream",
  "0-1kb downstream",
  "1-2kb downstream",
  "2-3kb downstream"
)
annotated_peaks$distanceToTSSCategories = cut(
  annotated_peaks$distanceToTSS,
  breaks = c(-Inf, -2000, -1000, 0, 1000, 2000, Inf),
  labels = distance_categories
)


annotated_peaks_range = GRanges(
  seqnames = annotated_peaks$chr,
  ranges = IRanges(start = annotated_peaks$peak_start, end = annotated_peaks$peak_end),
  peak_id = annotated_peaks$peak_id
)

bursting_data = as.data.frame(read.csv("./resources/bursting_all_genes.csv"))

#bursting_data = subset(bursting_data, bestmodel != "no change")

names(bursting_data)[names(bursting_data) == "km"] = "burst_frequency"
names(bursting_data)[names(bursting_data) == "B"] = "burst_size"

bursting_data$bestmodel = factor(bursting_data$bestmodel,
                                 levels = c("no change", "size", "frequency", "both"))

bursting_data$bursting_diff = bursting_data$burst_frequency - bursting_data$burst_size


promoter_coverages = as.data.frame(read.delim("./resources/promoter_coverages_hg_38.tsv"))


ebox_motifs = c("CACGTG", "CACGCG", "CATGCG", "CATGTG", "CACGAG", "CACATG")

# Path to the RDS file where preprocessed data will be saved
featureCounts_rds_file_path <- "./resources/processed_talk_seq_data.rds"
featureCounts_TC_rds_file_path <- "./resources/processed_tc_talk_seq_data.rds"
featureCounts_introns_rds_file_path <- "./resources/processed_featureCounts_introns_data.rds"
featureCounts_introns_TC_rds_file_path <- "./resources/processed_featureCounts_introns_TC_data.rds"

# Function to load or process data Talk-seq data
load_or_process_talk_seq_data <- function(use_TC = FALSE,
                                          introns_only = FALSE) {
  path = if (use_TC &
             introns_only) {
    featureCounts_introns_TC_rds_file_path
  } else if (use_TC) {
    featureCounts_TC_rds_file_path
  } else if (introns_only) {
    featureCounts_introns_rds_file_path
  } else {
    featureCounts_rds_file_path
  }
  if (file.exists(path)) {
    # Load preprocessed data from RDS file
    message("Loading data from RDS file...")
    data <- readRDS(path)
  } else {
    # Load the GTF file (path to your downloaded GTF file)
    gtf_file <- "./resources/homo_sapiens.90.gtf"
    
    # Import the GTF file
    gtf_data <- as.data.frame(import(gtf_file))
    
    # Filter the data for only gene entries
    gene_data <- subset(gtf_data, type == "gene")
    # gene_data <- subset(gtf_data, type == "transcript")
    # gene_data = gene_data %>%
    #   group_by(gene_id) %>%
    #   filter(transcript_version == max(transcript_version)) %>%
    #   arrange(gene_name, factor(transcript_source, levels = c("ensembl", "ensembl_havana", "havana", "insdc", "mirbase"))) %>%  # Prioritize ensembl > ensembl_havana
    #   slice(1) %>%  # Keep the first entry (which will be ensembl if present)
    #   ungroup()
    
    gene_list = gene_data %>% select(gene_name, gene_id, seqnames, start, end, strand)
    
    names(gene_list)[names(gene_list) == "start"] = "start_position"
    names(gene_list)[names(gene_list) == "end"] = "end_position"
    names(gene_list)[names(gene_list) == "seqnames"] = "chromosome"
    
    DESeq2_data = if (use_TC &
                      introns_only) {
      read.csv("./resources/DESeq2_results_introns_TC_new.csv")
    } else if (use_TC) {
      read.csv("./resources/DESeq2_results_TC_new.csv")
    } else if (introns_only) {
      read.csv("./resources/DESeq2_results_introns_new.csv")
    } else{
      read.csv("./resources/DESeq2_results_new.csv")
    }
    
    names(DESeq2_data)[names(DESeq2_data) == "X"] = "gene_id"
    
    merged = merge(gene_list, DESeq2_data, by = "gene_id")
    
    toKeep = readRDS("./resources/talk-seq_genes_to_keep_new.rds")
    
    merged = merged[merged$gene_id %in% toKeep, ]
    
    merged = merged[!is.na(merged$log2FoldChange) &
                      !is.na(merged$padj), ]
    
    min_pvalue <- 1e-300  # Or choose a smaller value if needed
    
    # Replace p-values of 0
    merged$pvalue[merged$pvalue == 0] <- min_pvalue
    merged$padj[merged$padj == 0] <- min_pvalue
    
    merged$logPadj <- -log10(ifelse(is.na(merged$padj), merged$pvalue, merged$padj))
    
    merged$significance <- ifelse(!is.na(merged$logPadj) &
                                    merged$logPadj > 1.3,
                                  "Significant",
                                  "Insignificant")
    
    merged_subset = merged[, c(
      "gene_name",
      "gene_id",
      "chromosome",
      "start_position",
      "end_position",
      "strand",
      "log2FoldChange",
      "logPadj",
      "significance"
    )]
    
    # Load the MYC signature genes CSV file
    myc_signature <- read.csv("./resources/MYC_signature.csv",
                              stringsAsFactors = FALSE)
    
    # Check for MYC signature genes in your DESeq2 results
    merged_subset$MYC_signature <- ifelse(
      merged_subset$gene_name %in% myc_signature$`HGNC.gene.symbol`,
      "MYC_signature",
      "Other"
    )
    
    saveRDS(merged_subset, file = path)
    
    data = merged_subset
    
  }
  return(data)
}


talk_seq_data = load_or_process_talk_seq_data(FALSE, FALSE)
tc_talk_seq_data = load_or_process_talk_seq_data(TRUE, FALSE)

intronic_talk_seq_data = load_or_process_talk_seq_data(FALSE, TRUE)
intronic_tc_talk_seq_data = load_or_process_talk_seq_data(TRUE, TRUE)



grandR_rds_single_cell_total_file_path <- "./resources/processed_slam_seq_data.rds"
grandR_rds_single_cell_new_file_path <- "./resources/processed_new_slam_seq_data.rds"
load_or_process_grandR_singel_cell_data <- function(total = TRUE) {
  path = if (total) {
    grandR_rds_single_cell_total_file_path
  } else {
    grandR_rds_single_cell_new_file_path
  }
  if (file.exists(path)) {
    # Load preprocessed data from RDS file
    message("Loading data from RDS file...")
    grandRdata <- readRDS(path)
  } else {
    library(grandR)
    
    
    single_cell_data = readRDS("./resources/pseudobulk_MYC.rds")
    
    single_cell_data = subset(single_cell_data,
                              Coldata(single_cell_data, "Sample") == "A375")
    
    single_cell_data = FilterGenes(single_cell_data)
    
    
    analysisData = GetAnalysisTable(single_cell_data)
    
    analysisData$"total.Aux vs DMSO.A375.LFC" = analysisData$`total.Aux vs DMSO.A375.LFC` * -1
    analysisData$"new.Aux vs DMSO.A375.LFC" = analysisData$`new.Aux vs DMSO.A375.LFC` * -1
    
    analysisData$logPvalue <- -log10(if (total) {
      analysisData$`total.Aux vs DMSO.A375.P`
    } else {
      analysisData$`new.Aux vs DMSO.A375.P`
    })
    
    
    grandRdata = as.data.frame(analysisData)
    
    grandRdata$significance <- ifelse(if (total) {
      analysisData$`total.Aux vs DMSO.A375.P`
    } else {
      analysisData$`new.Aux vs DMSO.A375.P`
    } < 0.05, "Significant", "Insignificant")
    
    names(grandRdata)[names(grandRdata) == "Symbol"] = "gene_name"
    names(grandRdata)[names(grandRdata) == if (total) {
      "total.Aux vs DMSO.A375.LFC"
    } else {
      "new.Aux vs DMSO.A375.LFC"
    }] = "log2FoldChange"
    names(grandRdata)[names(grandRdata) == "logPvalue"] = "logPadj"
    names(grandRdata)[names(grandRdata) == "Gene"] = "gene_id"
    
    # Load the MYC signature genes CSV file
    myc_signature <- read.csv("./resources/MYC_signature.csv",
                              stringsAsFactors = FALSE)
    
    # Check for MYC signature genes in your DESeq2 results
    grandRdata$MYC_signature <- ifelse(
      grandRdata$gene_name %in% myc_signature$`HGNC.gene.symbol`,
      "MYC_signature",
      "Other"
    )
    
    saveRDS(grandRdata, file = path)
  }
  return(grandRdata)
}

slam_seq_data = load_or_process_grandR_singel_cell_data(TRUE)
new_slam_seq_data = load_or_process_grandR_singel_cell_data(FALSE)

sc_new_counts = as.data.frame(GetMatrix(
  readRDS("./resources/pseudobulk_MYC.rds"),
  "new.count"
))
sc_new_counts$gene_name = rownames(sc_new_counts)

sc_new_counts = sc_new_counts %>% rowwise() %>% mutate(IAA = sum(c(
  A375.Aux.4sU.A, A375.Aux.4sU.B, A375.Aux.4sU.C
)), DMSO = sum(c(
  A375.DMSO.4sU.A, A375.DMSO.4sU.B, A375.DMSO.4sU.C
)))


bursting_data = merge(bursting_data,
                      sc_new_counts,
                      by.x = "gene",
                      by.y = "gene_name")
bursting_data$read_counts = bursting_data$IAA + bursting_data$DMSO

bursting_data = bursting_data %>% select(gene,
                                         burst_size,
                                         burst_frequency,
                                         bursting_diff,
                                         read_counts,
                                         bestmodel)

sc_data = new_slam_seq_data
sc_data = sc_data %>% select(gene_id, gene_name, log2FoldChange, MYC_signature)
bursting_data = merge(bursting_data, sc_data, by.x = "gene", by.y = "gene_name")


exon_data_file_path <- "./resources/processed_exon_data.rds"
# Function to load or process data. This is needed for an annotation track in the Coverage Viewer
load_or_process_exon_data <- function() {
  if (file.exists(exon_data_file_path)) {
    # Load preprocessed data from RDS file
    message("Loading data from RDS file...")
    exon_data <- readRDS(exon_data_file_path)
  } else {
    gtf_file <- "./resources/homo_sapiens.90.gtf"
    
    # Import the GTF as df
    gtf_data = as.data.frame(import(gtf_file))
    
    # Filter the data for only gene entries
    exon_data <- subset(gtf_data, type == "exon")
    
    genes_included = c(talk_seq_data$gene_id, slam_seq_data$gene_id)
    
    exon_data = exon_data %>% filter(gene_id %in% genes_included) %>% select(gene_id, start, end)
    
    saveRDS(exon_data, file = exon_data_file_path)
    
  }
  return(exon_data)
}

exon_data = load_or_process_exon_data()


# Function to create coverage track for a given region
calc_coverage <- function(bamFiles, chr, start, end, strand) {
  total_cov <- numeric(end - start + 1)
  
  
  region <- GRanges(
    seqnames = chr,
    ranges = IRanges(start = start, end = end),
    strand = strand
  )
  
  precomputed_cov <- lapply(bamFiles, function(bamFile) {
    param <- ScanBamParam(which = region,
                          flag = scanBamFlag(isMinusStrand = strand == "-"))
    reads <- readGAlignments(bamFile, param = param)
    cov <- coverage(reads)[[chr]]
    
    # Align the coverage to the region's range explicitly
    as.numeric(cov[start:end])
  })
  
  # Sum the coverage across BAM files
  total_cov <- Reduce(`+`, precomputed_cov)
  
  total_cov
}

library(shiny)
library(DT)

# UI Element for data selection of global plots
data_with_peak_selection_ui = function(id) {
  ns = NS(id)
  category_name = as.character(ns("category"))
  comarison_name = ns("comparison_value")
  fluidRow(
    column(
      3,
      
      # Dropdown Menu for Selecting a Plot Type
      selectInput(
        ns("data_source"),
        label = "Select Plot Data Source:",
        choices = c(
          "exonic Talk-seq",
          "exonic TC Talk-seq",
          "intronic Talk-seq",
          "intronic TC Talk-seq",
          "scSLAM-seq",
          "new scSLAM-seq"
        ),
        selected = "intronic TC Talk-seq"
      ),
      # Default selection
      selectInput(
        ns("comparison_value"),
        label = "Select Comparison value:",
        choices = c(
          "Regulatory LFC",
          "MYC Coverage",
          "Bursting Difference",
          "Bursting Size",
          "Bursting Frequency"
        )
      ),
      conditionalPanel(
        condition = sprintf(
          "input['%s'] === 'Bursting Difference' |
                                 input['%s'] === 'Bursting Size' |
                                 input['%s'] === 'Bursting Frequency'",
          ns("comparison_value"),
          ns("comparison_value"),
          ns("comparison_value")
        ),
        
        selectInput(
          ns("bursting_selection"),
          label = "Choose Bursting Data:",
          choices = c("All", "With Change", "LFC > 1", "Top 10% Reads", "Shown in Table"),
        ),
      ),
      selectInput(ns("category"), label = "Select Category:", choices = NULL),
      conditionalPanel(
        condition = sprintf("input['%s'] != 'MYC signature genes'", ns("category")),
        selectInput(
          ns("which_peaks"),
          label = "Select which Peaks to consider:",
          choices = c("All Peaks", "Strongest Peaks"),
          selected = "Strongest Peaks"
        )
      ),
      # Conditionally show slider for "Peak Count" category
      conditionalPanel(
        condition = sprintf("input['%s'] != 'MYC signature genes'", ns("category")),
        
        # Slider between 0 and 1 when "Peak Count" is selected
        sliderInput(
          ns("peak_slider"),
          label = "Select Peak Percentile",
          min = 0,
          max = 100,
          value = 0,
          step = 5
        ),
        
        verbatimTextOutput(ns("num_peaks_selected"))
      )
    ),
    column(9, conditionalPanel(
      condition = sprintf("input['%s'] != 'MYC signature genes'", ns("category")),
      plotOutput(
        ns("avgProfPlot"),
        height = "400px",
        brush = brushOpts(
          id = ns("avgProf_brush"),
          direction = "x",
          clip = TRUE,
          resetOnNew = TRUE
        ),
        dblclick = ns("avgProfPlot_reset")
      )
    ))
  )
}

# Server module for data selection of global plots
data_with_peak_selection_getter = function(id,
                                           category_getter,
                                           filtered_bursting_data_indices,
                                           shared_selection_values) {
  moduleServer(id, function(input, output, session) {
    output$avgProfPlot = renderPlot({
      ggplot(percentile_filtered_peak_data(), aes(x = distanceToTSS)) +
        geom_density(alpha = 0.5) +
        labs(title = "Exclude from filtering", x = "Distance to TSS", y = "Density") +
        theme_minimal() +
        scale_x_continuous(breaks = seq(-3000, 3000, 1000),
                           limits = c(-3000, 3000))
    })
    
    exclude_range <- reactiveVal(c(-Inf, Inf))
    
    observeEvent(input$avgProf_brush, {
      brush <- input$avgProf_brush
      if (!is.null(brush)) {
        shared_selection_values$x_min = brush$xmin
        shared_selection_values$x_max = brush$xmax
        exclude_range(c(brush$xmin, brush$xmax))
      }
    })
    
    category_choices = reactiveVal(c())
    
    observeEvent(input$comparison_value, {
      current_category = isolate(input$category)
      temp = category_getter(input$comparison_value, current_category)
      
      category_choices(temp$choices)
      
      # Update the dependent selectInput choices
      updateSelectInput(
        session,
        "category",
        choices = temp$choices,
        selected = temp$current_category
      )
    })
    
    ## Synchonize the selection between different tabs
    observe({
      updateSelectInput(session, "data_source", selected = shared_selection_values$data_source)
    })
    observe({
      updateSelectInput(session,
                        "comparison_value",
                        selected = shared_selection_values$comparison_value)
    })
    observe({
      if (shared_selection_values$comparison_value == "Bursting Difference" |
          shared_selection_values$comparison_value == "Bursting Size" |
          shared_selection_values$comparison_value == "Bursting Frequency") {
        updateSelectInput(session,
                          "bursting_selection",
                          selected = shared_selection_values$bursting_selection)
      }
    })
    observe({
      if (shared_selection_values$category %in% isolate(category_choices)()) {
        updateSelectInput(session, "category", selected = shared_selection_values$category)
      }
    })
    observe({
      updateSelectInput(session, "which_peaks", selected = shared_selection_values$which_peaks)
    })
    observe({
      updateSliderInput(session, "peak_slider", value = shared_selection_values$peak_slider)
    })
    
    observeEvent(input$avgProfPlot_reset, {
      session$resetBrush("avgProf_brush")
      exclude_range(c(-Inf, Inf))
    })
    
    observeEvent(input$peak_slider, {
      shared_selection_values$peak_slider = input$peak_slider
      exclude_range(c(-Inf, Inf))
    })
    
    regulation_data = reactive({
      shared_selection_values$data_source = input$data_source
      data = if (input$data_source == "exonic Talk-seq") {
        talk_seq_data
      } else if (input$data_source == "exonic TC Talk-seq") {
        tc_talk_seq_data
      } else if (input$data_source == "intronic Talk-seq") {
        intronic_talk_seq_data
      } else if (input$data_source == "intronic TC Talk-seq") {
        intronic_tc_talk_seq_data
      } else if (input$data_source == "scSLAM-seq")  {
        slam_seq_data
      } else {
        new_slam_seq_data
      }
      data
    })
    
    percentile_filtered_peak_data = reactive({
      if (input$peak_slider == 0) {
        return(annotated_peaks)
      }
      top_nth_percentile <- quantile(annotated_peaks$coverage_myc, input$peak_slider / 100)
      
      annotated_peaks[annotated_peaks$coverage_myc >= top_nth_percentile, ]
    })
    
    distance_filtered_peak_data = reactive({
      data = percentile_filtered_peak_data()
      exclude_start <- exclude_range()[1]
      exclude_end <- exclude_range()[2]
      
      data[data$distanceToTSS > exclude_start &
             data$distanceToTSS < exclude_end, ]
      
    })
    
    bursting_data_getter = reactive({
      data = bursting_data
      shared_selection_values$bursting_selection = input$bursting_selection
      if (input$bursting_selection == "With Change") {
        data = subset(data, bestmodel != "no change")
      } else if (input$bursting_selection == "LFC > 1") {
        data = data[data$log2FoldChange > 1, ]
        data = data %>% select(gene, bursting_diff, burst_size, burst_frequency)
      } else if (input$bursting_selection == "Top 10% Reads") {
        top_nth_percentile <- quantile(data$read_counts, 0.9)
        data = data[data$read_counts >= top_nth_percentile, ]
      } else if (input$bursting_selection == "Shown in Table") {
        if (length(filtered_bursting_data_indices()) != 0) {
          data = data[filtered_bursting_data_indices(), ]
        }
      }
      # set these to NULL so it isnt dupliacted after merging with regulatory data
      data$gene_id = NULL
      data$log2FoldChange = NULL
      data$MYC_signature = NULL
      data$bursting_diff = pmax(data$bursting_diff, -10)
      data$bursting_diff = pmin(data$bursting_diff, 10)
      data
    })
    
    data_with_peaks <- reactive({
      shared_selection_values$which_peaks = input$which_peaks
      data = regulation_data()
      
      data$logPadj_capped = pmin(data$logPadj, 100)
      
      filtered_peak_data = distance_filtered_peak_data()
      
      if (input$comparison_value == "Bursting Difference" |
          input$comparison_value == "Bursting Size" |
          input$comparison_value == "Bursting Frequency") {
        bursting = bursting_data_getter()
        data = merge(data, bursting, by.x = "gene_name", by.y = "gene")
      }
      
      data = merge(data,
                   filtered_peak_data,
                   by = "gene_id",
                   all.x = TRUE)
      if (input$which_peaks == "Strongest Peaks") {
        data = data %>% group_by(gene_id) %>% filter(is.na(coverage_myc) |
                                                       max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()
      } else {
        combined_genes = data %>%
          filter(!is.na(coverage_myc)) %>%
          group_by(gene_id) %>%
          mutate(
            CpG_island = any(CpG_island, na.rm = TRUE),
            TATA = sum(TATA, na.rm = TRUE),
            sum_eboxes = sum(sum_eboxes, na.rm = TRUE),
            sum_eboxes_and_CANNTG_total = sum(sum_eboxes_and_CANNTG_total, na.rm = TRUE),
            CACGTG = sum(CACGTG, na.rm = TRUE),
            CACGCG = sum(CACGCG, na.rm = TRUE),
            CATGCG = sum(CATGCG, na.rm = TRUE),
            CATGTG = sum(CATGTG, na.rm = TRUE),
            CACGAG = sum(CACGAG, na.rm = TRUE),
            CACATG = sum(CACATG, na.rm = TRUE),
            coverage_myc = sum(coverage_myc, na.rm = TRUE)
          ) %>%
          filter(is.na(distanceToTSS) |
                   min(abs(distanceToTSS)) == abs(distanceToTSS)) %>%
          distinct(gene_id, .keep_all = TRUE) %>%
          ungroup()
        data_without_peaks = data %>% filter(is.na(coverage_myc))
        data = rbind(combined_genes, data_without_peaks)
      }
      
      
      gene_counts = table(filtered_peak_data$gene_id)
      data$peak_count = as.numeric(gene_counts[match(data$gene_id, names(gene_counts))])
      data$peak_count[is.na(data$peak_count)] = 0
      
      data
    })
    
    comparison_value <- reactive({
      shared_selection_values$comparison_value = input$comparison_value
      if (input$comparison_value == "Regulatory LFC") {
        "log2FoldChange"
      } else if (input$comparison_value == "Bursting Difference") {
        "bursting_diff"
      } else if (input$comparison_value == "Bursting Size") {
        "burst_size"
      } else if (input$comparison_value == "Bursting Frequency") {
        "burst_frequency"
      } else {
        updateSliderInput(session, "peak_slider", value = 0)
        "coverage_myc"
      }
    })
    
    comparison_name <- reactive({
      if (input$comparison_value == "Regulatory LFC") {
        "-log2(Fold Change MYC vs. MYC Depletion)"
      } else if (input$comparison_value == "Bursting Difference") {
        "-log2(Fold Change Burst Frequency / Size MYC vs. MYC Depletion)"
      } else if (input$comparison_value == "Bursting Size") {
        "-log2(Fold Change Burst Size MYC vs. MYC Depletion)"
      } else if (input$comparison_value == "Bursting Frequency") {
        "-log2(Fold Change Burst Frequency MYC vs. MYC Depletion)"
      } else {
        "Coverage MYC"
      }
    })
    
    category <- reactive({
      shared_selection_values$category = input$category
      if (input$category == "MYC signature genes") {
        "MYC_signature"
      } else if (input$category == "Peak Count") {
        "peak_count"
      } else if (input$category %in% ebox_motifs) {
        input$category
      } else if (input$category == "Main Motifs") {
        "sum_eboxes"
      } else if (input$category == "Main Motifs + CANNTG") {
        "sum_eboxes_and_CANNTG_total"
      } else if (input$category == "CpG Islands") {
        "CpG_island"
      } else if (input$category == "TATA boxes") {
        "TATA"
      } else if (input$category == "None")  {
        "None"
      } else {
        "distanceToTSSCategories"
      }
    })
    
    output$num_peaks_selected <- renderPrint({
      filtered_peak_data <- distance_filtered_peak_data()
      
      cat(nrow(filtered_peak_data), "peaks considered")
    })
    
    list(
      data_with_peaks = data_with_peaks,
      filtered_peak_data = distance_filtered_peak_data,
      comparison_value = comparison_value,
      comparison_name = comparison_name,
      category = category,
      which_peaks = reactive(input$which_peaks),
      category_name = reactive(input$category),
      data_source = reactive(input$data_source)
    )
  })
}


ui <- navbarPage(
  title = "Integrative Analysis of MYC-Regulated Genes",
  id = "active_tab",
  
  tabPanel("Coverage Viewer", fluidRow(
    column(
      4,
      h3("Gene List"),
      
      selectInput(
        "coverage_data_source",
        label = "Select Data Source:",
        choices = c(
          "exonic Talk-seq",
          "exonic TC Talk-seq",
          "intronic Talk-seq",
          "intronic TC Talk-seq"
        ),
        selected = "intronic TC Talk-seq"
      ),
      # Default selection
      DTOutput("gene_table")
    ),
    column(
      8,
      h3("Coverage Plot"),
      plotOutput(
        "coverage_plot",
        brush = brushOpts(
          id = "coverage_plot_brush",
          resetOnNew = TRUE,
          direction = "x",
          clip = TRUE
        ),
        width = "100%",
        height = "500px",
        dblclick = "coverage_plot_reset"
      ),
      plotOutput("start_coverage_plot", width = "100%")
    )
  )),
  
  
  tabPanel(
    "Data Source Comparison",
    fluidRow(
      column(
        2,
        h3("Data Source Comparison"),
        
        selectInput(
          "LFC_data_source",
          label = "Select Talk-seq Data Source:",
          choices = c(
            "exonic Talk-seq",
            "exonic TC Talk-seq",
            "intronic Talk-seq",
            "intronic TC Talk-seq"
          ),
          selected = "intronic TC Talk-seq"
        ),
        
        selectInput(
          "single_cell_source",
          label = "Select SLAM-seq Data Source:",
          choices = c("scSLAM-seq", "new scSLAM-seq"),
          selected = "new scSLAM-seq"
        ),
      ),
      column(
        10,
        
        plotOutput(
          "LFC_comparison_plot",
          brush = brushOpts(id = "LFC_plot_brush"),
          width = "100%",
          height = "800px"
        ),
      )
    ),
    fluidRow(wellPanel(
      h4("Selected Genes:"),
      verbatimTextOutput("LFC_selected_genes")
    ))
  ),
  
  tabPanel(
    "Bursting Data Viewer",
    fluidRow(DTOutput("bursting_table")),
    div(
      style = "display: flex; justify-content: center;",
      plotOutput("bursting_scatter_plot", width = "50%", height = "800px")
    )
  ),
  
  
  tabPanel(
    "Global Scatter Comparison",
    fluidRow(
      column(
        2,
        h3("Global Scatter Plot"),
        
        selectInput(
          "scatter_data_source",
          label = "Select Data Source:",
          choices = c(
            "exonic Talk-seq",
            "exonic TC Talk-seq",
            "intronic Talk-seq",
            "intronic TC Talk-seq",
            "scSLAM-seq",
            "new scSLAM-seq"
          ),
          selected = "intronic TC Talk-seq"
        ),
        
        selectInput(
          "scatter_metric1",
          label = "Select x Metric:",
          choices = c(
            "Regulatory LFC",
            "MYC Coverage (Strongest Peak)",
            "MYC Coverage (All Peaks)",
            "Promoter Coverage",
            "Promoter Coverage (Upstream)",
            "Promoter Coverage (Downstream)",
            "Bursting Size",
            "Bursting Frequency",
            "Bursting Difference"
          )
        ),
        
        selectInput(
          "scatter_metric2",
          label = "Select y Metric:",
          choices = c(
            "Regulatory LFC",
            "MYC Coverage (Strongest Peak)",
            "MYC Coverage (All Peaks)",
            "Promoter Coverage",
            "Promoter Coverage (Upstream)",
            "Promoter Coverage (Downstream)",
            "Bursting Size",
            "Bursting Frequency",
            "Bursting Difference"
          ),
          selected = "MYC Coverage (Strongest Peak)"
        ),
      ),
      column(
        10,
        
        plotOutput(
          "scatter_plot",
          brush = brushOpts(id = "scatter_plot_brush"),
          width = "100%",
          height = "800px"
        ),
      )
    ),
    fluidRow(wellPanel(
      h4("Selected Genes:"),
      verbatimTextOutput("scatter_selected_genes")
    ))
  ),
  
  tabPanel(
    "Global Volcano/Box Comparison",
    
    data_with_peak_selection_ui("global_plots"),
    
    
    fluidRow(column(
      6,
      h3("Global Volcano Plot"),
      
      plotOutput(
        "volcano_plot",
        height = "50vh",
        brush = brushOpts(id = "plot_brush")
      )
    ), column(
      6,
      h3("Global Box Plot"),
      
      plotOutput("global_box_plot", height = "50vh")   # Box plot output
    )),
    wellPanel(h4("Selected Genes:"), verbatimTextOutput("selected_genes"))
  ),
  
  tabPanel(
    "Motif Box Comparison",
    
    
    data_with_peak_selection_ui("motif"),
    
    fluidRow(
      div(style = "display: flex; justify-content: center;", plotOutput(
        "motif_plot", height = "60vh", width = "90vh"
      ))
    )
  ),
)

server <- function(input, output, session) {
  ### Coverage Viewer Tab
  coverage_data = reactive({
    data = if (input$coverage_data_source == "exonic Talk-seq") {
      talk_seq_data
    } else if (input$coverage_data_source == "exonic TC Talk-seq") {
      tc_talk_seq_data
    } else if (input$coverage_data_source == "intronic Talk-seq") {
      intronic_talk_seq_data
    } else if (input$coverage_data_source == "intronic TC Talk-seq") {
      intronic_tc_talk_seq_data
    }
    data = data %>% select(
      gene_name,
      gene_id,
      log2FoldChange,
      logPadj,
      MYC_signature,
      chromosome,
      start_position,
      end_position,
      strand
    )
    data$MYC_signature = factor(data$MYC_signature, levels = c("MYC_signature", "Other"))
    data
  })
  
  coverage_bam_files = reactive({
    data = if (input$coverage_data_source == "exonic Talk-seq" |
               input$coverage_data_source == "intronic Talk-seq") {
      list(
        myc_bamFiles = c(
          "./bams/DMSO.ALK.A.bam",
          "./bams/DMSO.ALK.B.bam",
          "./bams/DMSO.ALK.C.bam"
        ),
        
        non_myc_bamFiles = c(
          "./bams/IAA.ALK.A.bam",
          "./bams/IAA.ALK.B.bam",
          "./bams/IAA.ALK.C.bam"
        )
      )
    } else if (input$coverage_data_source == "exonic TC Talk-seq" |
               input$coverage_data_source == "intronic TC Talk-seq") {
      list(
        myc_bamFiles = c(
          "./bams/DMSO.ALK.A.TC.bam",
          "./bams/DMSO.ALK.B.TC.bam",
          "./bams/DMSO.ALK.C.TC.bam"
        ),
        
        non_myc_bamFiles = c(
          "./bams/IAA.ALK.A.TC.bam",
          "./bams/IAA.ALK.B.TC.bam",
          "./bams/IAA.ALK.C.TC.bam"
        )
      )
    }
    data
  })
  
  
  # Render gene table with filtering enabled
  output$gene_table <- DT::renderDataTable({
    DT::datatable(
      isolate(coverage_data()),
      selection = "single",
      filter = "top",
      escape = FALSE,
      options = list(
        pageLength = 10,
        fixedColumns = list(leftColumns = 1),
        search = list(regex = TRUE, caseInsensitive = TRUE),
        lengthMenu = list(c(5, 10, 25, 50, 100, -1), c(5, 10, 25, 50, 100, "All")),
        scrollX = TRUE
      )
    )
  })
  
  # When switching data source, we do not reprint the table to keep filtering 
  # etc. but only replace the underlying data source
  proxy <- dataTableProxy("gene_table")
  
  prev_selected_gene <- reactiveVal(NULL)
  
  observeEvent(input$gene_table_rows_selected, {
    selected_row <- input$gene_table_rows_selected
    data <- isolate(coverage_data())
    
    if (!is.null(selected_row) && length(selected_row) > 0) {
      selected_gene <- data$gene_id[selected_row]  # Save the selected Gene name
      prev_selected_gene(selected_gene)
    }
  })
  
  # Observe when dataset changes and restore selection based on Gene name
  observe({
    req(coverage_data())  # Ensure valid data
    new_data <- coverage_data()
    
    replaceData(proxy,
                new_data,
                resetPaging = FALSE,
                clearSelection = "all")  # Update table
    
    # Restore selection
    selected_gene <- isolate(prev_selected_gene())
    
    if (!is.null(selected_gene)) {
      new_row_index <- IRanges::match(selected_gene, new_data$gene_id)  # Find new row index
      if (!is.na(new_row_index)) {
        selectRows(proxy, new_row_index)  # Select the first matching row
      }
    }
  })
  
  selected_gene = reactive({
    selected_gene <- input$gene_table_rows_selected
    if (length(selected_gene) == 0) {
      return(NULL)
    }
    selected_gene_info <- isolate(coverage_data())[selected_gene, , drop = FALSE]
    selected_gene_info
  })
  
  # Set default zoom range
  zoom_range <- reactiveVal(c(-1, -1))
  
  # Adjust zoom range when brush event occurs
  observeEvent(input$coverage_plot_brush, {
    brush <- input$coverage_plot_brush
    if (!is.null(brush)) {
      zoom_range(c(brush$xmin, brush$xmax))
    }
  })
  
  observeEvent(input$gene_table_rows_selected, {
    zoom_range(c(-1, -1))
  })
  
  observeEvent(input$coverage_plot_reset, {
    session$resetBrush("coverage_plot_brush")
    zoom_range(c(-1, -1))
  })
  
  # Render the plot with exon highlights and coverage tracks
  output$coverage_plot <- renderPlot({
    req(selected_gene())
    
    selected_gene <- selected_gene()
    chr <- as.character(selected_gene$chromosome)
    start <- selected_gene$start_position
    end <- selected_gene$end_position
    str = selected_gene$strand
    
    exons =
      subset(exon_data, gene_id == selected_gene$gene_id)
    
    exons$start = pmax(exons$start, start)
    exons$end = pmin(exons$end, end)
    
    
    # Create BAM file lists
    bam_files = isolate(coverage_bam_files())
    
    # Generate coverage tracks
    myc_track <- calc_coverage(bam_files$myc_bamFiles, chr, start, end, str)
    non_myc_track <- calc_coverage(bam_files$non_myc_bamFiles, chr, start, end, str)
    
    
    tss = ifelse(str == "+", start, end)
    
    coverage_Data = data.frame(x = start:end,
                               myc = myc_track,
                               myc_depletion = non_myc_track)
    
    min_y = min(min(coverage_Data$myc),
                min(coverage_Data$myc_depletion))
    max_y = max(max(coverage_Data$myc),
                max(coverage_Data$myc_depletion))
    range_y = max_y - min_y
    
    plot = ggplot(coverage_Data) +
      geom_line(aes(x = x, y = myc_depletion, color = "MYC Depletion")) +
      geom_line(aes(x = x, y = myc, color = "MYC")) +
      geom_rect(
        data = exons,
        aes(
          xmin = start,
          xmax = end,
          ymin = -range_y * 0.3,
          ymax = -range_y * 0.1,
          fill = "Exons"
        ),
        # Adjust y-values to position exons
      ) +
      scale_fill_manual(values = c(
        "Exons" = "lightgrey"
      )) +
      geom_segment(
        x = tss,
        xend = tss,
        y = -range_y * 0.05,
        yend = max_y,
        linetype = "dashed",
        color = "grey"
      ) +
      annotate(
        "text",
        x = tss,
        y = max_y * 1.05,
        label = "TSS",
        hjust = 0.5,
        vjust = 0,
        size = 5,
        color = "grey"
      ) +
      
      labs(
        title = paste(selected_gene$gene_name, " Coverage"),
        x = "Position",
        y = "Coverage",
        color = "",
        fill = ""
      ) +
      theme_minimal() +
      scale_x_continuous(limits = c(start, end))
    
    
    plot
  })
  
  # Render the selected region with peaks
  output$start_coverage_plot <- renderPlot({
    req(selected_gene())
    
    zoomed_start <- zoom_range()[1]
    zoomed_end <- zoom_range()[2]
    
    selected_gene <- selected_gene()
    chr <- as.character(selected_gene$chromosome)
    start <- selected_gene$start_position
    end <- selected_gene$end_position
    str = selected_gene$strand
    
    tss = ifelse(str == "+", start, end)
    
    if (zoomed_start == -1 & zoomed_end == -1) {
      middle_aoi = tss
      
      start = middle_aoi - 1000
      end = middle_aoi + 1000
      title = "Promoter Region"
    } else {
      start = zoomed_start
      end = zoomed_end
      title = "Selected Region"
    }
    
    
    # Create BAM file lists
    bam_files = isolate(coverage_bam_files())
    
    # Generate coverage tracks
    myc_track <- calc_coverage(bam_files$myc_bamFiles, chr, start, end, str)
    non_myc_track <- calc_coverage(bam_files$non_myc_bamFiles, chr, start, end, str)
    
    coverage_Data = data.frame(x = start:end,
                               myc = myc_track,
                               myc_depletion = non_myc_track)
    
    min_y = min(min(coverage_Data$myc),
                min(coverage_Data$myc_depletion))
    max_y = max(max(coverage_Data$myc),
                max(coverage_Data$myc_depletion))
    range_y = max_y - min_y
    
    
    current_range = GRanges(seqnames = chr,
                            ranges = IRanges(start = start, end = end))
    # calc overlap
    overlaps = findOverlaps(current_range, annotated_peaks_range)
    
    overlaps <- as.data.frame(overlaps)
    contained <- overlaps$subjectHits
    
    peaks_in_range = as.data.frame(annotated_peaks_range[overlaps$subjectHits])
    
    peaks_in_range$start = pmax(peaks_in_range$start, start)
    peaks_in_range$end = pmin(peaks_in_range$end, end)
    
    plot = ggplot(coverage_Data) +
      geom_line(aes(x = x, y = myc_depletion, color = "MYC Depletion")) +
      geom_line(aes(x = x, y = myc, color = "MYC")) +
      geom_rect(
        data = peaks_in_range,
        aes(
          xmin = start,
          xmax = end,
          ymin = -range_y * 0.3,
          ymax = -range_y * 0.1,
          fill = "MYC ChIP-seq peaks"
        ),
        # Adjust y-values to position exons
      ) +
      scale_fill_manual(values = c("MYC ChIP-seq peaks" = "tomato1")) +
      geom_segment(
        x = tss,
        xend = tss,
        y = -range_y * 0.05,
        yend = max_y,
        linetype = "dashed",
        color = "grey"
      ) +
      annotate(
        "text",
        x = tss,
        y = range_y * 1.1,
        label = "TSS",
        hjust = 0.5,
        vjust = 0,
        size = 5,
        color = "grey"
      ) +
      labs(
        title = title,
        x = "Position",
        y = "Coverage",
        color = "",
        fill = ""
      ) +
      theme_minimal() +
      scale_x_continuous(limits = c(start, end))
    
    plot
  })
  
  
  ### Bursting Data Viewer Tab
  output$bursting_table <- DT::renderDataTable({
    DT::datatable(
      bursting_data,
      selection = "none",
      filter = "top",
      escape = FALSE,
      options = list(
        pageLength = 10,
        fixedColumns = list(leftColumns = 1),
        search = list(regex = TRUE, caseInsensitive = TRUE),
        lengthMenu = list(c(5, 10, 25, 50, 100, -1), c(5, 10, 25, 50, 100, "All"))
      )
    )
  })
  
  filtered_bursting_data_indices = reactive({
    input$bursting_table_rows_all
  })
  
  output$bursting_scatter_plot = renderPlot({
    data = bursting_data[filtered_bursting_data_indices(), ]
    
    data$burst_size = pmax(data$burst_size, -5)
    data$burst_size = pmin(data$burst_size, 5)
    data$burst_frequency = pmax(data$burst_frequency, -5)
    data$burst_frequency = pmin(data$burst_frequency, 5)
    
    data$density = density2d(data$burst_size, data$burst_frequency)
    ggplot(data, aes(x = burst_size, y = burst_frequency)) +
      geom_point(data = data[data$MYC_signature == "Other", ], aes(color = density), size = 1.5) +
      scale_color_gradient(
        low = "lightblue",
        high = "darkblue",
        name = "Density"
      ) +
      new_scale_color() +
      geom_point(data = data[data$MYC_signature == "MYC_signature", ],
                 aes(color = MYC_signature),
                 size = 1.5) +
      scale_color_manual(values = c("MYC_signature" = "red"),
                         name = "") +
      labs(x = "-log2(Fold Change Burst Size MYC vs. MYC Depletion)", y = "-log2(Fold Change Burst Size MYC vs. MYC Depletion)") +
      theme_minimal()
  })
  
  
  ### Global Volcano/Box Comparison Tab
  ebox_choices = c(ebox_motifs, "Main Motifs", "Main Motifs + CANNTG")
  
  peak_choices = c(ebox_choices, "CpG Islands", "TATA boxes", "Distance to TSS")
  
  global_plot_category_getter = function(comparison_value, current_category) {
    new_choices <- switch(
      comparison_value,
      "Regulatory LFC" = c("MYC signature genes", "Peak Count", peak_choices),
      "MYC Coverage" = peak_choices,
      "Bursting Difference" = c("MYC signature genes", "Peak Count", peak_choices),
      "Bursting Size" = c("MYC signature genes", "Peak Count", peak_choices),
      "Bursting Frequency" = c("MYC signature genes", "Peak Count", peak_choices),
    )
    if (!current_category %in% new_choices) {
      current_category = NULL
    }
    list(choices = new_choices, current_category = current_category)
  }
  
  shared_selection_values = reactiveValues(
    data_source = "",
    comparison_value = "",
    bursting_selection = "",
    category = "",
    which_peaks = "",
    peak_slider = 0,
    x_min = -Inf,
    x_max = Inf
  )
  
  global_plots_data_getter = data_with_peak_selection_getter(
    "global_plots",
    global_plot_category_getter,
    filtered_bursting_data_indices,
    shared_selection_values
  )
  
  output$volcano_plot <- renderPlot({
    selected_gene <- input$gene_table_rows_selected
    
    data <- global_plots_data_getter$data_with_peaks()
    category = global_plots_data_getter$category()
    
    # Define the thresholds
    logPadj_threshold <- 1
    log2FoldChange_threshold <- 1
    
    # Extend the range for annotation space
    x_range <- range(data$log2FoldChange, na.rm = TRUE)
    y_range <- range(data$logPadj_capped, na.rm = TRUE)
    
    # Count genes in each region
    counts <- data %>% mutate(
      region = case_when(
        logPadj_capped >= logPadj_threshold &
          log2FoldChange >= log2FoldChange_threshold ~ "Upper Right",
        logPadj_capped >= logPadj_threshold &
          log2FoldChange <= -log2FoldChange_threshold ~ "Upper Left",
        logPadj_capped < logPadj_threshold &
          log2FoldChange >= log2FoldChange_threshold ~ "Lower Right",
        logPadj_capped < logPadj_threshold &
          log2FoldChange <= -log2FoldChange_threshold ~ "Lower Left",
        TRUE ~ "Other"
      )
    ) %>%
      group_by(region, MYC_signature) %>%
      summarize(count = n(), .groups = "drop") %>%
      complete(
        region = c("Upper Right", "Upper Left", "Lower Right", "Lower Left"),
        MYC_signature = c("MYC_signature", "Other"),
        fill = list(count = 0)
      )
    
    threshold <- data$significance == "Significant" &
      abs(data$log2FoldChange) > 1
    
    data = data %>%
      arrange(!!sym(category))
    
    plot = ggplot(data, aes(x = log2FoldChange, y = logPadj_capped)) +
      geom_point(data = data %>%
                   filter(is.na(!!sym(category))),
                 size = 1.5,
                 aes(color = "NA")) +
      scale_color_manual(
        values = c("NA" = "grey"),
        labels = c("Gene without assigned peak"),
        name = ""
      ) +
      new_scale_color() +
      geom_point(data = data %>%
                   filter(!is.na(!!sym(category))),
                 aes(color = !!sym(category)),
                 size = 1.5) +
      geom_hline(yintercept = logPadj_threshold,
                 linetype = "dashed",
                 color = "blue") +
      geom_vline(
        xintercept = c(-log2FoldChange_threshold, log2FoldChange_threshold),
        linetype = "dashed",
        color = "blue"
      ) +
      labs(x = "-log2(Fold Change MYC vs. MYC Depletion)", y = "-log10(p-adj) (capped at 100)") +
      
      # Extend the coordinate limits slightly to allow space for annotations
      coord_cartesian(
        xlim = c(x_range[1] - 1, x_range[2] + 1),
        ylim = c(y_range[1] - 1, y_range[2] + 1)
      ) +
      
      # Add corner annotations
      annotate(
        "text",
        x = x_range[2] + 0.5,
        y = y_range[2] + 0.5,
        label = paste(
          "Upper Right\nMYC Sig:",
          counts$count[counts$region == "Upper Right" &
                         counts$MYC_signature == "MYC_signature"],
          "\nOther:",
          counts$count[counts$region == "Upper Right" &
                         counts$MYC_signature == "Other"]
        ),
        color = "black",
        size = 3,
        hjust = 0
      ) +
      
      annotate(
        "text",
        x = x_range[1] - 0.5,
        y = y_range[2] + 0.5,
        label = paste(
          "Upper Left\nMYC Sig:",
          counts$count[counts$region == "Upper Left" &
                         counts$MYC_signature == "MYC_signature"],
          "\nOther:",
          counts$count[counts$region == "Upper Left" &
                         counts$MYC_signature == "Other"]
        ),
        color = "black",
        size = 3,
        hjust = 1
      ) +
      
      annotate(
        "text",
        x = x_range[2] + 0.5,
        y = y_range[1] - 1.5,
        label = paste(
          "Lower Right\nMYC Sig:",
          counts$count[counts$region == "Lower Right" &
                         counts$MYC_signature == "MYC_signature"],
          "\nOther:",
          counts$count[counts$region == "Lower Right" &
                         counts$MYC_signature == "Other"]
        ),
        color = "black",
        size = 3,
        hjust = 0
      ) +
      
      annotate(
        "text",
        x = x_range[1] - 0.5,
        y = y_range[1] - 1.5,
        label = paste(
          "Lower Left\nMYC Sig:",
          counts$count[counts$region == "Lower Left" &
                         counts$MYC_signature == "MYC_signature"],
          "\nOther:",
          counts$count[counts$region == "Lower Left" &
                         counts$MYC_signature == "Other"]
        ),
        color = "black",
        size = 3,
        hjust = 1
      )
    
    if (category == "MYC_signature") {
      plot = plot +
        scale_color_manual(
          values = c(
            "Other" = "black",
            "MYC_signature" = "red"
          ),
          labels = c("MYC signature genes", "Other genes"),
          name = "Gene Category"
        )
    } else if (category == "CpG_island") {
      plot = plot +
        scale_color_manual(
          values = c("FALSE" = "black", "TRUE" = "red"),
          labels = c("No CpG Island", "CpG Island"),
          name = "Value"
        )
    } else if (category == "distanceToTSSCategories") {
      plot = plot +
        scale_color_manual(
          values = c("black", "#990000", "red", "red", "#990000", "black"),
          # Add NA value color
          name = "Distance to TSS"
        )
    } else  {
      plot = plot  +
        scale_color_gradient(low = "blue",
                             high = "red",
                             name = "Value")
    }
    
    
    if (length(selected_gene()) != 0) {
      selected_gene = data %>% filter(gene_id == selected_gene()$gene_id)
      plot <- plot +
        new_scale_color() +
        geom_point(data = selected_gene,
                   aes(color = "Selected gene"),
                   size = 1.5) +
        geom_text_repel(
          data = selected_gene,
          aes(label = gene_name),
          color = "green",
          size = 3,
          max.overlaps = 10
        ) +
        scale_color_manual(values = c("Selected gene" = "green"),
                           name = "")
    }
    
    plot = plot + theme_minimal()
    
    
    # Print the plot
    plot
  })
  
  output$global_box_plot <- renderPlot({
    selected_gene <- input$gene_table_rows_selected
    
    category = global_plots_data_getter$category()
    comparison_value = global_plots_data_getter$comparison_value()
    
    if (global_plots_data_getter$which_peaks() == "All Peaks" &
        comparison_value == "coverage_myc") {
      d <- global_plots_data_getter$filtered_peak_data()
    } else {
      d <- global_plots_data_getter$data_with_peaks()
    }
    
    d = d %>% filter(!is.na(!!sym(comparison_value)))
    
    category_levels <- sort(unique(d[[category]]))
    if (any(is.na(d[[category]]))) {
      d[[category]] <- ifelse(is.na(d[[category]]), "No Assigned Peak", as.character(d[[category]]))
      category_levels <- c("No Assigned Peak", as.character(category_levels))
    }
    
    d[[category]] <- factor(d[[category]], levels = as.character(category_levels))
    
    counts <- d %>%
      group_by(!!sym(category)) %>%  # Use !!sym for dynamic column names
      summarise(n = paste("n=", n(), sep = ""))
    
    min_y = min(d[[comparison_value]], na.rm = TRUE)
    max_y = max(d[[comparison_value]], na.rm = TRUE)
    range_y = max_y - min_y
    
    sequential_comparisons <- list()
    
    if (length(category_levels) > 1) {
      for (i in 1:(length(category_levels) - 1)) {
        sequential_comparisons[[i]] <- c(as.character(category_levels[i]),
                                         as.character(category_levels[i + 1]))
      }
    }
    
    # Generate the boxplot
    plot = ggplot(d, aes(
      x = !!sym(category),
      group = !!sym(category),
      y = !!sym(comparison_value)
    )) +  # Use !!sym to dynamically reference columns
      geom_boxplot(data = d[d[[category]] == "No Assigned Peak", ], color = "grey") +
      geom_boxplot(data = d[d[[category]] != "No Assigned Peak", ]) +
      
      labs(x = global_plots_data_getter$category_name(),
           y = global_plots_data_getter$comparison_name()) +
      
      # Add counts as text labels
      geom_text(
        data = counts,
        # Use counts summarised above
        aes(
          x = !!sym(category),
          y = min_y - range_y * 0.2,
          label = n
        ),
        # Place text above the boxes
        vjust = -1,
        size = 4
      ) +
      scale_x_discrete(limits = levels(d[[category]])) +
      
      
      stat_compare_means(
        method = "wilcox",
        label = "p.signif",
        comparisons = sequential_comparisons,
        hide.ns = FALSE,
        label.y = max_y + range_y * 0.1
      ) +
      theme_minimal()
    
    if (length(selected_gene()) != 0) {
      selected_gene = d %>% filter(gene_id == selected_gene()$gene_id)
      plot = plot +
        geom_point(data = selected_gene,
                   size = 1.5,
                   color = "green")
    }
    plot
  })
  
  # Observe selected points and extract gene names by index
  output$selected_genes <- renderPrint({
    data <- global_plots_data_getter$data_with_peaks()
    
    selected <- brushedPoints(data, input$plot_brush)
    if (nrow(selected) == 0) {
      cat("No genes selected")
    } else {
      selected %>%
        arrange(gene_name) %>%
        pull(gene_name) %>%
        cat(sep = "\n")
    }
  })
  
  
  ### Data Source Comparison Tab
  LFC_data <- reactive({
    if (input$LFC_data_source == "exonic Talk-seq") {
      talk_seq_data
    } else if (input$LFC_data_source == "exonic TC Talk-seq") {
      tc_talk_seq_data
    } else if (input$LFC_data_source == "intronic Talk-seq") {
      intronic_talk_seq_data
    } else {
      intronic_tc_talk_seq_data
    }
  })
  
  single_cell_data <- reactive({
    if (input$single_cell_source == "scSLAM-seq") {
      slam_seq_data
    } else {
      new_slam_seq_data
    }
  })
  
  LFC_merged_data = reactive({
    other_data <- LFC_data()
    
    single_cell_data = single_cell_data()
    
    names(other_data)[names(other_data) == "log2FoldChange"] = "log2FoldChangeOther"
    names(single_cell_data)[names(single_cell_data) == "log2FoldChange"] = "log2FoldChangeSingleCell"
    
    other_data_subset = other_data[, c("gene_id", "log2FoldChangeOther")]
    single_cell_data_subset = single_cell_data[, c("gene_id",
                                                   "gene_name",
                                                   "log2FoldChangeSingleCell",
                                                   "MYC_signature")]
    
    
    
    merged = merge(single_cell_data_subset, other_data_subset, by = "gene_id")
    
    max_peak_score_data = annotated_peaks %>%
      group_by(gene_id) %>%
      summarize(max_peak_score = max(peak_score))
    
    merged = merge(merged, max_peak_score_data, by = "gene_id")
    
    merged <- merged[order(merged$max_peak_score), ]
  })
  
  output$LFC_comparison_plot <- renderPlot({
    merged = LFC_merged_data()
    
    gene_counts = table(annotated_peaks$gene_id)
    
    merged$peak_count = as.numeric(gene_counts[match(merged$gene_id, names(gene_counts))])
    
    merged$peak_count[is.na(merged$peak_count)] <- 0
    
    merged$peak_category <- ifelse(merged$peak_count == 0, "no peak", "1+ peaks")
    
    #merged = subset(merged, peak_count > 0)
    
    max_abs_log2FoldChangeOther <- max(abs(merged$log2FoldChangeOther), na.rm = TRUE)
    max_abs_log2FoldChangeSingleCell <- max(abs(merged$log2FoldChangeSingleCell), na.rm = TRUE)
    
    # Find the overall maximum absolute value
    max_abs_value <- max(max_abs_log2FoldChangeOther,
                         max_abs_log2FoldChangeSingleCell)
    
    merged$density = density2d(merged$log2FoldChangeSingleCell,
                               merged$log2FoldChangeOther)
    
    plot <- ggplot(merged,
                   aes(x = log2FoldChangeSingleCell, y = log2FoldChangeOther)) +
      
      # Plot the non-MYC signature genes first (background)
      geom_point(data = merged[merged$MYC_signature == "Other", ], aes(color = density), size = 1.5) +
      
      # Use a color gradient for density of "Other" genes
      scale_color_gradient(
        low = "lightblue",
        high = "darkblue",
        name = "Density (Other genes)"
      ) +
      
      new_scale_color() +
      
      # Plot the MYC signature genes on top with fixed red color and create a label for the legend
      geom_point(data = merged[merged$MYC_signature == "MYC_signature", ],
                 aes(color = "MYC_signature"),
                 size = 1.5) +
      scale_color_manual(values = c("MYC_signature" = "red"),
                         name = "") +
      
      # Add a trendline
      geom_smooth(method = "lm",
                  color = "blue",
                  se = FALSE) + # 'lm' for a linear model
      
      geom_smooth(
        data = subset(merged, MYC_signature == "MYC_signature"),
        aes(x = log2FoldChangeSingleCell, y = log2FoldChangeOther),
        method = "lm",
        color = "red",
        se = FALSE
      ) +
      
      # Fix the x and y axis limits
      xlim(-max_abs_value - 0.5, max_abs_value + 0.5) +
      ylim(-max_abs_value - 0.5, max_abs_value + 0.5) +
      
      # Add a line with slope = 1 and intercept = 0
      geom_abline(
        slope = 1,
        intercept = 0,
        color = "black",
        linetype = "dashed"
      ) +
      
      # Fix the aspect ratio to ensure equal scaling
      coord_fixed(ratio = 1) +
      
      # Customize plot appearance
      theme_minimal() +
      
      labs(x = "scSLAM-seq data LFC", y = "Talk-seq data LFC")
    
    
    
    
    # Add the selected gene layer only if a gene is selected
    if (length(selected_gene()) != 0) {
      plot <- plot +
        new_scale_color() +
        geom_point(data = merged[merged$gene_id == selected_gene()$gene_id, ],
                   aes(color = "Selected gene"),
                   size = 1.5) +
        geom_text_repel(
          data = merged[merged$gene_id == selected_gene()$gene_id, ],
          aes(label = gene_name),
          color = "green",
          size = 3,
          max.overlaps = 10
        ) +
        scale_color_manual(values = c("Selected gene" = "green"),
                           name = "")
    }
    plot
  })
  
  # Observe selected points and extract gene names by index
  output$LFC_selected_genes <- renderPrint({
    data <- LFC_merged_data()
    
    selected <- brushedPoints(data, input$LFC_plot_brush)
    if (nrow(selected) == 0) {
      cat("No genes selected")
    } else {
      selected %>%
        arrange(gene_name) %>%
        pull(gene_name) %>%
        cat(sep = "\n")
    }
  })
  
  ### Motif Box Comparison tab
  motif_plot_category_getter = function(a, cur_choice) {
    choices = c("None", "CpG Islands", "TATA boxes")
    if (!cur_choice %in% choices) {
      cur_choice = NULL
    }
    list(choices = choices, current_category = cur_choice)
  }
  
  motif_plot_data_getter = data_with_peak_selection_getter(
    "motif",
    motif_plot_category_getter,
    filtered_bursting_data_indices,
    shared_selection_values
  )
  
  
  
  output$motif_plot <- renderPlot({
    selected_gene <- input$gene_table_rows_selected
    
    category = motif_plot_data_getter$category()
    comparison_value = motif_plot_data_getter$comparison_value()
    
    if (motif_plot_data_getter$which_peaks() == "All Peaks" &
        comparison_value == "coverage_myc") {
      data <- motif_plot_data_getter$filtered_peak_data()
    } else {
      data <- motif_plot_data_getter$data_with_peaks()
    }
    
    data = data %>% filter(!is.na(!!sym(comparison_value)))
    
    data$TATA = data$TATA > 0
    
    data$motif = "No Assigned Peaks"
    data[!is.na(data$sum_eboxes), ]$motif = ebox_motifs[apply(data[!is.na(data$sum_eboxes), ebox_motifs], 1, which.max)]
    data$motif = ifelse(
      is.na(data$sum_eboxes),
      "No Assigned Peaks",
      ifelse(
        data$sum_eboxes > 1,
        "2+ E-boxes",
        ifelse(data$sum_eboxes == 0, "No E-box", data$motif)
      )
    )
    
    if (any(is.na(data$sum_eboxes))) {
      data$motif = factor(data$motif,
                          levels = c("No Assigned Peaks", "No E-box", ebox_motifs, "2+ E-boxes"))
    } else {
      data$motif = factor(data$motif,
                          levels = c("No E-box", ebox_motifs, "2+ E-boxes"))
    }
    
    ggplot_colors <- scales::hue_pal()(length(ebox_motifs))
    
    # Generate lighter and darker versions
    lighter_colors <- lighten(ggplot_colors, amount = 0.3)  # Increase brightness
    darker_colors  <- darken(ggplot_colors, amount = 0.3)   # Decrease brightness
    
    sub_colors <- as.vector(rbind(lighter_colors, darker_colors))
    
    if (category != "None") {
      data$group <- interaction(data$motif, data[[category]])
      
      median_baseline = data %>%
        filter(data$motif == "No E-box") %>%
        group_by(!!sym(category)) %>%
        mutate(median_baseline = median(!!sym(comparison_value))) %>%
        select(!!sym(category), median_baseline) %>%
        arrange(!!sym(category)) %>%
        unique() %>%
        pull(median_baseline)
      
      
      counts <- data %>%
        group_by(motif, !!sym(category)) %>%
        summarise(n = n(), .groups = "drop") %>%
        group_by(motif) %>%
        summarise(n = paste("n=", paste(n, collapse = ", "), sep = ""))
      
      range_values = data %>%
        group_by(motif, !!sym(category)) %>%
        mutate(Q1 = quantile(!!sym(comparison_value), 0.25),
               Q3 = quantile(!!sym(comparison_value), 0.75)) %>%
        mutate(IQR = Q3 - Q1) %>%
        mutate(min_y = Q1 - 1.5 * IQR, max_y = Q3 + 1.5 * IQR) %>%
        select(Q1, Q3, IQR, min_y, max_y)
      
      min_y = max(min(data[[comparison_value]]), min(range_values$min_y))
      max_y = min(max(data[[comparison_value]]), max(range_values$max_y))
      
      range_y = max_y - min_y
      
      
      plot = ggplot(data, aes(x = motif, y = !!sym(comparison_value))) +
        geom_boxplot(
          data = data[data$motif == "No Assigned Peaks", ],
          color = "white",
          outlier.shape = NA
        ) +
        geom_boxplot(
          data = data[data$motif == "No E-box", ],
          aes(group = group),
          color = "white",
          outlier.shape = NA
        ) +
        geom_point(
          data = data[data$motif != "No E-box" &
                        data$motif != "No Assigned Peaks", ],
          aes(color = group, fill = !!sym(category)),
          size = 0.5,
          alpha = 0.1,
          position = position_jitterdodge(jitter.width = 0.2)
        ) +
        scale_color_manual(
          values = c(
            lighter_colors,
            "chocolate2",
            darker_colors,
            "chocolate4",
            "black",
            "black"
          ),
          guide = "none"
        ) +
        scale_fill_manual(values = c("white", "white"), guide = "none") +
        new_scale_color() +
        geom_boxplot(data = data[data$motif == "No E-box", ],
                     aes(color = group),
                     outlier.shape = NA) +
        geom_hline(
          yintercept = median_baseline,
          linetype = "dashed",
          color = c("azure4", "black")
        ) +
        scale_color_manual(
          values = c("azure4", "black"),
          labels = c("FALSE", "TRUE"),
          name = "",
          guide = "legend"
        ) +
        new_scale_color() +
        geom_boxplot(data = data[data$motif != "No E-box", ],
                     aes(color = group),
                     outlier.shape = NA) +
        scale_color_manual(
          values = c(
            lighter_colors,
            "chocolate2",
            darker_colors,
            "chocolate4"
          ),
          guide = "none"
        ) +
        theme_minimal() +
        labs(y = motif_plot_data_getter$comparison_name()) +
        scale_x_discrete(limits = levels(data$motif)) +
        theme(plot.title = element_text(hjust = 0.5)) +
        theme(
          panel.grid = element_blank(),
          axis.line = element_line(color = "black"),
          axis.ticks = element_line(color = "black", size = 1),
        )
      
      
      if (length(selected_gene()) != 0) {
        selected_gene = data[data$gene_id == selected_gene()$gene_id, ]
        if (is.na(selected_gene[[category]])) {
          selected_gene[[category]] = 0.5
        }
        plot = plot +
          new_scale_color() +
          geom_point(
            data = selected_gene,
            aes(color = "Selected gene"),
            size = 1.5,
            position = position_nudge(x = -0.2 + 0.4 * selected_gene[[category]])
          ) +
          scale_color_manual(values = c("Selected gene" = "green"),
                             name = "")
      }
      
      min_y = min(min_y, selected_gene[[comparison_value]])
      max_y = max(max_y, selected_gene[[comparison_value]])
      
      
      plot = plot +
        stat_compare_means(
          data = data,
          aes(
            x = motif,
            y = !!sym(comparison_value),
            group = !!sym(category)
          ),
          label = "p.signif",
          method = "wilcox.test",
          label.y =  max_y + range_y * 0.1
        ) +
        geom_text(
          data = counts,
          aes(
            x = motif,
            y = min_y - range_y * 0.1,
            label = n
          ),
          size = 4,
          position = position_dodge(width = 0.9),
          #angle = 20
        ) +
        scale_y_continuous(limits = c(min_y - range_y * 0.1, max_y + range_y * 0.1))
      
    } else {
      comparisons <- list(
        c("No E-box", "CACGTG"),
        c("No E-box", "CACGCG"),
        c("No E-box", "CATGCG"),
        c("No E-box", "CATGTG"),
        c("No E-box", "CACGAG"),
        c("No E-box", "CACATG")
      )
      
      if (any(is.na(data$sum_eboxes))) {
        comparisons = append(comparisons, list(c("No Assigned Peaks", "No E-box")), after = 0)
      }
      
      
      median_baseline = data %>%
        filter(data$motif == "No E-box") %>%
        mutate(median_baseline = median(!!sym(comparison_value))) %>%
        pull(median_baseline) %>%
        unique()
      
      
      counts <- data %>%
        group_by(motif) %>%
        summarise(n = paste("n=", n(), sep = ""))
      
      range_values = data %>%
        group_by(motif) %>%
        mutate(Q1 = quantile(!!sym(comparison_value), 0.25),
               Q3 = quantile(!!sym(comparison_value), 0.75)) %>%
        mutate(IQR = Q3 - Q1) %>%
        mutate(min_y = Q1 - 1.5 * IQR, max_y = Q3 + 1.5 * IQR) %>%
        select(Q1, Q3, IQR, min_y, max_y)
      
      min_y = max(min(data[[comparison_value]]), min(range_values$min_y))
      max_y = min(max(data[[comparison_value]]), max(range_values$max_y))
      
      range_y = max_y - min_y
      
      
      plot = ggplot(data, aes(x = motif, y = !!sym(comparison_value))) +
        geom_boxplot(
          data = data[data$motif == "No Assigned Peaks", ],
          outlier.shape = NA,
          color = "grey"
        ) +
        geom_boxplot(data = data[data$motif == "No E-box", ], outlier.shape = NA) +
        geom_jitter(
          data = data[as.character(data$motif) %in% ebox_motifs, ],
          aes(color = motif),
          width = 0.2,
          alpha = 0.1
        ) +
        geom_jitter(
          data = data[data$motif == "2+ E-boxes", ],
          width = 0.2,
          alpha = 0.1,
          color = "chocolate3"
        ) +
        geom_hline(
          yintercept = median_baseline,
          linetype = "dashed",
          color = "black"
        ) +
        geom_boxplot(
          data = data[data$motif == "No Peak", ],
          outlier.shape = NA,
          color = "grey"
        ) +
        geom_boxplot(data = data[as.character(data$motif) %in% ebox_motifs, ],
                     outlier.shape = NA,
                     aes(color = motif)) +
        geom_boxplot(
          data = data[data$motif == "2+ E-boxes", ],
          outlier.shape = NA,
          color = "chocolate3"
        ) +
        theme_minimal() +
        labs(y = motif_plot_data_getter$comparison_name()) +
        guides(color = "none") +
        scale_x_discrete(limits = levels(data$motif)) +
        theme(plot.title = element_text(hjust = 0.5)) +
        theme(
          panel.grid = element_blank(),
          axis.line = element_line(color = "black"),
          axis.ticks = element_line(color = "black", size = 1),
        )
      
      
      if (length(selected_gene()) != 0) {
        selected_gene = data[data$gene_id == selected_gene()$gene_id, ]
        plot = plot +
          new_scale_color() +
          geom_point(data = selected_gene,
                     aes(color = "Selected gene"),
                     size = 1.5) +
          scale_color_manual(values = c("Selected gene" = "green"),
                             name = "")
        
        min_y = min(min_y, selected_gene[[comparison_value]])
        max_y = max(max_y, selected_gene[[comparison_value]])
      }
      
      
      plot = plot +
        stat_compare_means(
          label = "p.signif",
          comparisons = comparisons,
          method = "wilcox.test",
          label.y =  seq(
            max_y + range_y * 0.05,
            max_y + range_y * 0.5,
            length.out = length(levels(data$motif))
          ),
          tip.length = 0.01
        ) +
        geom_text(
          data = counts,
          aes(
            x = motif,
            y = min_y - range_y * 0.1,
            label = n
          ),
          size = 4,
          position = position_dodge(width = 0.9),
          #angle = 20
        ) +
        scale_y_continuous(limits = c(min_y - range_y * 0.1, max_y + range_y * (
          0.6 + 0.2 * (comparison_value == "bursting_diff")
        )))
    }
    
    
    plot
  })
  
  
  
  ### Global Scatter COmparison tab
  regulation_data = reactive({
    data = if (input$scatter_data_source == "exonic Talk-seq") {
      talk_seq_data
    } else if (input$scatter_data_source == "exonic TC Talk-seq") {
      tc_talk_seq_data
    } else if (input$scatter_data_source == "intronic Talk-seq") {
      intronic_talk_seq_data
    } else if (input$scatter_data_source == "intronic TC Talk-seq") {
      intronic_tc_talk_seq_data
    } else if (input$scatter_data_source == "scSLAM-seq")  {
      slam_seq_data
    } else {
      new_slam_seq_data
    }
    data
  })
  
  
  get_scatter_data = reactive({
    function(metric) {
      data = regulation_data()
      
      if (metric == "Regulatory LFC") {
        data = data %>% select(gene_id, log2FoldChange)
      } else if (metric == "MYC Coverage (Strongest Peak)") {
        data = merge(data, annotated_peaks, by = "gene_id")
        data = data %>% group_by(gene_id) %>% filter(is.na(coverage_myc) |
                                                       max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()
        data = data %>% select(gene_id, coverage_myc)
      } else if (metric == "MYC Coverage (All Peaks)") {
        data = merge(data, annotated_peaks, by = "gene_id")
        data = data %>% group_by(gene_id) %>% mutate(coverage_myc = sum(coverage_myc)) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()
        data = data %>% select(gene_id, coverage_myc)
      } else if (metric == "Promoter Coverage") {
        data = merge(data, promoter_coverages, by = "gene_id")
        data = data %>% select(gene_id, coverage_myc)
        data$coverage_myc = pmax(data$coverage_myc, -50)
        data$coverage_myc = pmin(data$coverage_myc, 400)
      } else if (metric == "Promoter Coverage (Upstream)") {
        data = merge(data, promoter_coverages, by = "gene_id")
        data = data %>% select(gene_id, coverage_myc_upstream)
        data$coverage_myc_upstream = pmax(data$coverage_myc_upstream, -50)
        data$coverage_myc_upstream = pmin(data$coverage_myc_upstream, 400)
      } else if (metric == "Promoter Coverage (Downstream)") {
        data = merge(data, promoter_coverages, by = "gene_id")
        data = data %>% select(gene_id, coverage_myc_downstream)
        data$coverage_myc_downstream = pmax(data$coverage_myc_downstream, -50)
        data$coverage_myc_downstream = pmin(data$coverage_myc_downstream, 400)
      } else if (metric == "Bursting Size") {
        data = merge(data, bursting_data, by = "gene_id")
        data = data %>% select(gene_id, burst_size)
        data$burst_size = pmax(data$burst_size, -3)
        data$burst_size = pmin(data$burst_size, 3)
      } else if (metric == "Bursting Frequency") {
        data = merge(data, bursting_data, by = "gene_id")
        data = data %>% select(gene_id, burst_frequency)
        data$burst_frequency = pmax(data$burst_frequency, -3)
        data$burst_frequency = pmin(data$burst_frequency, 3)
      } else if (metric == "Bursting Difference") {
        data = merge(data, bursting_data, by = "gene_id")
        data = data %>% select(gene_id, bursting_diff)
        data$bursting_diff = pmax(data$bursting_diff, -5)
        data$bursting_diff = pmin(data$bursting_diff, 5)
      }
      
      data
    }
  })
  
  get_metric_name = reactive({
    function(metric) {
      if (metric == "Regulatory LFC") {
        "-log2(Fold Change MYC vs. MYC Depletion)"
      } else if (metric == "MYC Coverage (Strongest Peak)") {
        "Coverage MYC (Strongest Peak)"
      } else if (metric == "MYC Coverage (All Peaks)") {
        "Coverage MYC (All Peaks)"
      } else if (metric == "Promoter Coverage") {
        "Promoter Coverage (capped at -50 & 400)"
      } else if (metric == "Promoter Coverage (Upstream)") {
        "Promoter Coverage (Upstream, capped at -50 & 400)"
      } else if (metric == "Promoter Coverage (Downstream)") {
        "Promoter Coverage (Downstream, capped at -50 & 400)"
      } else if (metric == "Bursting Size") {
        "-log2(Fold Change Burst Size MYC vs. MYC Depletion) (capped at ±3)"
      } else if (metric == "Bursting Frequency") {
        "-log2(Fold Change Burst Frequency MYC vs. MYC Depletion) (capped at ±3)"
      } else if (metric == "Bursting Difference") {
        "-log2(Fold Change Burst Frequency / Size MYC vs. MYC Depletion) (capped at ±5)"
      }
    }
  })
  
  scatter_combined_data = reactive({
    data_x = get_scatter_data()(input$scatter_metric1)
    names(data_x) = c("gene_id", "x")
    data_y = get_scatter_data()(input$scatter_metric2)
    names(data_y) = c("gene_id", "y")
    data = merge(data_x, data_y, by = "gene_id")
    
    signature_data = regulation_data()
    signature_data = signature_data %>% select(gene_id, gene_name, MYC_signature)
    
    data = merge(data, signature_data, by = "gene_id")
  })
  
  output$scatter_plot = renderPlot({
    data = scatter_combined_data()
    
    label_x = get_metric_name()(input$scatter_metric1)
    label_y = get_metric_name()(input$scatter_metric2)
    
    data$density = density2d(data$x, data$y)
    
    correlation1 = cor(data$x, data$y, use = "complete.obs", method = "pearson")
    correlation2 = cor(data$x, data$y, use = "complete.obs", method = "spearman")
    
    x_min = min(data$x, na.rm = TRUE)
    
    y_min = min(data$y, na.rm = TRUE)
    y_max = max(data$y, na.rm = TRUE)
    y_range = y_max - y_min
    
    plot = ggplot(data, aes(x = x, y = y)) +
      
      # Plot the non-MYC signature genes first (background)
      geom_point(data = data[data$MYC_signature == "Other", ], aes(color = density), size = 1.5) +
      
      # Use a color gradient for density of "Other" genes
      scale_color_gradient(
        low = "lightblue",
        high = "darkblue",
        name = "Density (Other genes)"
      ) +
      
      new_scale_color() +
      
      # Plot the MYC signature genes on top with fixed red color and create a label for the legend
      geom_point(data = data[data$MYC_signature == "MYC_signature", ],
                 aes(color = "MYC_signature"),
                 size = 1.5) +
      scale_color_manual(values = c("MYC_signature" = "red"),
                         name = "") +
      theme_minimal() +
      geom_smooth(method = "lm", color = "blue") +
      geom_smooth(data = data[data$MYC_signature == "MYC_signature", ],
                  method = "lm",
                  color = "red") +
      labs(x = label_x, y = label_y) +
      annotate(
        "text",
        x = x_min,
        y = y_max + 0.1 * y_range,
        label = paste0("r = ", round(correlation1, 2), ", p = ", round(correlation2, 2)),
        hjust = 0,
        vjust = 0,
        size = 4,
        color = "black"
      )
    
    # Add the selected gene layer only if a gene is selected
    if (length(selected_gene()) != 0) {
      plot <- plot +
        new_scale_color() +
        geom_point(data = data[data$gene_id == selected_gene()$gene_id, ],
                   aes(color = "Selected gene"),
                   size = 1.5) +
        geom_text_repel(
          data = data[data$gene_id == selected_gene()$gene_id, ],
          aes(label = gene_name),
          color = "green",
          size = 3,
          max.overlaps = 10
        ) +
        scale_color_manual(values = c("Selected gene" = "green"),
                           name = "")
    }
    
    plot
  })
  
  # Observe selected points and extract gene names by index
  output$scatter_selected_genes <- renderPrint({
    data <- scatter_combined_data()
    
    selected <- brushedPoints(data, input$scatter_plot_brush)
    if (nrow(selected) == 0) {
      cat("No genes selected")
    } else {
      selected %>%
        arrange(gene_name) %>%
        pull(gene_name) %>%
        cat(sep = "\n")
    }
  })
  
}



# Run the Shiny app
shinyApp(ui, server)