# SPDX-FileCopyrightText: 2025 Julius Deynet <julius.deynet@stud-mail.uni-wuerzburg.de>
#
# SPDX-License-Identifier: GPL-3.0-or-later

suppressPackageStartupMessages({
  library(GenomicAlignments)
  library(Rsamtools)
  library(TxDb.Hsapiens.UCSC.hg19.knownGene)
  library(dplyr)
  library(ggplot2)
  
  library(GenomicRanges)
  library(rtracklayer)
  library(Biostrings)
  library(ggpubr)
  library(ggplot2)
  library(BSgenome.Hsapiens.UCSC.hg38)
  library(data.table)
})
input_upstream <- vroom::vroom("./coverage_JH477_A375_MYC_promoter_upstream_Input.bed",
                      col_names = c(
                        "chr", "start", "end", "gene_id", "peak_score", "strand", "coverage_input"
                      )
)

ctr_upstream <- vroom::vroom("./coverage_JH477_A375_MYC_promoter_upstream_MYC.bed",
                    col_names = c(
                      "chr", "start", "end", "gene_id", "peak_score", "strand", "coverage_ctr"
                    )
)

input_upstream$coverage_input = input_upstream$coverage_input * 940008/1512647

ctr_upstream$coverage_myc_upstream = ctr_upstream$coverage_ctr - input_upstream$coverage_input

ctr_upstream <- ctr_upstream %>% filter(grepl("^[0-9XY]+$", chr))
ctr_upstream$chr = paste0("chr", ctr_upstream$chr)


gr <- GRanges(seqnames = ctr_upstream$chr,
              ranges = IRanges(start = ctr_upstream$start, end = ctr_upstream$end))



session <- browserSession()
genome(session) <- "hg38"

query <- ucscTableQuery(session, table="cpgIslandExt")
cpg_islands_hg38 <- track(query)  # GRanges object


overlaps <- findOverlaps(gr, cpg_islands_hg38)
overlaps_df <- as.data.frame(overlaps)
contained <- overlaps_df$queryHits


ctr_upstream$CpG_island_upstream <- FALSE
ctr_upstream$CpG_island_upstream[contained] <- TRUE


tata_motifs = c("TATAAAA", "TATATAA", "TATAAAT", "TATATAT")
ref_genome <- BSgenome.Hsapiens.UCSC.hg38

sequences <- getSeq(ref_genome, gr)

count_TATA_per_seq <- function(seq, motifs) {
  sum(sapply(motifs, function(motif) countPattern(motif, seq)))
}

counts <- lapply(sequences, count_TATA_per_seq, motifs = tata_motifs)

# Convert the list of counts to a data frame
counts_df <- do.call(rbind, counts)
names(counts_df) <- "TATA"

ctr_upstream <- cbind(ctr_upstream, counts_df)

names(ctr_upstream)[names(ctr_upstream) == "counts_df"] = "TATA_upstream"



input_downstream <- vroom::vroom("./coverage_JH477_A375_MYC_promoter_downstream_Input.bed",
                               col_names = c(
                                 "chr", "start", "end", "gene_id", "peak_score", "strand", "coverage_input_downstream"
                               )
)

ctr_downstream <- vroom::vroom("./coverage_JH477_A375_MYC_promoter_downstream_MYC.bed",
                             col_names = c(
                               "chr", "start", "end", "gene_id", "peak_score", "strand", "coverage_ctr_downstream"
                             )
)

input_downstream$coverage_input_downstream = input_downstream$coverage_input_downstream * 940008/1512647

ctr_downstream$coverage_myc_downstream = ctr_downstream$coverage_ctr_downstream - input_downstream$coverage_input_downstream

ctr_downstream <- ctr_downstream %>% filter(grepl("^[0-9XY]+$", chr))
ctr_downstream$chr = paste0("chr", ctr_downstream$chr)


gr <- GRanges(seqnames = ctr_downstream$chr,
              ranges = IRanges(start = ctr_downstream$start, end = ctr_downstream$end))


overlaps <- findOverlaps(gr, cpg_islands_hg38)
overlaps_df <- as.data.frame(overlaps)
contained <- overlaps_df$queryHits


ctr_downstream$CpG_island_downstream <- FALSE
ctr_downstream$CpG_island_downstream[contained] <- TRUE


tata_motifs = c("TATAAAA", "TATATAA", "TATAAAT", "TATATAT")
ref_genome <- BSgenome.Hsapiens.UCSC.hg38

sequences <- getSeq(ref_genome, gr)

count_TATA_per_seq <- function(seq, motifs) {
  sum(sapply(motifs, function(motif) countPattern(motif, seq)))
}

counts <- lapply(sequences, count_TATA_per_seq, motifs = tata_motifs)

# Convert the list of counts to a data frame
counts_df <- do.call(rbind, counts)
names(counts_df) <- "TATA"

ctr_downstream <- cbind(ctr_downstream, counts_df)

names(ctr_downstream)[names(ctr_downstream) == "counts_df"] = "TATA_downstream"

ctr_upstream = ctr_upstream %>% dplyr::select(gene_id, coverage_myc_upstream, CpG_island_upstream, TATA_upstream)
merged = merge(ctr_upstream, ctr_downstream, by="gene_id")
merged$coverage_ctr_downstream = NULL
merged$coverage_myc = merged$coverage_myc_upstream + merged$coverage_myc_downstream
merged$CpG_island = merged$CpG_island_upstream | merged$CpG_island_downstream
merged$TATA = merged$TATA_upstream + merged$TATA_downstream
merged$chr = NULL
merged$start = NULL
merged$end = NULL
merged$peak_score = NULL
merged$strand = NULL

write.table(merged, "./resources/promoter_coverages_hg_38.tsv", sep = "\t", row.names = FALSE, quote = FALSE)


lfc_data = readRDS("./resources/processed_featureCounts_introns_TC_data.rds")

merged = merge(lfc_data, merged, by="gene_id")


cor(merged$coverage_myc_upstream, merged$coverage_myc_downstream, method="spearman")
cor(merged$coverage_myc_upstream, merged$log2FoldChange, method="spearman")
cor(merged$coverage_myc_downstream, merged$log2FoldChange, method="spearman")
cor(merged$coverage_myc, merged$log2FoldChange, method="spearman")
cor(merged$coverage_myc_upstream, merged$CpG_island_upstream, method="spearman")
cor(merged$coverage_myc_downstream, merged$CpG_island_downstream, method="spearman")
cor(merged$coverage_myc, merged$CpG_island, method="spearman")


ggplot(merged, aes(x=log2FoldChange, y=coverage_myc_upstream)) + geom_point(size = 0.5) + theme_minimal() + geom_smooth(method="lm") + ggtitle("MYC upstream")
ggplot(merged, aes(y=coverage_myc_downstream, x=log2FoldChange)) + geom_point(size = 0.5) + theme_minimal() + geom_smooth(method="lm") + ggtitle("MYC downstream")
ggplot(merged, aes(y=coverage_myc, x=log2FoldChange)) + geom_point(size = 0.5) + theme_minimal() + geom_smooth(method="lm") + ggtitle("MYC combined")


annotated_peaks = as.data.frame(read.delim("./resources/annotated_peaks_hg38_right.tsv"))
lfc_data = readRDS("./resources/processed_featureCounts_introns_TC_data.rds")


merged_peaks = merge(lfc_data, annotated_peaks, by="gene_id")
merged_peaks = merged_peaks %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()


cor(merged_peaks$coverage_myc, merged_peaks$log2FoldChange, method="spearman")
ggplot(merged_peaks, aes(y=coverage_myc, x=log2FoldChange)) + geom_point(size = 0.5) + theme_minimal() + geom_smooth(method="lm") + ggtitle("MYC strongest peak coverage")


bursting_data = as.data.frame(read.csv("./resources/bursting_all_genes.csv"))

names(bursting_data)[names(bursting_data) == "km"] = "burst_frequency"
names(bursting_data)[names(bursting_data) == "B"] = "burst_size"

bursting_data = subset(bursting_data, bestmodel != "no change")

merged_bursting = merge(merged, bursting_data, by.x="gene_name", by.y="gene")

cor(merged_bursting$coverage_myc, merged_bursting$burst_frequency, method="spearman")
cor(merged_bursting$coverage_myc, merged_bursting$burst_size, method="spearman")
cor(merged_bursting$coverage_myc, merged_bursting$burst_frequency - merged_bursting$burst_size, method="spearman")
