# SPDX-FileCopyrightText: 2025 Julius Deynet <julius.deynet@stud-mail.uni-wuerzburg.de>
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys


def parse_gtf(file_path):
    transcripts = {}

    with open(file_path, "r") as file:
        for line in file:
            if line.startswith("#"):
                continue  # Skip header lines
            fields = line.strip().split("\t")
            if fields[2] == "exon":
                attributes = {
                    kv.split(" ")[0]: kv.split(" ")[1].strip('";')
                    for kv in fields[8].split("; ")
                    if kv
                }
                transcript_id = attributes.get("transcript_id", "")

                if transcript_id not in transcripts:
                    transcripts[transcript_id] = []

                transcripts[transcript_id].append(
                    (int(fields[3]), int(fields[4]), fields)
                )

    return transcripts


def extract_introns(transcripts):
    introns = []

    for transcript_id, exons in transcripts.items():
        # Sort exons by start position
        exons.sort()

        for i in range(len(exons) - 1):
            exon_end = exons[i][1]
            next_exon_start = exons[i + 1][0]

            if next_exon_start > exon_end + 1:
                # Create intron as a GTF entry labeled as "exon"
                intron_entry = exons[i][2][:]
                intron_entry[2] = "exon"  # Label as exon
                intron_entry[3] = str(exon_end + 1)  # Start after current exon
                intron_entry[4] = str(next_exon_start - 1)  # End before next exon

                # Keep the same attributes as the original exons
                introns.append("\t".join(intron_entry))

    return introns


def write_introns_to_file(introns, output_file):
    with open(output_file, "w") as out:
        out.write("#!genome-build GRCh38.p10\n")
        out.write("#!genome-version GRCh38\n")
        out.write("#!genome-date 2013-12\n")
        out.write("#!genome-build-accession NCBI:GCA_000001405.25\n")
        out.write("#!genebuild-last-updated 2017-06\n")
        out.writelines("\n".join(introns) + "\n")


if __name__ == "__main__":
    input_gtf = "homo_sapiens.90.gtf"
    output_gtf = "homo_sapiens.90.introns.gtf"

    transcripts = parse_gtf(input_gtf)
    introns = extract_introns(transcripts)
    write_introns_to_file(introns, output_gtf)

    print(f"Intron GTF file saved as {output_gtf}")
