# SPDX-FileCopyrightText: 2025 Julius Deynet <julius.deynet@stud-mail.uni-wuerzburg.de>
#
# SPDX-License-Identifier: GPL-3.0-or-later

library(RColorBrewer)

annotated_peaks = as.data.frame(read.delim("./resources/annotated_peaks_hg38_right.tsv"))
annotated_peaks_top10 =as.data.frame(read.delim("./resources/annotated_peaks_hg38_right_top10.tsv"))
lfc_data = readRDS("./resources/processed_featureCounts_introns_TC_data.rds")
lfc_sc_data = readRDS("./resources/processed_grandR_single_cell_new_data.rds")

check_significance = function(lfc_data, annotated_peaks) {

merged = merge(lfc_data, annotated_peaks, by="gene_id")


merged = merged %>%
  group_by(gene_id) %>%
  mutate(total = as.numeric(any(TATA))) %>%
  ungroup()

unique_genes1 = merged[!duplicated(merged$gene_id), ]

unique_genes1 = unique_genes1[, c("gene_id", "coverage_myc", "log2FoldChange", "total")]


merged = merge(lfc_data, annotated_peaks, by="gene_id")
merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()


unique_genes2 = merged[!duplicated(merged$gene_id), ]
unique_genes2 = unique_genes2[, c("gene_id", "sum_eboxes", "CpG_island", "TATA")]


total = merge(unique_genes1, unique_genes2, by="gene_id")

subset1 = total$coverage_myc[total$total > 0]
subset2 = total$coverage_myc[total$TATA > 0]

wilcox_result <- wilcox.test(subset1, subset2)

# Print p-value
wilcox_result$p.value

}

check_significance(lfc_data, annotated_peaks)
check_significance(lfc_data, annotated_peaks_top10)
check_significance(lfc_sc_data, annotated_peaks)
check_significance(lfc_sc_data, annotated_peaks_top10)

box_for_considered_peaks = function(lfc_data, annotated_peaks, lfc_data_name, annotated_peaks_name, to_compare) {

merged = merge(lfc_data, annotated_peaks, by="gene_id", all.x = TRUE)


merged = merged %>%
  group_by(gene_id) %>%
  mutate(temp = any(!!sym(to_compare))) %>%
  ungroup()

merged[[to_compare]] = merged$temp

unique_genes1 = merged[!duplicated(merged$gene_id), ]
unique_genes1$ebox_exists = ifelse(is.na(unique_genes1$sum_eboxes), "No Assigned Peak", ifelse(unique_genes1$sum_eboxes == 0, "No E-box", "1+ E-boxes"))
unique_genes1$ebox_exists = factor(unique_genes1$ebox_exists, levels = c("No Assigned Peak", "No E-box", "1+ E-boxes"))
unique_genes1 = unique_genes1[, c("gene_id", "log2FoldChange", to_compare, "ebox_exists")]
unique_genes1$category = factor("All Peaks", levels = c("All Peaks", "Strongest Peak"))


merged = merge(lfc_data, annotated_peaks, by="gene_id")
merged = merged %>% group_by(gene_id) %>% filter(is.na(coverage_myc) | max(coverage_myc) == coverage_myc) %>% distinct(gene_id, .keep_all = TRUE) %>% ungroup()

unique_genes2 = merged[!duplicated(merged$gene_id), ]

unique_genes2[[to_compare]] = as.logical(unique_genes2[[to_compare]])

unique_genes2$ebox_exists = ifelse(is.na(unique_genes2$sum_eboxes), "No Assigned Peak", ifelse(unique_genes2$sum_eboxes == 0, "No E-box", "1+ E-boxes"))
unique_genes2$ebox_exists = factor(unique_genes2$ebox_exists, levels = c("No Assigned Peak", "No E-box", "1+ E-boxes"))

unique_genes2 = unique_genes2[, c("gene_id", "log2FoldChange", to_compare, "ebox_exists")]
unique_genes2$category = factor("Strongest Peak", levels = c("All Peaks", "Strongest Peak"))


total = unique_genes1 %>% rbind(unique_genes2)


counts <- total %>%
  group_by(ebox_exists, category) %>%
  summarise(n = paste("n=", n(), sep = ""))

y_min = min(total$log2FoldChange)

ggplot() +
  geom_boxplot(data = total[is.na(total[[to_compare]]), ], aes(x = ebox_exists, y = log2FoldChange), color = "grey") +
  geom_boxplot(data = total[!is.na(total[[to_compare]]), ], aes(x = ebox_exists, y = log2FoldChange, color = category)) +
  labs(title = lfc_data_name,
       x = NULL,
       y = "-log2(Fold Change MYC vs. MYC Depletion)",
       color = "Considered Peaks") +
  scale_color_manual(values = brewer.pal(n = 2, name = "Dark2"), name = "Considered Peaks") +
  
  
   stat_compare_means(data = total,
                      aes(x = ebox_exists, y = log2FoldChange, group = category),
                      label = "p.signif", 
                      method = "wilcox.test"
   )+
  geom_text(
    data = counts,
    aes(x = ebox_exists, y = y_min - 0.5, label = n, group = category),
    size = 4,
    position = position_dodge(width = 0.9)
  ) + 
  guides(color = "none") +
  # set background to white
  theme_minimal() +
  theme(panel.grid = element_blank(), axis.line = element_line(color = "black"), axis.ticks = element_line(color = "black", size = 1),) +
  theme( plot.title = element_text(hjust = 0.5)) 
}

plot = box_for_considered_peaks(lfc_data, annotated_peaks, "intronic TC Talk-seq", "all", "sum_eboxes")
plot
ggplot2::ggsave("./box_lfc_existence_ebox_per_considered_peaks_talk.pdf", plot, width = 9, height = 6)

plot = box_for_considered_peaks(lfc_data, annotated_peaks_top10, "intronic TC Talk-seq", "top 10% of", "sum_eboxes")
plot
ggplot2::ggsave("./box_lfc_existence_ebox_per_considered_peaks_talk_top10.svg", plot, width = 9, height = 6)

plot = box_for_considered_peaks(lfc_sc_data, annotated_peaks, "new scSLAM-seq", "all", "sum_eboxes")
plot
ggplot2::ggsave("./box_lfc_existence_ebox_per_considered_peaks_slam.pdf", plot, width = 9, height = 6)

plot = box_for_considered_peaks(lfc_sc_data, annotated_peaks_top10, "new scSLAM-seq", "top 10% of", "sum_eboxes")
plot
ggplot2::ggsave("./box_lfc_existence_ebox_per_considered_peaks_slam_top10.svg", plot, width = 9, height = 6)
