# SPDX-FileCopyrightText: 2025 Julius Deynet <julius.deynet@stud-mail.uni-wuerzburg.de>
#
# SPDX-License-Identifier: GPL-3.0-or-later

import pandas as pd
import numpy as np
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import StandardScaler
from sklearn.linear_model import LinearRegression
from sklearn.metrics import mean_squared_error, r2_score, mean_absolute_error


# Load your dataset
df = pd.read_csv(
    "./resources/gene_with_strongest_peak_hg38_right.tsv",
    sep="\t",
)  # Adjust accordingly

# Preprocessing
numerical_features = [
    "CACGTG",
    "CACGCG",
    "CATGCG",
    "CATGTG",
    "CACGAG",
    "CACATG",
    "TATA",
    "peak_score",
    "coverage_myc",
    "coverage_myc_ratio",
    "distanceToTSS",
    "peak_count",
]
binary_features = [
    "CpG_island",
]

# Normalize numerical data
scaler = StandardScaler()
df[numerical_features] = scaler.fit_transform(df[numerical_features])

# Concatenate processed features
X = np.hstack((df[numerical_features], df[binary_features]))
y = df["log2FoldChange"].values

# Split data
X_train, X_test, y_train, y_test = train_test_split(
    X, y, test_size=0.2, random_state=42
)

regressor = LinearRegression()
regressor.fit(X_train, y_train)

# Make predictions
y_pred = regressor.predict(X_test)

# Calculate performance metrics
mse = mean_absolute_error(y_test, y_pred)
r2 = r2_score(y_test, y_pred)

print(f"Mean Absolute Error: {mse}")
print(f"R² Score: {r2}")

# Get feature coefficients
feature_importance = pd.DataFrame(
    {"Feature": numerical_features + binary_features, "Coefficient": regressor.coef_}
)
feature_importance = feature_importance.sort_values(by="Coefficient", ascending=False)

print(feature_importance)


y_true = y

y_mean = np.mean(y_true)

mae_baseline = np.mean(np.abs(y_true - y_mean))

print(f"Baseline MAE: {mae_baseline:.4f}")

percent_improvement = ((mae_baseline - mse) / mae_baseline) * 100

print(f"Percent improvement: {percent_improvement:.2f}%")


def compute_mae(X_subset, y_train, y_test):
    regressor = LinearRegression()
    regressor.fit(X_train[:, X_subset], y_train)
    y_pred = regressor.predict(X_test[:, X_subset])
    return mean_absolute_error(y_test, y_pred)


# MAE for individual features
print("MAE for individual features:")
features = []
maes = []
for feature in numerical_features + binary_features:
    feature_idx = np.where(np.array(numerical_features + binary_features) == feature)[
        0
    ][0]
    mae = compute_mae([feature_idx], y_train, y_test)
    features.append('"' + feature + '"')
    maes.append(mae)

feature_mae = pd.DataFrame({"Feature": features, "MAE": maes})
feature_mae = feature_mae.sort_values(by="MAE", ascending=True)
feature_mae["MAE"] = feature_mae["MAE"].astype(str)
print("c(", ",".join(feature_mae["Feature"]), ")")
print("c(", ",".join(feature_mae["MAE"]), ")")

# MAE for pairs of features
print("\nMAE for pairs of features:")
from itertools import combinations

features = []
maes = []

for combo in combinations(numerical_features + binary_features, 2):
    indices = [
        np.where(np.array(numerical_features + binary_features) == feat)[0][0]
        for feat in combo
    ]
    mae = compute_mae(indices, y_train, y_test)
    features.append('"(' + combo[0] + ", " + combo[1] + ')"')
    maes.append(mae)

feature_mae = pd.DataFrame({"Feature": features, "MAE": maes})
feature_mae = feature_mae.sort_values(by="MAE", ascending=True)
# convert mae to str
feature_mae["MAE"] = feature_mae["MAE"].astype(str)
pd.set_option("display.max_rows", None)
print("c(", ",".join(feature_mae["Feature"]), ")")
print("c(", ",".join(feature_mae["MAE"]), ")")
