# SPDX-FileCopyrightText: 2025 Julius Deynet <julius.deynet@stud-mail.uni-wuerzburg.de>
#
# SPDX-License-Identifier: GPL-3.0-or-later

import tensorflow as tf
from tensorflow import keras
from tensorflow.keras.models import Sequential
from tensorflow.keras.layers import Dense, BatchNormalization, Dropout
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import StandardScaler, OneHotEncoder
import pandas as pd
import numpy as np
import keras_tuner as kt
import shap
import matplotlib.pyplot as plt
from sklearn.inspection import permutation_importance

# Load your dataset
df = pd.read_csv(
    "./resources/gene_with_strongest_peak_hg38_right.tsv",
    sep="\t",
)  # Adjust accordingly

# Preprocessing
numerical_features = [
    "CACGTG",
    "CACGCG",
    "CATGCG",
    "CATGTG",
    "CACGAG",
    "CACATG",
    "TATA",
    "peak_score",
    "coverage_myc",
    "coverage_myc_ratio",
    "distanceToTSS",
    "peak_count",
]
binary_features = [
    "CpG_island",
]

# Normalize numerical data
scaler = StandardScaler()
df[numerical_features] = scaler.fit_transform(df[numerical_features])

# Concatenate processed features
X = np.hstack((df[numerical_features], df[binary_features]))
# set minimum value to 0
y = df["log2FoldChange"].values

# Split data
X_train, X_temp, y_train, y_temp = train_test_split(
    X, y, test_size=0.2, random_state=42
)

X_val, X_test, y_val, y_test = train_test_split(
    X_temp, y_temp, test_size=0.2, random_state=42
)


def build_model(hp):
    model = Sequential()

    # First hidden layer
    model.add(
        Dense(
            units=hp.Int("units_1", min_value=32, max_value=256, step=32),
            activation=hp.Choice("activation_1", ["relu", "tanh"]),
            input_shape=(X_train.shape[1],),
        )
    )
    model.add(BatchNormalization())
    model.add(Dropout(rate=hp.Float("dropout_1", min_value=0, max_value=0.5, step=0.1)))

    # Second hidden layer (optional)
    if hp.Boolean("second_layer"):
        model.add(
            Dense(
                units=hp.Int("units_2", min_value=32, max_value=256, step=32),
                activation=hp.Choice("activation_2", ["relu", "tanh"]),
            )
        )
        model.add(BatchNormalization())
        model.add(
            Dropout(rate=hp.Float("dropout_2", min_value=0.1, max_value=0.5, step=0.1))
        )

    # Output layer
    model.add(Dense(1))  # Regression task

    # Compile model
    model.compile(
        optimizer=keras.optimizers.Adam(
            hp.Float("learning_rate", min_value=1e-4, max_value=1e-2, sampling="LOG")
        ),
        loss="mse",
        metrics=["mae"],
    )

    return model


tuner = kt.RandomSearch(
    build_model,
    objective="val_mae",  # Minimize validation mean absolute error
    max_trials=20,  # Number of different hyperparameter combinations to try
    executions_per_trial=2,  # Run each combination 2 times for stability
    directory="my_tuning_results",
    project_name="lfc_prediction",
)


tuner.search(X_train, y_train, validation_data=(X_val, y_val), epochs=50, batch_size=32)


best_hps = tuner.get_best_hyperparameters(num_trials=1)[0]

print(
    f"""
Optimal Hyperparameters:
- First layer units: {best_hps.get('units_1')}
- First layer activation: {best_hps.get('activation_1')}
- First layer dropout: {best_hps.get('dropout_1')}
- Second layer: {'Yes' if best_hps.get('second_layer') else 'No'}
- Second layer units: {best_hps.get('units_2') if best_hps.get('second_layer') else 'N/A'}
- Second layer activation: {best_hps.get('activation_2') if best_hps.get('second_layer') else 'N/A'}
- Second layer dropout: {best_hps.get('dropout_2') if best_hps.get('second_layer') else 'N/A'}
- Learning rate: {best_hps.get('learning_rate')}
"""
)


feature_names = np.array(numerical_features + binary_features)

num_runs = 10
mae_scores = []
losses = []
all_val_losses = []
all_val_maes = []
shap_values_list = []
feature_importance_list = np.empty((len(feature_names), 0))

for run in range(num_runs):
    print(f"\n=== Run {run + 1}/{num_runs} ===")

    best_model = tuner.hypermodel.build(best_hps)

    history = best_model.fit(
        X_train,
        y_train,
        validation_data=(X_val, y_val),
        epochs=20,
        batch_size=32,
        verbose=1,
    )

    # Store validation loss and MAE for this run
    all_val_losses.append(history.history["val_loss"])
    all_val_maes.append(history.history["val_mae"])

    # Evaluate performance
    loss, mae = best_model.evaluate(X_test, y_test, verbose=1)
    mae_scores.append(mae)
    losses.append(loss)

    # Compute SHAP values
    explainer = shap.Explainer(best_model, X_train)
    shap_values = explainer(X_test)
    shap_values_list.append(shap_values.values)

    # Compute Permutation Importance
    result = permutation_importance(
        best_model, X_test, y_test, scoring="neg_mean_absolute_error", n_repeats=20
    )
    feature_importance_list = np.hstack(
        (feature_importance_list, np.array(result.importances_mean).reshape(-1, 1))
    )


# Compute average MAE
mean_mae = np.mean(mae_scores)
std_mae = np.std(mae_scores)
print(f"\nAverage Test MAE over {num_runs} runs: {mean_mae:.4f} ± {std_mae:.4f}")
print(mae_scores)
print(losses)

all_val_losses = np.array(all_val_losses)  # Shape: (num_runs, epochs)
all_val_maes = np.array(all_val_maes)  # Shape: (num_runs, epochs)

# Compute mean across runs
avg_val_loss = np.mean(all_val_losses, axis=0)
avg_val_mae = np.mean(all_val_maes, axis=0)

# print with , sep
print(*avg_val_loss, sep=",")
print(*avg_val_mae, sep=",")

# Aggregate SHAP values (Mean across runs)
avg_shap_values = np.mean(shap_values_list, axis=0)

# Plot averaged SHAP summary
shap.summary_plot(
    avg_shap_values,
    X_test,
    feature_names=numerical_features + binary_features,
)

# Plot averaged feature importance
fig, ax = plt.subplots()

sorted_idx = np.mean(feature_importance_list, axis=1).argsort()

ax.boxplot(
    feature_importance_list[sorted_idx].T, vert=False, labels=feature_names[sorted_idx]
)
ax.set_title("Permutation Importance of each feature")
ax.set_ylabel("Features")
fig.tight_layout()
plt.show()


y_true = y

y_mean = np.mean(y_true)

mae_baseline = np.mean(np.abs(y_true - y_mean))

print(f"Baseline MAE: {mae_baseline:.4f}")
